#!/usr/bin/env python2
import mylib, residues
import os,sys,string, math
doc="""============== 3J-coupling constant calculations v. 1.2 ========================
based on the workflow described in EKN178-2
by Evgenii Kovrigin, 02/20/05

J=arccos(Ib/Ia)/(2*pi*delta)

Error estimate:
1. d(arccos x)/dx is discontinuous function so usual error propagation is not
   possible.

2. Use "maximum" error estimate as maximum and mimimum deviation from J
   calculated on basis of noise RMSDs

   when Ia-Ib > 2dI use:
       +dJ=arccos( (Ib-dI)/(Ia+dI) )
       -dJ=arccos( (Ib+dI)/(Ia-dI) )
   else
       J=0 +dJ=arccos( (Ib-dI)/(Ia+dI) )

----------

Usage: J_calc.py decoupled_LT_list dec_rmsd_noise coupled_LT_list coupl_rmsd_noise  delta

            where
            ...LT_list -  Sparky 'LT' peak list with ntensities and S/Ns
            dec_rmsd_noise - is rmsd noise for the spectrum ('rm' in Sparky)
            delta      - ms, full J-coupling evolution delay (2*delta3 in ek_jncg and ek_jcocg)


Changes:
05/20/05 - made compatible with separate_J_calc_output.py

==============================================================================

"""


if len(sys.argv)<6 :
    print doc
    sys.exit()


#--- Notation: a - reference/decoupled spectrum, b - attenuated/coupled spectrum ---


decoupled_file=sys.argv[1]
a_rmsd=float(sys.argv[2])
coupled_file=sys.argv[3]
b_rmsd=float(sys.argv[4])

delta=float(sys.argv[5])*1e-3 #-- convert to sec 

print "\n\n============== 3J-coupling constant calculations ========================"
print "\nDecoupled/reference spectrum:\t", decoupled_file, "\tnoise RMSD:\t", a_rmsd
print "\nCoupled/attenuated spectrum:\t", coupled_file, "\tnoise RMSD:\t", b_rmsd
print "\nFull J-coupling evolution delay: %.2f ms" % (delta*1000)
print 


#--- minimal value of SN to consider artifact (if found) to be significant
MIN_SN=2 




#-- load LT v.3.1 lists ---
decoupled_data=residues.Sparky_lt_Peaklist(decoupled_file,nonempty_header_lines=1)
print decoupled_data

coupled_data=residues.Sparky_lt_Peaklist(coupled_file,nonempty_header_lines=1)
print coupled_data

#-- check that the lists match --
residues.check_keys_match(decoupled_data,coupled_data)

print "Processing:\n==========="
#outfile=open("Jcalc_result.txt",'w')
result="Res\tDec_SN\t3J\tdJ\n"

A=1/(math.pi*delta)
result=""

for resname in decoupled_data.sorted_keys():
    result+=resname
    Ib=coupled_data.get(resname,"Height") #--- Data height
    Ia=decoupled_data.get(resname,"Height")  #--- Data height


    #-- If intensities are insignificant -> replace them with noise figure --
    if (abs(Ia) < MIN_SN * a_rmsd) and (abs(Ib) < MIN_SN * b_rmsd):
        #--- no significant peaks. Stop processing this residue
        result+="\tBoth coupled and decoupled within noise.\n"
        continue
        
    if (abs(Ia) < MIN_SN * a_rmsd) : Ia=a_rmsd
    if (abs(Ib) < MIN_SN * b_rmsd) : Ib=b_rmsd

    #-- Filter out significant negatives ---
    if Ia < 0 or Ib < 0:
        result+= "Warning! Significant negative peaks in the list!\n"
        continue #--- stop processing this residue
        
    #-- OK. Peaks are positive
    if abs(Ia-Ib) > MIN_SN*(a_rmsd+b_rmsd)/2:  
        #-- significant difference found ---
        if Ib>Ia:
            #-- Error in data!  Ib cannot be larger than Ia
            result+= "\tWARNING! Coupled intensity is larger than De-coupled! Ib-Ia=%.1f,    %.1f * mean rmsd=%.1f" % (
                 Ib-Ia, MIN_SN, (a_rmsd+b_rmsd)*MIN_SN/2)
            result+= "\tThat may be an artifact of the pulse sequence!!!\n"
            continue   #--- stop processing this residue
            
        else:
            #--- Everything is fine: difference is right sign and significant
            
            #-- calculate J:
            J =  A * math.acos( Ib / Ia )
            max_J =  A * math.acos( (Ib-b_rmsd) / (Ia+a_rmsd) )
            min_J = A * math.acos( (Ib+b_rmsd) / (Ia-a_rmsd) )

            #-- signed uncertainty:
            plus_dJ=max_J-J
            minus_dJ=J-min_J

            result+= "\tJ= %.2f Hz\t+%.2f\t-%.2f\t\t[ %.2f < J < %.2f ]\n" % (J,plus_dJ,minus_dJ, min_J, max_J)
            continue #--- processing finished
        
    else:
        #-- Difference between intensities is insignificant: J=0, calculate +dJ uncertainty
        #-- based on noise rmsd
        plus_dJ =  A * math.acos( (Ib-b_rmsd) / (Ia+a_rmsd) )
        
        result+="\tJ= 0.0  Hz\t+%.2f\t \t\t[ 0.0 < J < %.2f ]\n" % (plus_dJ, plus_dJ)

#outfile.write(result);
print result

#outfile.close()



    
