#!/usr/bin/env python
import os, sys, string
from math import *
import keyfile, xmgr, mylib


import colors
from colors import *
if '-color' in sys.argv: colors.COLOR=ON


doc="""
********************** NOEratio.py *****************************************

 v.2 by Evgenii Kovrigin		    11/15/02

 Calculations of I(noe)/I(no_noe) ration with its uncertainty
 (for the flowchart see EKN10-1c)

	- intensities and noise are normalized by nt= in parts

	- Palmer_sigma/sqrt(2) is calculated for dI/I distributions
	      with dI=I1-I2 (intensity in parts) and 

	- generalized error is reconstructed as Sigma(added_rmsd_noise)+dHH_sigma*height

    - I(noe)/I(no_noe) and its error is calculated assuming error independence
      in NOE and no_NOE datasets
      
 Input:
   - NOEratio.ini file with parameters 
   - six 'lt' files from Sparky with peak heghts

 Usage: NOEratio.py ini_file [-color]   
   
 Program files:
   NOEratio.py -     module to run
   NOEratio_ini.py - constats 
   NOEratio_lib.py - functions



!!!  This script assumes that there is T1-noise present (duplicate error scales with
the peak height! - make sure it is true for your dataset !!!)
**********************************************************************

""" 



#--------------- INPUT SECTION -------------------------
if len(sys.argv)<2:
	print doc
	sys.exit()
inipars=keyfile.Keyword_File(os.getcwd(),sys.argv[1])

temperature=inipars.get_string("<temperature>")   # a part in the file names
protocol_suffix=inipars.get_string("<protocol_suffix>")   # a part in the file names

part1_nt=inipars.get_integer("<part1_nt>")
part2_nt=inipars.get_integer("<part2_nt>")

#-- 'rm' averaged noise rmsd for 'parts' and 'added' spectra
#    normalize it right away 

NOEadded_rmsd=inipars.get_integer("<NOEadded_rmsd>")/(part1_nt+part2_nt) 
noNOEadded_rmsd=inipars.get_integer("<noNOEadded_rmsd>")/(part1_nt+part2_nt)

times_noise_cutoff=inipars.get_integer("<times_noise_cutoff>")  # *rmsd=intensity_cutoff
tolerance=inipars.get_integer("<tolerance>")     # how many times sigma should exceed  the mean dH/H
                # not to have a systematic contribution

#----------------- END OF INPUT -------------------------


from NOEratio_lib import *   # - read function library
from NOEratio_ini import *   # - import common constants

cprint (YELLOW, "*"*60)
cprint (GREEN,"NOEratio.py")
cprint (GREEN,"Calculations of NOE/no-NOE ratio with its uncertainty")
cprint (YELLOW, "*"*60)
# Assume the script is run in the peak list directory!
path_to_peak_lists=os.getcwd()
#******************************************************************
#-------- load six files ----------------
#- old naming convention
noNOEpart1=lt_file(path_to_peak_lists,"no_noe.%s.part1.%s.sp.list" % (temperature,protocol_suffix))
noNOEpart2=lt_file(path_to_peak_lists,"no_noe.%s.part2.%s.sp.list" % (temperature,protocol_suffix))
noNOEadded=lt_file(path_to_peak_lists,"no_noe.%s.added.%s.sp.list" % (temperature,protocol_suffix))
NOEpart1=lt_file(path_to_peak_lists,"noe.%s.part1.%s.sp.list" % (temperature,protocol_suffix))
NOEpart2=lt_file(path_to_peak_lists,"noe.%s.part2.%s.sp.list" % (temperature,protocol_suffix))
NOEadded=lt_file(path_to_peak_lists,"noe.%s.added.%s.sp.list" % (temperature,protocol_suffix))

###--- new naming convention ---
##noNOEpart1=lt_file(path_to_peak_lists,"no_noe.%s.%s.part1.sp.list" % (temperature,protocol_suffix))
##noNOEpart2=lt_file(path_to_peak_lists,"no_noe.%s.%s.part2.sp.list" % (temperature,protocol_suffix))
##noNOEadded=lt_file(path_to_peak_lists,"no_noe.%s.%s.added.sp.list" % (temperature,protocol_suffix))
##NOEpart1=lt_file(path_to_peak_lists,"noe.%s.%s.part1.sp.list" % (temperature,protocol_suffix))
##NOEpart2=lt_file(path_to_peak_lists,"noe.%s.%s.part2.sp.list" % (temperature,protocol_suffix))
##NOEadded=lt_file(path_to_peak_lists,"noe.%s.%s.added.sp.list" % (temperature,protocol_suffix))



#--- CALCULATIONS ------------
#--- normalize by number of scans:
print "Normalize: part1=%s scans, part2= %s, added= %s" % (part1_nt,part2_nt,part1_nt+part2_nt)
for res in noNOEpart1:
	res[1]=res[1]/part1_nt
for res in NOEpart1:
	res[1]=res[1]/part1_nt

for res in noNOEpart2:
	res[1]=res[1]/part2_nt
for res in NOEpart2:
	res[1]=res[1]/part2_nt


for res in noNOEadded:
	res[1]=res[1]/(part2_nt+part1_nt)
for res in NOEadded:
	res[1]=res[1]/(part2_nt+part1_nt)

#--- calculate differences 
noNOEdHbyH=dHbyH(noNOEpart1,noNOEpart2)
NOEdHbyH=dHbyH(NOEpart1,NOEpart2)

#-- statistics --------
print "Height cutoff is set to %d times noise" % times_noise_cutoff
print "\nEstimating sigma for no-NOE data"
noNOEmean,noNOEsigma=dHbyH_statistics(noNOEdHbyH,noNOEadded_rmsd*times_noise_cutoff)
print "\nEstimating sigma for no-NOE data"
NOEmean,NOEsigma=dHbyH_statistics(NOEdHbyH,NOEadded_rmsd*times_noise_cutoff)

#--- generate warning if mean is comparable to sigma
#   what means presence of systematic error between parts 1 and 2
warning="Systematic shift present:"
sys_error_detected=0
if noNOEmean*tolerance>noNOEsigma :
        warning=warning+" noNOE"
        sys_error_detected=1
if NOEmean*tolerance>NOEsigma :
        warning=warning+" NOE"
        sys_error_detected=1
if  sys_error_detected<1: warning=""



#-------- prepare datasets with right sigmas --------
add_sigmas(noNOEadded,noNOEsigma,noNOEadded_rmsd)
add_sigmas(NOEadded,NOEsigma,NOEadded_rmsd)

#-------- calculate ratios of NOE/no_NOE and errors ------------
ratios_list=[]
for i in range(len(noNOEadded)):
    residue=NOEadded[i][RESIDUE]
    h1=NOEadded[i][HEIGHT]
    s1=NOEadded[i][SIGMA]
    h2=noNOEadded[i][HEIGHT]
    s2=noNOEadded[i][SIGMA]
    ratio=h1/h2
    sq_ratio=ratio*ratio
    sq_err1=(s1/h1)*(s1/h1)
    sq_err2=(s2/h2)*(s2/h2)
    sigma=sqrt(sq_ratio*(sq_err1+sq_err2))
    ratios_list.append([residue,ratio,sigma])
mylib.write_lists_out(ratios_list,"noe_ratios.%s.list" % temperature)


#----------------- GRAPHICS -------------------------
# plotting dH/H vs H for both datasets to test absence of systematic error
# and (dH/H)noe vs ...no_noe to check for absence of correlation
#
#  No global variables are changed here!

#------ look at dH/H vs H ----------

noNOEdHbyHplot=xmgr.Dataset(noNOEdHbyH,
                            "no NOE (mean = %.5f; sigma = %.5f)"
                            % (noNOEmean, noNOEsigma))
noNOEdHbyHplot.circle()
noNOEdHbyHplot.filled_sym()

NOEdHbyHplot=xmgr.Dataset(NOEdHbyH,"NOE (mean = %.5f;  sigma = %.5f)"
                            % (NOEmean, NOEsigma))
NOEdHbyHplot.square()
NOEdHbyHplot.empty_sym()

dHbyHgraph=xmgr.Graph("(%s) dH/H vs H for NOE" % temperature, warning)
dHbyHgraph.add_dataset(noNOEdHbyHplot)
dHbyHgraph.add_dataset(NOEdHbyHplot)
dHbyHgraph.set_xaxis("Average height",0,5e5)
dHbyHgraph.set_yaxis("dH/H",-0.3,0.3)
dHbyHgraph.writeout(os.path.join(path_to_peak_lists,(
    "dHHvsH.%s.xmgr" % temperature)))

#--- check for correlations between NOE and no_NOE dH/Hs
dHbyHgraph.remove_dataset(noNOEdHbyHplot)
dHbyHgraph.remove_dataset(NOEdHbyHplot)
new_list=[]
for i in range(len(noNOEdHbyH)):
    new_list.append([noNOEdHbyH[i][1],NOEdHbyH[i][1]])
NOEnoNOEdHHcorr=xmgr.Dataset(new_list,"NOE vs no-NOE")
dHbyHgraph.set_title("(%s) dH/Hnoe vs dH/Hno_noe" % temperature)
dHbyHgraph.add_dataset(NOEnoNOEdHHcorr)
dHbyHgraph.set_xaxis("noNOE dH/H",-0.2,0.2)
dHbyHgraph.set_yaxis("NOE dH/H",-0.2,0.2)
dHbyHgraph.writeout(os.path.join(path_to_peak_lists,(
    "NOEnoNOEdHHcorr.%s.xmgr" % temperature)))


#-- look at NOE data with right sigmas -----------------
NOE_HdH=xmgr.Dataset(residue2number(NOEadded),"NOE")
NOE_HdH.square()
NOE_HdH.empty_sym()
noNOE_HdH=xmgr.Dataset(residue2number(noNOEadded),"no NOE")
noNOE_HdH.circle()
noNOE_HdH.filled_sym()
HdH=xmgr.Graph("(%s) NOE relaxation data" % temperature)
HdH.add_dataset(NOE_HdH)
HdH.add_dataset(noNOE_HdH)
HdH.set_xaxis("Residue number",1,125)
HdH.set_yaxis("Peak height",0,1e6)
HdH.writeout(os.path.join(path_to_peak_lists,("HdH.%s.xmgr" % temperature)))

#--- plot NOE/noNOE ratios and sigmas ---
ratios_data=xmgr.Dataset(residue2number(ratios_list))
ratios_data.circle()
ratios_data.filled_sym()
ratios_graph=xmgr.Graph("(%s) NOE/no-NOE" % temperature)
ratios_graph.add_dataset(ratios_data)
ratios_graph.set_xaxis("Residue number",0,125,5)
ratios_graph.set_yaxis("Hnoe/Hno_noe",0,1)
ratios_graph.writeout(os.path.join(path_to_peak_lists,("ratios.%s.xmgr" % temperature)))


cprint (BLINK,"\nDone...")
