# -----------------------------------------------------------------------------
# 'integrate peak v.2'
# 
#

import sputil
import tkutil, pyutil, re
import tkMessageBox, tkFileDialog
import popen2,string, copy, pickle

import mylib, keyfile, ek_tkutil
import os
from math import *


#-- Default fontsize for my Sparky projects --
default_font=("Helvetica",10)
larger_font=('Helvetica',11)



# -----------------------------------------------------------------------------
#
#--- Global error diagnostics: return codes --
RETURN_ERROR=0
RETURN_OK=1

#-- global definition of dimensions: following sparky's convention
W1=0  
W2=1

TIMEPOINT_SEPARATION=0.05  #-- Timepoints should differ by more than this percentage from one another

#--- This is the parameter in sparky files set to some value corresponding
#     to experiment you used. 

INDEX_PARAMETER='relaxation_time_parameter'
SCALE_FACTOR='scale_factor'
#--- positions of these in spectrum_times()
SPECTRUM_POS=0
INDEX_PARAMETER_POS=1
SCALE_FACTOR_POS=2



#-------- Peak_Parameters ------------------------------------------------------

class Peak_Parameters:
    'parameters of a single peak in a single spectrum'


    def __init__(self, w1, L_w1, w2, L_w2):
        'load a peak'
        #-- frequencies --
        self.w1=w1
        self.w2=w2
        #-- linewidths ---
        self.L_w1=L_w1
        self.L_w2=L_w2
        self.assignment=None
        
    def set_name(self, assignment):
        'set name of the peak'
        self.assignment=assignment 
      

    def __repr__(self):
        return "%s - w1=%.2f (L_w1=%.2f), w2=%.2f (L_w2=%.2f)" % (
               self.assignment,
               self.w1,self.L_w1,
               self.w2,self.L_w2
               )

    def intensity(self,spectrum, scale_factor=1):
        'find peak intensity without baseline correction'
        height=spectrum.data_height([self.w1,self.w2])*scale_factor
        return height

    #--- check for completeness of object contents ---
    def check_completeness(self):
      error_message=""
      if not self.assignment :
        error_message+="Peak name is not defined!\n"

      #-- any errors?
      if error_message:
         tkMessageBox.showerror("2D Peak Fitting",error_message)
         return RETURN_ERROR
      else:
         return RETURN_OK
 
# -----------------------------------------------------------------------------




#---------- Data_Region ----------------------------------------------------
     
class Data_Region:
    """Region of spectrum.
    Offset calculation for the region must be done
    using separate region because the region picked
    to fit may be not suitable."""

    def __init__(self, sparky_region):

        if not sparky_region:
          tkMessageBox.showerror("2D Peak Fitting","Region '%s' is not valid!" % sparky_region)
      

        self.region=sparky_region

        self.min_w1=sparky_region[0][W1]
        self.min_w2=sparky_region[0][W2]

        self.max_w1=sparky_region[1][W1]
        self.max_w2=sparky_region[1][W2]


    def __repr__(self):
        return "w1 = [ %.2f - %.2f ]     w2 = [ %.2f - %.2f ]" % (
                self.min_w1, self.max_w1, self.min_w2, self.max_w2
                )

    def baseline_estimate(self,spectrum, scale_factor=1):
        'estimate baseline offset as a median of four corners of the region'

        offset=scale_factor *(spectrum.data_height([self.min_w1,self.min_w2])+
                             spectrum.data_height([self.max_w1,self.min_w2])+
                             spectrum.data_height([self.min_w1,self.max_w2])+
                             spectrum.data_height([self.max_w1,self.max_w2]))/4

        return offset


    #---- Export just one 2D for plotting purposes ---
    def save_one_spectrum_data(self, fname, spectrum, scale_factor=1 ):
      'save for Gnuplot'

      outp=open(fname,'w')
      outp.write(self.export_data(spectrum,time_parameter="None", scale_factor=scale_factor))
      outp.close()



    #--- Export a series of spectra ----
    def save_time_series(self,fname, spectrum_times):
      'saves a list from spectrum_times (spectrum, time-parameter, scale-factor)'
      outstring=""
      for i in range(len(spectrum_times)):
        
        spectrum=spectrum_times[i][SPECTRUM_POS]
        time_parameter=spectrum_times[i][INDEX_PARAMETER_POS]
        scale_factor=spectrum_times[i][SCALE_FACTOR_POS]

        outstring+=self.export_data(spectrum, time_parameter, scale_factor)

      outstring+="\n\n\n"

      outp=open(fname,'w')
      outp.write(outstring)
      outp.close()
      
        
      
    #--- Export data from a spectrum ----
    def export_data(self,spectrum, time_parameter="None", scale_factor=1):
      'return a text string'

      ppm_per_index = map(lambda w,s: float(w)/s,
                          spectrum.spectrum_width, spectrum.data_size)


      X_steps=(self.max_w1-self.min_w1)/ppm_per_index[W1]
      Y_steps=(self.max_w2-self.min_w2)/ppm_per_index[W2]

      #--- go through the spectrum ---
      string="\n\n# %s \n" % spectrum.name
      for i in range(X_steps):
        
        curr_X=self.min_w1 + ppm_per_index[W1]*i

        string+="\n" #-- separator of 'scans' for gnuplot

        for j in range(Y_steps):

            curr_Y=self.min_w2 + ppm_per_index[W2]*j
                       
            height=spectrum.data_height([curr_X,curr_Y])*scale_factor
                       
            if time_parameter=="None":
                string+="%e\t%e\t%e\n" % (curr_X, curr_Y, height)
            else:
                #--- output a 4D dataset --
                string+="%e\t%e\t%e\t%e\n" % (curr_X, curr_Y, time_parameter, height)
              
      
      return string



      
#--------------------------------------------------------------
    
        
#---------- Peak_Group ----------------------------------------------------

class Peak_Group:
    'group of peaks to be fit together'

    def __init__(self, group_name, fit_region):
        
        self.group_name=group_name #-- composite name - identifier / string
        self.peaks={} #-- assignment : peak_parameter_set
        self.fitting_region=Data_Region(fit_region)
        self.baseline_est_region=None  #-- the region to pick a baseline from


    def set_baseline_region(self, baseline_est_region):
        self.baseline_est_region=Data_Region(baseline_est_region)

    def add_peak(self,peak):
        self.peaks[peak.assignment]=peak

    def remove_peak(self,peak):
        if peak.assignment in self.peaks.keys():
          del self.peaks[peak.assignment]

    

    def __repr__(self):
        return "Group name: '%s'\nFit Region: %s\nBaseline Region: %s\nPeaks: %s" % (
            self.group_name, self.fitting_region, self.baseline_est_region, self.peaks.keys()
            )

    #--- check for completeness of object contents ---
    def check_completeness(self):
      error_message=""
      if not self.group_name :
        error_message+="Peak group name is not defined!\n"
      if not self.peaks:
        error_message+="Peaks are not selected!\n"
      if not self.fitting_region :
        error_message+="Fitting region is not defined!\n"
      if not self.baseline_est_region:
        error_message+="Baseline estimate region is not defined!\n"

      #-- any errors?
      if error_message:
         tkMessageBox.showerror("2D Peak Fitting",error_message)
         return RETURN_ERROR
      else:
         return RETURN_OK
         
#--------------------------------------------------------------
    
        
#---------- Fitting_Project ----------------------------------------------------

class Fitting_Project:
  'run fitting of a group of peaks on a set of spectra'


  def __init__(self, peak_group, spectrum_times, equations, parameters):

    self.peak_group=peak_group  #-- group of peaks to work on

    self.spectrum_list=sparky_spectrum_list  #-- list of sparky spectra to work on

    #-- NOTE ON SPECTRAL INDEXING: I use order of spectra in the list to index variables!
  
    offsets=[]        #-- offset values (in the same order as spectra)

    
  def save_parameter_file_1(self):
    """Save parameters for Gnuplot fit command.
    
    Peak positions and linewidths don't change from spectrum to spectrum.
    
    Heights are taken from the spectrum with the smallest '%s'
    
    Offsets are spectrum-dependent""" % INDEX_PARAMETER

    

    outstring=""

##    #-- generate offsets for a list of spectra
##    for i in range(len(self.spectrum_list)):
##         
##      outstring+= """offset_%d= %f
##                  """ % (i,
##                         self.peak_group.baseline_est_region.baseline_estimate(
##                           self.spectrum_list[i])
##                         )
##          
##    #-- find spectrum with the smallest INDEX_PARAMETER
##    index_values=[]
##    for i in range(len(self.spectrum_list)):
##      value=self.spectrum_list[i].saved_value(INDEX_PARAMETER)
##      index_values.append(value)
##
##    print index_values
##
##          
##      I_max = %f
##    
##    w0_x= %f
##    L_x=  %f
##    
##    w0_y= %f
##    L_y=  %f
##
##    offset= %f




#--------------------------------------------------------------



#--- GNUPLOT GLOBAL PARAMETERS -------
      
GNUPLOT_CALL="/usr/local/bin/gnuplot\n"

#--- gnuplot iterations between reporting the progress: don't set to less than 5! Gets stuck!
max_iterations=5

#---- END OF GNUPLOT GLOBAL PARAMETERS ---
w1=0
w2=1

class integrate_peak_dialog(tkutil.Dialog):

  fname="" #-- file name for gnuplot to fit --
  init_pars=[] #-- tuples of initial parameters for fitting : ((peak_position_w1,...w2),(linewidth_w1,...w2))
  parfname="" #-- The file with gnuplot parameters (before and after fitting)
  gnuplot=None #-- gnuplot I/O pipes

  
  #--- initial numbers for wanted peak maximum ---
  w1_slice=[]
  w2_slice=[]


  
  current_spectrum=None




#$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
  def test_code(self):
    print "Test:"
#    peak=self.session.selected_peaks()[0]
#    a=Peak_Parameters(peak.assignment,peak.position[0],0.5,peak.position[1],0.5)
    b=Data_Region(self.current_region)
    print b
    print b.baseline_estimate(self.session.selected_spectrum())
#    print a.intensity(self.session.selected_spectrum())
#    self.refresh_info(str(a.intensity(self.session.selected_spectrum())))

#$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$














  def __init__(self, session):

    self.session = session
    self.drag_notice = None

    #--- new ---
    self.peak_groups=[]           #--- Analog of a peak list ---
    self.current_peak_group=None  #--- current group ---
    self.current_peak=None

    #-- relax dialog legacy --
    self.spectrum_times = []

    #---- old ---

    self.fit_region=None #--- Region of data to fit --
    self.current_region=None  #--- most recently grabbed region --
    self.region_baseline_offset=None
    
    tkutil.Dialog.__init__(self, session.tk, '2D Peak Fitting')

    self.top.columnconfigure(0, weight = 1)

    import Tkinter
    explain = ('Dragging a box defines the region. Buttons assign parameters.')
    w = Tkinter.Label(self.top, text = explain, justify = 'left', font=default_font)
    w.grid(row = 0, column = 0, sticky = 'nw')



    #--- region line ---
    br = ek_tkutil.button_row(self.top,
                           ('Region to Fit', self.region_to_fit),
                           ('4-point Baseline Estimate', self.region_for_baseline_estimate),
                              #                           ('Test code', self.test_code),
			   )
    br.frame.grid(row = 1, column = 0, sticky = 'nw')



    #--- parameters line ---

    br4 = ek_tkutil.button_row(self.top,
                           ('Take Region for Initial Parameters', self.estimate_parameters),
                           ('Set Peak Label', self.set_label),
                           ('Add Peak to Group', self.add_to_group),
                           ('Undo Last', self.undo_add),
                           #('Save File for Fitting', self.save_parameters),
			   )
    br4.frame.grid(row = 2, column = 0, sticky = 'nw')
    

    #-- main info ---
    r = Tkinter.Label(self.top, justify = 'left', font=larger_font)
    r.grid(row = 3, column = 0, sticky = 'nw')
    self.result_label = r


    #-- service buttons ---
    br3 = ek_tkutil.button_row(self.top,
                           ('Reinitialize', self.reinitialize),
                           ('Save Group', self.save_group),
                           ('Load Group', self.load_group),
                           ('Display Group', self.show_group),
                           ('Close', self.close_cb),
                           ('Help', self.help),
			   )
    br3.frame.grid(row = 4, column = 0, sticky = 'nw')

    #--- gnuplot line ---
    br2 = ek_tkutil.button_row(self.top,
                           ('Select Spectra to Fit', self.setup_spectrum_list),
                           ('Save Fitting Project', self.save_fitting_project),
                           #('test', self.prepare_parameters_v1),
                           #('View 1 peak color maps', self.view_1peak_3D),
                           #('View slices', self.view_slices),
        		   )
    br2.frame.grid(row = 5, column = 0, sticky = 'nw')




    self.register_callback(1)
    self.top.bind('<Map>', lambda e, s=self: s.register_callback(1))
    self.top.bind('<Unmap>', lambda e, s=self: s.register_callback(0))


  # ---------------------------------------------------------------------------
  #
  def register_callback(self, register):

    if register:
      if self.drag_notice == None:
        self.drag_notice = self.session.notify_me('drag region',
                                                  self.region_selected_cb)
    else:
      if self.drag_notice:
        self.session.dont_notify_me(self.drag_notice)
        self.drag_notice = None

  # ---------------------------------------------------------------------------
  #
  def region_selected_cb(self, region):

    if self.is_window_destroyed():
      self.register_callback(0)
      return

    #--ek
    self.current_region=region

    self.refresh_info("Selected region: %s" % Data_Region(region))
    

  #----- REFRESH INFO --------
    
  def refresh_info(self,additional_text=""):
    'Refresh message in the main window'

    self.current_spectrum = self.session.selected_spectrum()

    msg=""
    if self.current_peak_group:
      msg=str(self.current_peak_group)+'\n'

    if self.current_peak:
      msg+=str(self.current_peak)+'\n'

    self.result_label['text'] = msg + additional_text +'\n'
    
    
  #-------- menu:  My help function ----------
  def help(self):
    tkMessageBox.showinfo("2D Peak Fitting",
    """Written by Evgenii Kovrigin 05/09/05
(1) Define data region: drag the box around the peak, make sure peak gets selected.
(2) Hit 'Take as Region to Fit'.
(3) Estimate initial peak position and the linewidth: drag a smaller box. Its width and height will be taken as estimate of the line widths.
    Middle of the box is taken for initial peak position.
(4) Hit 'Take for Initial Parameters' to save.
Repeating (3),(4) adds another peak estimate for overlapped fitting. To remove all hit 'Reinitialize'.



Important to remember:

  - Before using this extension: run 'cv' and click on 'Center on Peak' to allow proper display of selected regions!

  - I assume that your times differ by more than %d% from each other. It is done to allow Gnuplot distinguish
    data coming from different spectra which need separate baseline offset corrections.

""" % TIMEPOINT_SEPARATION)
    

  def reinitialize(self):
    self.current_peak_group=None
    self.current_peak=None
    
    self.result_label['text'] = "Parameters reset. Pick new."


  # --------------menu: region_to_fit ---------------------------------------------------------------
  #
  def region_to_fit(self):
    'define fitting region'    

    #-- check if there was any assignment selected to form a peak name
    if not self.current_region: 
        tkMessageBox.showerror("2D Peak Fitting","\n\n!!! Select the region first with the mouse !!!\n\n")
        return

    
    if not  self.session.selected_peaks(): 
        tkMessageBox.showerror("2D Peak Fitting","\n\n!!! Error!  Switch to selection mode (F1)and have at least one peak in a selected region !!!\n\n")
        return

      
    #-- form a composite name for a group
    peaks=""
    for peak in self.session.selected_peaks():
      peaks+=peak.assignment[:-3]+"_"
    group_name=peaks[:-1]

    self.current_peak_group= Peak_Group(group_name,self.current_region)
    self.refresh_info()

  # --------------menu: region_for_baseline_estimate ---------------------------------------------------------------
  #
  def region_for_baseline_estimate(self):
    'define baseline offset'    

    #-- check if there was any assignment selected to form a peak name
    if not self.current_region: 
        tkMessageBox.showerror("2D Peak Fitting","\n\n!!! Select the region first with the mouse !!!\n\n")
        return
    if not self.current_peak_group: 
        tkMessageBox.showerror("2D Peak Fitting","\n\n!!! Select the region to fit first!!!\n\n")
        return


    self.current_peak_group.set_baseline_region(self.current_region)
    self.refresh_info()


##  # -------------- output_region_to_fit ---------------------------------------------------------------
##  #
##  def output_region_to_fit(self, spectrum, region, fname):
##    'outputs region to fit to  a temporary file'
##
##    ppm_per_index = map(lambda w,s: float(w)/s,
##                        spectrum.spectrum_width, spectrum.data_size)
##
##
##    min_X=region[0][w1]
##    min_Y=region[0][w2]
##
##    max_X=region[1][w1]
##    max_Y=region[1][w2]
##
##    X_steps=(max_X-min_X)/ppm_per_index[w1]
##    Y_steps=(max_Y-min_Y)/ppm_per_index[w2]
##
##    #--- go through the spectrum ---
##    data=[]
##    outp=open(fname,'w')
##    for i in range(X_steps):
##      
##      curr_X=min_X + ppm_per_index[w1]*i
##
##      outp.write("\n") #-- separator of 'scans' for gnuplot
##
##      for j in range(Y_steps):
##
##          curr_Y=min_Y + ppm_per_index[w2]*j
##                     
##          height=spectrum.data_height([curr_X,curr_Y])
##                     
##          data.append([curr_X, curr_Y, height])
##
##          outp.write("%e\t%e\t%e\n" % (curr_X, curr_Y, height))
##          
##    
##    outp.close()
##    self.refresh_info() #--- refreshes the main window information
##
##
##    #--- estimate baseline offset for the region using corners and return it
##
##    baseline_offset=(spectrum.data_height([min_X,min_Y])+
##                     spectrum.data_height([max_X,min_Y])+
##                     spectrum.data_height([min_X,max_Y])+
##                     spectrum.data_height([max_X,max_Y]))/4
##                
##    return baseline_offset
##

  # ------- menu: estimate parameters for fitting --------------------------------------------------------
  #
  
  def estimate_parameters(self):
    'estimate linewidth and peak position from the rectangle'


    if not self.current_peak_group: 
        tkMessageBox.showerror("2D Peak Fitting","\n\n!!! Select the fit region first!!!\n\n")
        return

    est_region=Data_Region(self.current_region)
    


    peak_w1=(est_region.max_w1+est_region.min_w1)/2
    peak_w2=(est_region.max_w2+est_region.min_w2)/2

    lw_w1=est_region.max_w1-est_region.min_w1
    lw_w2=est_region.max_w2-est_region.min_w2

    self.current_peak=Peak_Parameters(peak_w1,lw_w1,peak_w2,lw_w2)

    self.refresh_info() #--- refreshes the main window information 
    

  def test_code(self):
    'test'
    print self.spectrum_times
    self.refresh_info()


  #----- menu: set peak label ----
  def set_label(self):
    'save selected peak name as a label'
    peaklist=self.session.selected_peaks()
    if len(peaklist)>1:
        tkMessageBox.showerror("2D Peak Fitting","\n\n Select just one peak for the label!\n\n")
        return

    name=peaklist[0].assignment

    #--- assign to a last parameter set
    if not self.current_peak: 
        tkMessageBox.showerror("2D Peak Fitting","\n\nTake the region to estimate parameters first!\n\n")
        return

    self.current_peak.set_name(name)
    self.refresh_info()

  #-------- menu: add peak to peak group -----
  def add_to_group(self):
    'add current peak to a current group'
    if not self.current_peak_group: 
        tkMessageBox.showerror("2D Peak Fitting","\n\n!!! Select the fit region first!!!\n\n")
        return
    if not self.current_peak: 
        tkMessageBox.showerror("2D Peak Fitting","\n\n!!! Take peak parameters and label first!!!\n\n")
        return

    if self.current_peak.check_completeness()==RETURN_ERROR:
      return
    self.current_peak_group.add_peak(self.current_peak)
    self.refresh_info()
    

  #-------- menu: undo adding last peak  -----

  def undo_add(self):
    'undo peak addition to a group'
    if not self.current_peak_group: 
        tkMessageBox.showerror("2D Peak Fitting","\n\n!!! Select the fit region first!!!\n\n")
        return
    if not self.current_peak: 
        tkMessageBox.showerror("2D Peak Fitting","\n\n!!! Take peak parameters and label first!!!\n\n")
        return
      
    self.current_peak_group.remove_peak(self.current_peak)
    self.refresh_info()


  #---- menu: saving group information ----
  def save_group(self):
    'save with the group name as a pickle object'

    if  not self.current_peak_group :
        tkMessageBox.showerror("2D Peak Fitting","\n\nGroup of peaks is not defined!\n\n")
        return
    if self.current_peak_group.check_completeness()==RETURN_ERROR:
      return

    fname=self.current_peak_group.group_name+'.group'
    outfile=open(fname,'w')
    outfile.write(pickle.dumps(self.current_peak_group))
    outfile.close
    self.refresh_info("File '%s' is written." % fname)

  #---- menu: loading group information ----
  def load_group(self):
    'load the group from a pickle object'

    infile=tkFileDialog.askopenfile(filetypes=[("Saved Peak Groups","*group")])

    if not infile: return

    string=infile.read()
    self.current_peak_group=pickle.loads(string)
    infile.close()
    self.refresh_info()

  #--- menu: show group ---
  def show_group(self):
    'display group'

    if  not self.current_peak_group :
        tkMessageBox.showerror("2D Peak Fitting","\n\nGroup of peaks is not defined!\n\n")
        return
    if self.current_peak_group.check_completeness()==RETURN_ERROR:
      return

    #--- show group in a Sparky window

    #-- unselect peaks first --
    list=self.current_spectrum.selected_peaks()
    for peak in list:
      peak.selected=0

    #-- select one from the group. Alas, something doesn't work...
      
##    first_peak_name=self.current_peak_group.peaks.keys()[0]
##    one_peak=self.current_spectrum.find_peak(first_peak_name)
##    print one_peak
##    one_peak.selected=1
##    self.session.command_characters("cp")
##    
  
    #-- show Gnuplot figure ---
    temp_fname='temp.txt'
    self.current_peak_group.fitting_region.save_one_spectrum_data(temp_fname,
                                                              self.current_spectrum)

    terminal="""
    set term x11

    """
   
    plotting=("""
      
      set pm3d map
      set view 0, 90
      set yrange [*:*] reverse
      unset ztics
      set ytics 
      splot '%s' using 1:2:3 title "Group: %s      Data Region to fit"
      """ % (temp_fname, self.current_peak_group.group_name) )
              

    gnuplot_command=terminal+ plotting
    
    self.plot(gnuplot_command)
    self.refresh_info() #--- refreshes the main window information 
      


  #------- GENERAL PLOTTING WITH PAUSE UNTIL 'OK' IS PRESSED -----
  #
  def plot(self, gnuplot_command):
    'plotting with Gnuplot, press OK to exit'

    gnuplot_command+="\n"
    finish="\nexit\n"

    print gnuplot_command
    self.gnuplot=popen2.Popen4(GNUPLOT_CALL,0)
    self.gnuplot.tochild.write(gnuplot_command)
    tkMessageBox.showinfo("2D Peak Fitting","Press OK to continue.")
    self.gnuplot.tochild.write(finish)
    gnuplot_output=self.gnuplot.fromchild.readlines() #-- can only be executed after gnuplot exits
    self.gnuplot=None
    print "--- Gnuplot messages ----"
    for line in gnuplot_output:
        print line
    print "--- End of Gnuplot messages ----"
     

















  #----- SAVE FITTING PROJECT ----
  def save_fitting_project(self):
    'save start.par, spectra.dat and equations.txt'

    if not  self.current_peak_group: 
        tkMessageBox.showerror("2D Peak Fitting","\n\nPeak group is not defined!\n\n")
        return

    if  self.current_peak_group.check_completeness()==RETURN_ERROR:
        return

    if not  self.spectrum_times: 
        tkMessageBox.showerror("2D Peak Fitting","\n\nSelect spectra by 'Setup' first!\n\n")
        return


    parameters,equations=self.generate_parameters_v1(self.current_peak_group, self.spectrum_times)

    #--- Save parameters ---
    par_fname=self.current_peak_group.group_name+".fitpar.txt"
    par_file=open(par_fname,'w')
    par_file.write(parameters)
    par_file.close()

    #--- Save equations ---
    eq_fname=self.current_peak_group.group_name+".equations.txt"
    eq_file=open(eq_fname,'w')
    eq_file.write(equations)
    eq_file.close()
    
    #--- Save spectral data ---
    sp_fname=self.current_peak_group.group_name+".spectra.dat"
    if len(self.spectrum_times)==1:
       self.current_peak_group.fitting_region.save_one_spectrum_data(sp_fname,self.spectrum_times[0][SPECTRUM_POS])
    else:
       self.current_peak_group.fitting_region.save_time_series(sp_fname,self.spectrum_times)

    #-- Refresh window --   
    self.refresh_info("Files '%s', '%s' and '%s' written." % (par_fname,eq_fname, sp_fname)) #--- refreshes the main window information 
   
        


  #-- MAIN GENERATOR OF EQUATIONS AND STARTING PARAMETERS ---
    
  def generate_parameters_v1(self, current_peak_group, spectrum_times):
    """Signal intensity evolution with same linewidths and peak positions.

    Plan:
    - Each group has a few peaks labelled as p1,p2,p3, they have corresponding
      frequencies x0_w1,y0_w2 and linewidths Lw1,Lw2 (constant)

    - Peak intensity changes but Imax is measured in the spectrum with minimal
      value of time-parameter (this is the first spectrum, thanks to sorting
      procedure by Tom Goddard).

    - If only one spectrum is found in the list: time evolution part is not inserted
      in equations.
    


    """


    if not  current_peak_group: 
        tkMessageBox.showerror("2D Peak Fitting","\n\nPeak group is not defined!\n\n")
        return

    if  current_peak_group.check_completeness()==RETURN_ERROR:
        return

    if not  spectrum_times: 
        tkMessageBox.showerror("2D Peak Fitting","\n\nSelect spectra by 'Setup' first!\n\n")
        return

    
    parameters=""
    peaks= current_peak_group.peaks.keys()
    first_spectrum= spectrum_times[0][SPECTRUM_POS]
    first_spectrum_scale_factor= spectrum_times[0][SCALE_FACTOR_POS]
    first_spectrum_offset= current_peak_group.baseline_est_region.baseline_estimate(first_spectrum,
                                                                                        first_spectrum_scale_factor)

    #--- Add parameters for each peak ---
    for i in range(len(peaks)):


      data_height= current_peak_group.peaks[peaks[i]].intensity(first_spectrum, first_spectrum_scale_factor)
      
      peak_intensity=(data_height-first_spectrum_offset)


      parameters+=("""
                   w10_p%d = %e
                   w20_p%d = %e
                   Lw1_p%d = %e
                   Lw2_p%d = %e
                   Imax_p%d = %e
                   """ % (i, current_peak_group.peaks[peaks[i]].w1,
                          i, current_peak_group.peaks[peaks[i]].w2,
                          i, current_peak_group.peaks[peaks[i]].L_w1,
                          i, current_peak_group.peaks[peaks[i]].L_w2,
                          i,peak_intensity
                          )
                   )
                


    #-- Add offsets for each spectrum ---

    for i in range(len( spectrum_times)):
      spectrum= spectrum_times[i][SPECTRUM_POS]
      scale_factor= spectrum_times[i][SCALE_FACTOR_POS]
      offset= current_peak_group.baseline_est_region.baseline_estimate(spectrum, scale_factor)

      parameters+=("""
                  offset_s%d = %e
                  """ % (i,offset)
                   )


    if len( spectrum_times)>1:
      #--- Add time-evolution parameter for each peak (R, 3J or sth else)---
      for i in range(len(peaks)):
        
        #-- !!!!! Need to find ways for reasonable estimation of the parameters ---

        #-- case1: relaxation rate ---
        R_init=40 #-- 1/s
        
        parameters+=("""
                    Time_coefficient_p%d = %e
                    """ % (i,R_init)
                     )
      
      


    #------- Generate equations ------------
    time_evolution=""
    if len( spectrum_times)==1:
        function_head="""
                  Y(w1,w2)="""
        
    else:
        function_head="""
                  Y(w1,i,w2,time)="""

        time_evolution=" * exp(-Time_coefficient_p%d*time) " 


    #--- Multiple spectra present ?
    offset_equations=""
    baseline_offset="offset_s0" #-- in case of one spectrum
    
    if len( spectrum_times)>1: 

      N=len( spectrum_times)  
#      for i in range(N-1,-1,-1):
      for i in range(N):
        
        
        if i>1:
          offset_equations+="""
                          Baseline_s%d(i)=(i==%d)? offset_s%d : Baseline_s%d(i)
                          """% (i,i,i,i-1)
        else:
          offset_equations+="""
                          Baseline_s1(i)=(i==1)? offset_s1 : offset_s0
                          """
        
      baseline_offset="Baseline_s%d(i)"%(N-1)  #-- in case of multiple spectra



    equations=""
    function_body=""
    
    for i in range(len(peaks)):

        #--- One-dimensional lorentzians ---      
        equations+="""
        
              W1_p%d(w1) = Lw1_p%d**2 / ( Lw1_p%d**2 + (w10_p%d - w1)**2 )
              W2_p%d(w2) = Lw2_p%d**2 / ( Lw2_p%d**2 + (w20_p%d - w2)**2 )
              """ % (i,i,i,i,i,i,i,i)


        #--- Two-dimensional lorentzian --
        function_body+=""" Imax_p%d * W1_p%d(w1) * W2_p%d(w2)  """ % (i,i,i) 


        #--- Adding time dependence of intensity --        
        if len( spectrum_times)>1:
          function_body+=time_evolution % i + " + "
        else:
          function_body+=" + "


    equations+= offset_equations + "\n\n" + function_head+function_body+baseline_offset + "\n\n"
      

    #--- return results
    
    return parameters, equations

    


    #self.refresh_info("Initial parameters saved to '%s'.\n Ready to fit.\n" % self.parfname) #--- refreshes the main window information 


###=================== 1 PEAK-SPECIFIC FUNCTIONS ==========================
##  #--- fitting functions definitions ---
##  one_lorentzian="""
##  
##      W1(x)=L_x**2/(L_x*L_x + (w0_x-x)**2) 
##
##      W2(y)=L_y**2/(L_y*L_y + (w0_y-y)**2)
##
##      Y(x,y)=I_max*W1(x)*W2(y) + offset
##
##      """ 
##    
##  # ---------------------menu: FITTING 1 PEAK --------------------------------------------------------
##  #
##  def fit_1_peak(self):
##    'fitting of a single 2D peak with gnuplot'
##
##    #-- check if only one peak is to fit! --
##    if len(self.init_pars)!=1:
##      tkMessageBox.showerror("2D Peak Fitting","You defined %d initial parameter sets while \
##asking to fit 1 peak! Use 'Reinitialize' and define ONE set of initial parameters." % len(self.init_pars))
##      return
##
##
##    if self.fit(self.one_lorentzian)==RETURN_ERROR: return
##
##    
##    #---- read in results of fitting ----
##    fitted_pars=keyfile.Ini_File(self.parfname)
##
##    self.init_pars[0][INTENSITY]=float(fitted_pars.get_list("I_max")[1])
##    self.init_pars[0][OFFSET]=float(fitted_pars.get_list("offset")[1])
##    self.init_pars[0][FREQUENCY][w1]=float(fitted_pars.get_list("w0_x")[1])
##    self.init_pars[0][LINEWIDTH][w1]=float(fitted_pars.get_list("L_x")[1])
##    self.init_pars[0][FREQUENCY][w2]=float(fitted_pars.get_list("w0_y")[1]) 
##    self.init_pars[0][LINEWIDTH][w2]=float(fitted_pars.get_list("L_y")[1])
##                  
##
##    self.refresh_info("Gnuplot finished. See Python Shell for details.\n Fit parameters saved in '%s'." % self.parfname) #--- refreshes the main window information 
##
##  # ------------ menu: VIEW 1 PEAK -----------------------------------------------------------------
##  def view_1peak_3D(self):
##    'color map of one peak'
##
##    if len(self.init_pars)!=1:
##      tkMessageBox.showerror("2D Peak Fitting","You defined %d initial parameter sets while \
##asking to fit 1 peak! Use 'Reinitialize' and define ONE set of initial parameters." % len(self.init_pars))
##      return
##
##    set_parameters="""
##    I_max =  %f
##    w0_x  =  %f
##    L_x   =  %f
##    w0_y  =  %f
##    L_y   =  %f
##    offset=  %f
##    """ % (self.init_pars[0][INTENSITY],
##    self.init_pars[0][FREQUENCY][w1],
##    self.init_pars[0][LINEWIDTH][w1],
##    self.init_pars[0][FREQUENCY][w2], 
##    self.init_pars[0][LINEWIDTH][w2], 
##    self.init_pars[0][OFFSET])
##    
##    
##    function=self.one_lorentzian 
##  
##    self.view_3D(set_parameters+function)



####  def view_1peak_slice(self):
####    'color map of one peak'
####
####    if len(self.init_pars)!=1:
####      tkMessageBox.showerror("2D Peak Fitting","You defined %d initial parameter sets while \
####asking to fit 1 peak! Use 'Reinitialize' and define ONE set of initial parameters." % len(self.init_pars))
####      return
####
####    set_parameters="""
####    I_max =  %f
####    w0_x  =  %f
####    L_x   =  %f
####    w0_y  =  %f
####    L_y   =  %f
####    offset=  %f
####    """ % (self.init_pars[0][INTENSITY],
####    self.init_pars[0][FREQUENCY][w1],
####    self.init_pars[0][LINEWIDTH][w1],
####    self.init_pars[0][FREQUENCY][w2], 
####    self.init_pars[0][LINEWIDTH][w2], 
####    self.init_pars[0][OFFSET])
####    
####    
####    function=self.one_lorentzian 
####  
####    self.view_slices(set_parameters+function,)
####
##
##
##
##
##
###@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
###@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
###@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
##
##
##
##  #-------------- menu: VIEW 2D SLICE THROUGH THE PEAK ----------------
##  def  view_slices(self):
##    'generate slices'
##
##    expdata_fnames=[]
##    for i in range(len(self.w1_slice)):
##      expdata_fnames.append( self.generate_slices(self.w1_slice[i], self.w2_slice[i],str(i)) )
##
##    print expdata_fnames
##
##    finish="\nexit\n"
##
##    gnuplot_command="""
##    set term x11
##    set multiplot
##    """
##    
##
##    #----- w1 dimension ----
##    w1_graph="""
##      set size 1,0.5
##      set origin 0,0 
##      set ytics
##      set xrange [*:*] reverse
##      plot """
##
##    #-- add experimental data --
##    w1_set="""'%s' title "%s" with points lt -1 pt %d """
##    for i in range(len(expdata_fnames)):
##        w1_graph+= (w1_set % (expdata_fnames[i][w1],
##                                        self.init_pars[i][LABEL], #-- peak name
##                                        4+i #-- nice symbol
##                                        ) +
##                               ",\\\n")
##    w1_graph=w1_graph[:-3]+"\n" #-- remove last slash and a comma
##
##
##
##
##    #----- w2 dimension ----
##    w2_graph="""
##      set size 1,0.5
##      set origin 0,0.5 
##      set ytics
##      set xrange [*:*] reverse
##      plot """ 
##
##    #-- add experimental data --
##    w2_set="""'%s' title "%s" with points lt -1 pt %d """
##    for i in range(len(expdata_fnames)):
##        w2_graph+= (w1_set % (expdata_fnames[i][w2],
##                                        self.init_pars[i][LABEL], #-- peak name
##                                        4+i #-- nice symbol
##                                        ) +
##                               ",\\\n")
##    w2_graph=w2_graph[:-3]+"\n" #-- remove last slash and a comma
##
##
##
####
####    """
####      set origin 0.48,0.5 
####      unset ytics
####      set colorbox vertical noborder
####      set cbtics 100000
####      splot '%s' using 1:2:(Y($1,$2)) title "Fitting function" """ % self.fname +
####    """
####      set origin 0,0
####      set xtics
####      set ytics
####      splot	'%s' using 1:2:($3-Y($1,$2)) title "Exp - Fit: same scale" """ % self.fname +
####    """
####      set origin 0.48,0
####      set cbrange [*:*]
####      set cbtics 10000
####      unset ytics
####      splot	'%s' using 1:2:($3-Y($1,$2)) title "Exp - Fit: magnified" """ % self.fname +
####    """
####
####    """
####
##    finish="""
##    unset multiplot
##    exit\n
##    """
##
##    
##    gnuplot_command+=w1_graph + w2_graph
##    
##    print gnuplot_command
##    self.gnuplot=popen2.Popen4(GNUPLOT_CALL,0)
##    self.gnuplot.tochild.write(gnuplot_command)
##    tkMessageBox.showinfo("2D Peak Fitting","Press OK to continue.")
##    self.gnuplot.tochild.write(finish)
##    gnuplot_output=self.gnuplot.fromchild.readlines() #-- can only be executed after gnuplot exits
##    self.gnuplot=None
##    print "--- Gnuplot messages ----"
##    for line in gnuplot_output:
##        print line
##    print "--- End of Gnuplot messages ----"
## 
##
###@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
###@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
###@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
##
##  def   generate_slices(self, peak_w1, peak_w2, extension):
##    'display orthogonal slices of experimental data and fitted function through the peak maximum'
##
##
##    #--- find step size ---
##    spectrum=self.current_spectrum
##    ppm_per_index = map(lambda w,s: float(w)/s,
##                        spectrum.spectrum_width, spectrum.data_size)
##
##    #-- ranges --
##    min_w2=self.fit_region[0][w2]
##    max_w2=self.fit_region[1][w2]
##    min_w1=self.fit_region[0][w1]
##    max_w1=self.fit_region[1][w1]
##
##
##    w1_steps=(max_w1-min_w1)/ppm_per_index[w1]
##    w2_steps=(max_w2-min_w2)/ppm_per_index[w2]
##
##
##          
##    #--- along direct dimension first (w2) ---
##    fname_w2=self.fname+".w2_"+extension+".txt"
##    outp=open(fname_w2,'w')
##
##    for i in range(w2_steps):
##      
##      w=min_w2 + ppm_per_index[w2]*i
##
##      intensity=self.current_spectrum.data_height([peak_w1,w])
##      outp.write("%e\t%e\n" % ( w, intensity))
##    outp.close()
##
##
##    #--- along indirect dimension first (w1) ---
##    fname_w1=self.fname+".w1_"+extension+".txt"
##    outp=open(fname_w1,'w')
##
##    for i in range(w1_steps):
##      
##      w=min_w1 + ppm_per_index[w1]*i
##
##      intensity=self.current_spectrum.data_height([w,peak_w2])
##      outp.write("%e\t%e\n" % (w, intensity))
##    outp.close()
##
##    #-- return a file names
##    return fname_w1,fname_w2
##
###=================== END OF 1 PEAK-SPECIFIC FUNCTIONS ==========================
##
##
##
##
##
##  #----------------- MAIN FITTING PROCEDURE ------------------------------------
##  #
##  def fit(self, function):
##    'general fitting procedure'
##
##    #--- check if the file was defined ---
##    if not self.fname:
##      tkMessageBox.showerror("2D Peak Fitting","Region to fit is not defined!")
##      return RETURN_ERROR
##
##    if not os.path.isfile(self.parfname):
##      tkMessageBox.showerror("2D Peak Fitting","Parameters for fitting were not saved!")
##      return  RETURN_ERROR
##      
##
##    fit_command="""
##      FIT_MAXITER=%d
##      set fit errorvariables
##      fit Y(x,y) '%s'  using 1:2:3:(1) via '%s'
##      update '%s'
##      """ % (max_iterations,self.fname,self.parfname,self.parfname)
##
##
##    finish="\nexit\n"
##     
##    gnuplot_command=function + fit_command
##
##    #--- see gnuplot output
##    #self.session.command_characters("py")
##    print gnuplot_command
##
##    iterations=0
##    while 1:
##      iterations+=max_iterations
##      #-- run gnuplot fitting --
##      self.gnuplot=popen2.Popen4(GNUPLOT_CALL,0)
##      self.gnuplot.tochild.write(gnuplot_command + finish)
##      gnuplot_output=self.gnuplot.fromchild.readlines() #-- can only be executed after gnuplot exits
##      self.gnuplot=None
##
##
##      #--- Update iterations count in the window --
##      self.refresh_info("Iterations: %d" % iterations)
##      if mylib.Find_String_Num(gnuplot_output,'converged.')!=-1:
##        break
##      
##    print "--- Gnuplot messages ----"
##    for line in gnuplot_output:
##        print line
##    print "--- End of Gnuplot messages ----"
##
##    return RETURN_OK
##    
##
##   
##
##  #-------- MAIN 3D PLOTTING PROCEDURE ------
##  #
##  def view_3D(self,function):
##    'color maps of results'
##
##    #--- check if the file was defined ---
##    if not self.fname:
##      tkMessageBox.showerror("2D Peak Fitting","Region to fit is not defined!")
##      return
##
##
##    terminal="""
##    set term x11
##
##    """
##   
##    plotting_3D=("""
##      
##      set pm3d map
##      set cbrange [-200000:200000]
##
##      set multiplot
##      set size 0.5,0.5
##      set origin 0,0.5 
##      unset colorbox
##      unset xtics
##      set ytics 
##      splot '%s' title "Experimental data" """ % self.fname +
##    """
##      set origin 0.48,0.5 
##      unset ytics
##      set colorbox vertical noborder
##      set cbtics 100000
##      splot '%s' using 1:2:(Y($1,$2)) title "Fitting function" """ % self.fname +
##    """
##      set origin 0,0
##      set xtics
##      set ytics
##      splot	'%s' using 1:2:($3-Y($1,$2)) title "Exp - Fit: same scale" """ % self.fname +
##    """
##      set origin 0.48,0
##      set cbrange [*:*]
##      set cbtics 10000
##      unset ytics
##      splot	'%s' using 1:2:($3-Y($1,$2)) title "Exp - Fit: magnified" """ % self.fname +
##    """
##      unset multiplot
##  """ )
##    finish="\nexit\n"
##
##    gnuplot_command=terminal+ function + plotting_3D
##    
##    print gnuplot_command
##    self.gnuplot=popen2.Popen4(GNUPLOT_CALL,0)
##    self.gnuplot.tochild.write(gnuplot_command)
##    tkMessageBox.showinfo("2D Peak Fitting","Press OK to continue.")
##    self.gnuplot.tochild.write(finish)
##    gnuplot_output=self.gnuplot.fromchild.readlines() #-- can only be executed after gnuplot exits
##    self.gnuplot=None
##    print "--- Gnuplot messages ----"
##    for line in gnuplot_output:
##        print line
##    print "--- End of Gnuplot messages ----"
##      
##      
##  
##
##  # ------------ EXIT GNUPLOT -----------------------------------------------------------------
##  #
####  def exit_fit(self):
####    'send exit command to gnuplot'
####    if self.gnuplot:
####      self.gnuplot.tochild.write("exit\n")
####      gnuplot_output=self.gnuplot.fromchild.readlines() #-- can only be executed after gnuplot exits
####      self.gnuplot=None
####      print "--- Gnuplot messages ----"
####      for line in gnuplot_output:
####        print line
####      print "--- End of Gnuplot messages ----"
##
##
##
##
##
##
##
##
##
##
##
##
##
##
##   
####
####
####  # -----------------------------------------------------------------------------
####  #
####  def integrate_all_spectra(self, region, X_axis):
####
####    path=os.path.dirname(self.session.project.save_path)
####    #--- make a name of peaks names ---
####    peaks=""
####    peak_names=""
####    for peak in self.session.selected_peaks():
####      peaks+=peak.assignment[:-3]+"."
####      peak_names+=peak.assignment[:-3]+" "
####    peaks+="x_"+X_axis
####    peaks_path=os.path.join(path,peaks)
####    if not os.path.isdir(peaks_path):   os.mkdir(peaks_path)
####    
####    int_graph=xmgr.Graph(peak_names)
####   
####    #--- run integration
####    y_maxs=[]
####    y_mins=[]
####    
####    for spectrum in self.session.project.spectrum_list():
####      integrated, min_X, max_X, min_Y, max_Y = self.integrate_region(spectrum,region, X_axis)
####      y_maxs.append(max_Y)
####      y_mins.append(min_Y)
####      
####      #--- plain output ----
####      fname=spectrum.name+".txt"
####      file=open(os.path.join(peaks_path,fname),'w')
####      for line in integrated:
####        file.write("%f\t%f\n" % (line[0], line[1]))
####      file.close()
####
####      #-------- make XMGR file to look at -------
####      int_dataset=xmgr.Dataset(integrated,spectrum.name)
####      int_dataset.solid_line()
####      int_dataset.no_sym()
####      int_graph.add_dataset(int_dataset)
####      
####    int_graph.set_xaxis(X_axis,min_X, max_X)
####    int_graph.set_yaxis("Data height",min(y_mins), max(y_maxs))
####    int_graph.writeout(os.path.join(peaks_path,peaks)+".xmgr", legend='off')
####
####
#####---------------------------------------------------------------------------------------
####  
####  def integrate_region(self,spectrum, region, X_axis='w1'):
####    '->integrated spectrum along "x" or "y"'
####
####    if spectrum.dimension!=2:
####        tkMessageBox.showwarning("Integrate Region","Cannot work with %d dimensions!" % spectrum.dimension)
####    
####
####    ppm_per_index = map(lambda w,s: float(w)/s,
####                        spectrum.spectrum_width, spectrum.data_size)
#####    print ppm_per_index
####
####    dims={ 'w1':0,  # numerical order of dimensions in arrays
####           'w2':1
####           }
####    #--- assign dimensions
####    primary_dimension=dims[X_axis]
####    secondary_dimension= not primary_dimension
####
#####    print spectrum.data_height([120,8])
####
####    
####    curr_point=[0,0]
####
####    #--- go along main dimension, integrate secondary
####    
####    integrated=[]
####
####    #--- one index ---    
####    min_X=region[0][primary_dimension]
####    min_Y=region[0][secondary_dimension]
####
####    max_X=region[1][primary_dimension]
####    max_Y=region[1][secondary_dimension]
####    
####    X_steps=(max_X-min_X)/ppm_per_index[primary_dimension]
####    Y_steps=(max_Y-min_Y)/ppm_per_index[secondary_dimension]
####
####    # just for xmgr scaling
####    max_y=spectrum.data_height(curr_point)/Y_steps 
####    min_y=max_y
####
####    for i in range(X_steps):
####      
####      curr_X=min_X + ppm_per_index[primary_dimension]*i
####      curr_point[primary_dimension]=curr_X
####      
####      sum=0
####
####      for j in range(Y_steps):
####
####          curr_Y=min_Y + ppm_per_index[secondary_dimension]*j
####          curr_point[secondary_dimension]=curr_Y
####                     
####          height=spectrum.data_height(curr_point)
####                     
####          sum+=height
####
####      Y_point= sum/Y_steps
####      integrated.append([curr_X,Y_point])
####                     
####      if Y_point>max_y: max_y=Y_point
####      if Y_point<min_y: min_y=Y_point
####
####    #--- ranges for plotting ---
####    min_x=min_X
####    max_x=min_x + ppm_per_index[primary_dimension] * (X_steps-1)
####    
####    
####    return integrated, min_x, max_x, min_y, max_y
####
####
####


  #---- menu: Selecting spectra and assigning time parameter/scale factor 

  def setup_spectrum_list(self):
    #-- a resulting spectrum list gets assigned to self.spectrum_times[]

    setup_dialog = sputil.the_dialog(relax_setup_dialog, self.session)
    settings = {'spectrum-times': self.spectrum_times}
    setup_dialog.set_parent_dialog(self, settings, self.set_params)
    setup_dialog.show_window(1)

##---- The stuff below was written by Tom Goddard for his relax.py. Thanks, Tom!
  def set_params(self, settings):
    'actually assign a new spectrum list'
    self.spectrum_times = settings['spectrum-times'] 

#---------------------------------------------------------------------------------------
class relax_setup_dialog(tkutil.Settings_Dialog):

  def __init__(self, session):

    tkutil.Settings_Dialog.__init__(self, session.tk, 'Relaxation Spectra')

    headings = ('Spectrum  ', 'Time parameter', 'Scale factor')
    st = sputil.spectrum_table(session, self.top, headings,
                               self.add_spectrum, self.remove_spectrum)
    st.frame.pack(side = 'top', anchor = 'w')
    self.spectrum_widgets = {}

    spectra = session.project.spectrum_list()
    def time_param(s, self=self):
      return pyutil.string_to_float(self.default_time_parameter(s), 0)
    spectra = pyutil.sort_by_function_value(spectra, time_param)
    for spectrum in spectra:
      st.add_spectrum(spectrum)

   
    br = ek_tkutil.button_row(self.top,
                           ('Ok', self.ok_cb),
                           ('Apply', self.apply_cb),                           
                           ('Close', self.close_cb),
                           #('Help', sputil.help_cb(session, 'RelaxFit')),
			   )
    br.frame.pack(side = 'top', anchor = 'w')

  # ---------------------------------------------------------------------------
  #
  def add_spectrum(self, spectrum, table, row):

    onoff = self.default_onoff(spectrum)

    b = tkutil.checkbutton(table.frame, spectrum.name, onoff)
    b.button.grid(row = row, column = 0, sticky = 'nw')

    ttext = self.default_time_parameter(spectrum)
    e = tkutil.entry_field(table.frame, '', ttext, trailer='s')
    e.frame.grid(row = row, column = 1, sticky = 'nw')

    ftext=spectrum.saved_value(SCALE_FACTOR)
    if ftext==None: ftext=1
    f = tkutil.entry_field(table.frame, 'Intensity *', ftext, width=4)
    f.frame.grid(row = row, column = 2, sticky = 'nw')

    self.spectrum_widgets[spectrum] = (b, e, f)

  # ---------------------------------------------------------------------------
  #
  def remove_spectrum(self, spectrum, table):

    b, e = self.spectrum_widgets[spectrum]
    del self.spectrum_widgets[spectrum]
    b.button.destroy()
    e.frame.destroy()
    
  # ---------------------------------------------------------------------------
  #
  def default_onoff(self, spectrum):

    return spectrum.saved_value(INDEX_PARAMETER) != None
  
  # ---------------------------------------------------------------------------
  #
  def default_time_parameter(self, spectrum):

    #------ get time parameter --- 
    ttext = spectrum.saved_value(INDEX_PARAMETER)
    if ttext:
      return ttext
    
    #--- automatic time determination --
    match = re.search('[0-9]+', spectrum.name)
    if match:
      return match.group(0)
    
    return '0'

  # ---------------------------------------------------------------------------
  #
  def show_settings(self, settings):

    stimes = {}
    for s, t, f in settings['spectrum-times']:
      stimes[s] = (t, f)
      
    for s, (b, te, fe) in self.spectrum_widgets.items():
      b.variable.set(stimes.has_key(s))
      if stimes.has_key(s) :
	t, f = stimes[s]
        if pyutil.string_to_float(te.variable.get()) != t:
          te.variable.set('%.5g' % t)
        if pyutil.string_to_float(fe.variable.get()) != f:
          fe.variable.set('%.5g' % f)
      
  # ---------------------------------------------------------------------------
  #
  def get_settings(self):

    settings = {'spectrum-times': self.spectrum_times() }
    return settings
    
  # ---------------------------------------------------------------------------
  #
  def spectrum_times(self):

    stf = []
    for s, (b, te, fe) in self.spectrum_widgets.items():
      if b.variable.get():
        ttext = te.variable.get()
        t = pyutil.string_to_float(ttext)
        ftext = fe.variable.get()
        f = pyutil.string_to_float(ftext, 1.0)


        stf.append((s, t, f))
        
        s.save_value(INDEX_PARAMETER, ttext)
        s.save_value(SCALE_FACTOR,str(ftext))
    stf.sort(self.spectrum_time_order)
    return tuple(stf)

  # ---------------------------------------------------------------------------

  # ---------------------------------------------------------------------------
  #
  def spectrum_time_order(self, st1, st2):

    return cmp(st1[1], st2[1]) or cmp(st1[0].name, st2[0].name)







def run(session):

  sputil.the_dialog(integrate_peak_dialog,session).show_window(1)
