"""
Class Any_Func provides simple interface to 'curvefit3' classes and methods
by Evgenii Kovrigin    07/27/03


Use it to define new fitting functions and test-run fitting with them: it
exports the fitting results to XMGR

Individual error weighting is implemented but never carefully tested!!!

"""

import curvefit3
reload(curvefit3)
import xmgr
from math import *

#-------------------------------------------------------------------------
class Any_Func:
  """
  Parent class for fitting functions (no error weighting!)
  Usage:
  -- make your function ----
  class My_Function(Any_Function):

    def parameter_count(self): return NUMBER_OF_PARAMETERS

    def value(self, params, x):
      'define your function'
      a, b = params
      return a+b*x

  -- use it ---
    a=5
    b=3

    --- setting parameters---    
    ini_params=[a,b]
    
    func = Linear_Func("linear function")
    

    -- data may be (x,y) and then dy is defaulted to 0
    func.set_data(data)
    
    func.set_same_dy(dy=0.7)
    func.set_init_params(ini_params)

    func.fit_data(error_estimate_trials=25,epsilon=1e-6)
    func.results_to_xmgr("test")

  

  """
  def __init__(self, name=""):
    if name=='': self.name=self.__doc__
    else:        self.name=name
    self.have_errors=0
    self.fit=None
  #------------------------------------------------------------------------
  def parameter_count(self):
    raise UserWarning, "\n\nNot implemented in a subclass!!!"
  #------------------------------------------------------------------------  
  def value(self, params, x):
    raise UserWarning, "\n\nNot implemented in a subclass!!!"

  #------------------------------------------------------------------------
  #--- numerical derivatives: redefine it for speed with analytical forms!
  def parameter_derivatives(self, params, x):

    epsilon=1e-6
    derivatives=[]

    
    #--- check for 0 in parameters: numerical derivative wouldn't work!!!
    if 0 in params:
      raise UserWarning, ("\n\n\n!!!!! Warning form function'%s'\nStarting parameters for fitting cannot be zero because numerical\n\
derivative calculation wouldn't work!\nYou should either \n(1) set non-zero values\
for initial parameter estimates or\n\(2) define them analytically (it also makes calculations a lot faster!).\n" % self.__doc__)
    
    for i in range(len(params)):
      #-- step up ---
      temp_params2=list(params)
      temp_params2[i]+=params[i]*epsilon
      #-- step down ---      
      temp_params1=list(params)
      temp_params1[i]-=params[i]*epsilon

      
      dfda=(self.value(temp_params2,x)-self.value(temp_params1,x))/(2*params[i]*epsilon)
      
      derivatives.append(dfda)

  
    return derivatives

  #------------------------------------------------------------------------
  def set_data(self,data):
    'load data'
    #-- here we set errors to 'fake' if they are not supplied 
    self.data=data

    if len(self.data[0])==3:
      self.have_errors=1

    else:
      self.have_errors=0
      if len(self.data[0])==2:
        #--- set errors column --------
        for i in range(len(self.data)):
            self.data[i].append('fake')
      else:
          raise  UserWarning, "Input data for fitting can have 2 or 3 columns, not %d!"% len(self.data[0])
                                                                                             
    
        
  #------------------------------------------------------------------------------
  def set_same_dy(self,dy):
    'set errors to one value'
    N=len(self.data)
    if dy==0:
      raise UserWarning, "\n\n%s: Error dy cannot be 0!" % self.name
    self.have_errors=1
    for i in range(N):
        self.data[i][2]=dy

  #------------------------------------------------------------------------------
  def set_init_params(self,init_params):
    self.init_params=init_params
      

  #------------------------------------------------------------------------
  def fit_data(self,error_estimate_trials=10, epsilon=1e-6):

      self.fit = curvefit3.least_squares_fit(self.data, self, self.init_params, epsilon)
      if self.have_errors:
        self.fit.estimate_parameter_error(error_estimate_trials)
        

  #------------------------------------------------------------------------
  #-- Just wrap-around curvefit3 features ----
      
  def converged(self): return  self.fit.converged
  
  def fitted_params(self): return self.fit.params
  
  def params_sd(self):
    if self.have_errors: return  self.fit.param_sd
    return None
  
  def steps_taken(self): return self.fit.step_count





  #------------------------------------------------------------------------
  def results_to_xmgr(self, fname, graph_title=""):

      

    #---- ideal data -----------
    init_set=[]
    steps=25

    #--- find maximum and minimum
    x_min=0
    x_max=0
    y_min=0
    y_max=0
    for x,y, dy in self.data:
      if x>x_max: x_max=x
      if y>y_max: y_max=y
      if x<x_min: x_min=x
      if y<y_min: y_min=y
      
    x_step=(x_max-x_min)/steps
    x_current=x_min

    for i in range(steps):
        x_current=x_min +i * x_step
        init_set.append([x_current,func.value(ini_params,x_current)])

        
    inifit_dataset=xmgr.Dataset(init_set,"initial approximation")
    inifit_dataset.solid_line()
    inifit_dataset.red_line()
    inifit_dataset.no_sym()


    #--- calculate smooth function ---   
    fitted_set=[]
    for i in range(steps):
        x_current=x_min +i * x_step
        fitted_set.append([x_current,func.value(self.fit.params,x_current)])
        
    fit_dataset=xmgr.Dataset(fitted_set,"final fit")
    fit_dataset.solid_line()
    fit_dataset.no_sym()






    #--- initial dataset ---
    if not self.have_errors :
      #-- remove fake errors
      for i in range(len(self.data)):
        del self.data[i][2]
    int_dataset=xmgr.Dataset(data,"experimental data")



    title=graph_title
    if not title: title="Fitting  "+self.name
    subtitle=""
    if self.have_errors:
      for i in range(len(self.fit.params)):
        subtitle+="%.2g+-%.2g  " % (self.fitted_params()[i], self.params_sd()[i])
    else:
      for i in range(len(self.fit.params)):
        subtitle+="%.2g  " % (self.fitted_params()[i])
      

          
    int_graph=xmgr.Graph(title,subtitle)
    int_graph.add_dataset(int_dataset)
    int_graph.add_dataset(inifit_dataset)
    int_graph.add_dataset(fit_dataset)
    int_graph.set_xaxis("X",x_min, x_max)
    int_graph.set_yaxis("Y",y_min, y_max)
    int_graph.writeout(fname+".xmgr")

  #------------------------------------------------------------------------




  #------------------------------------------------------------------------

#-------------------------------------------------------------------------
class Linear_Func(Any_Func):
  'a+bx'

  def parameter_count(self):
    return 2
  
  def value(self, params, x):
    a, b = params
    return a+b*x
#---------------------------------------------------

#-------------------------------------------------------------------------
class Square_Func(Any_Func):
  'a+bx+cxx'

  def parameter_count(self):
    return 3
  
  def value(self, params, x):
    a, b, c = params
    return a+b*x+c*x*x
#---------------------------------------------------

#-------------------------------------------------------------------------
# -- this function is being used for relaxation fitting in relax_ek.py
class Exponent_Func2(Any_Func):
  'a*exp(b*x)'

  def parameter_count(self):
    return 2
  
  def value(self, params, x):
    a, b = params
    
    return a *exp(b*x)

  def parameter_derivatives(self, params, x):
    a, b = params
    return (exp(b*x),x*a*exp(b*x))

#-- double exponential function for cases of mixed relaxation mechanisms
class Double_Exponent_Func(Any_Func):
  'a1*exp(b1*x)+a2*exp(b2*x)'

  def parameter_count(self):
    return 4
  
  def value(self, params, x):
    a1, b1, a2, b2 = params
    
    return a1 *exp(b1*x) + a2 *exp(b2*x)

#-------------------------------------------------------------------------


#---- CLEANEX exchange peaks buildup--------------------------------------
class Cleanex_Buildup(Any_Func):
  'cleanex_buildup function'

  def parameter_count(self):
    return 3
  
  def value(self, params, t):
    R1A, k, R1B = params
    #print params, t
    return ( k/(R1A+k-R1B) ) * ( exp(-R1B*t) - exp(-(R1A+k)*t) )

class init_Cleanex_Buildup(Any_Func):
  'cleanex_buildup function'

  def parameter_count(self):
    return 2
  
  def value(self, params, t):
    R1B, k = params
    #print params, t
    return  ( exp(-R1B*t) - exp(-(R1B+k)*t) )


#-------------------------------------------------------------------------



if __name__=="__main__":

    data=open("test.fit").readlines()
    for i in range(len(data)):
        data[i]=map(float,data[i].split())
        


##    #normalize y
##    ymax =  apply(max, tuple(map(lambda xy: xy[1], data)))
##    
##    for i in range(len(data)):
##      data[i][1]/=ymax
##    print data
##      
    #--- fitting parameters ----
##    k=0.01#0.0016#10.0
##    r1a=0.01#0.0074
##    r1b=0.1#0.0074/2
    
#    ini_params=[r1a,k]
#    ini_params=[r1a,k,r1b]
    ini_params=[10000,-10,46000,-1]   
#    func = init_Cleanex_Buildup()
#    func = Cleanex_Buildup()
    func=Double_Exponent_Func()
#    func = Square_Func()
    func.set_data(data)
    #func.set_same_dy(dy=0.1)
    func.set_init_params(ini_params)
    func.fit_data(error_estimate_trials=3,epsilon=1e-20)
    func.results_to_xmgr("test")
    print '='*80
    print func.data
    print "Converged? ", func.converged()
    print "Init Params: ", func.init_params    
    print "Params: ", func.fitted_params()
    print "SD:",  func.params_sd()
    print "Step count: ", func.steps_taken()
    
 

