#!/usr/bin/env python
import sys, os, math
print "="*40, " ek_rccpmg.py ", "="*40
print "\n\nCalculations of VDLIST and VCLIST values for 'ek_rccpmg15N' pulse program"
print "\n------------------\nVersion 3.1, 12/09/07"
print "Evgenii Kovrigin, Medical College of Wisconsin"


doc=""" Here we prepare input data for rcCPMG pulse program. Reference 
experiments will have taucp value < REF_THRESHOLD to relaxation periods are
skipped in the pulse program. The program also checks for bad combinations
of parameters (too low or too high frequencies) that could lead to artifacts.

Definitions:

taucp - full delay between 180-degree pulses

cycles - number of repeats of a single spin-echo  [taucp/2-180-taucp/2]. This number
       must be even for proper suppresion of DD/CSA cross-relaxation. This number is
       output to the vclist file

ncyc = cycles/2 - number of repeats of spin-echo pair [taucp/2-180-taucp-180-taucp/2].
                    Used only internally to guard for integer and even number of
                    repeats of [taucp/2-180-taucp/2] echos.


Tested 12/09/07.

Bugs and fixes:
1/17/08 - points counted from 0 now.
"""


#--- USER INPUT -----

Ttotal=40e-3  #-- total relaxation time seconds

#-- desired frequencies, Hz. Insert 0 for reference expt
#frequency=[ 0, 25, 50, 100, 200, 300, 400, 500, 600, 700, 900, 1000]
frequency=[
	500,
	50,
	250,
	0,
	250,
	50,
	500,
	0,
	0,
	250,
	500,
	50,
	50,
	500,
	250,
	0,
	0,
	500,
	50,
	250,
	0,
	250,
	50,
	500
        ]


#--- Reference experiment setup
REF=1e-4 # s, threshold value used in the pulse program. If taucp is less than
         #   this value - it signifies reference experiment and relaxation delay will be skipped.
REF_FLAG=1e-5 # s, the value of taucp to put in the list as a reference expt flag (must be less than a REF)


#-- X-nuleus 90-pulse length for CPMG, seconds (for checking of the duty cycle)
pX90=47e-6  #-- s

#--- maximum duty cycle in the CPMG pulse train => 180-pulse/taucp
DUTY_CYCLE_MAX=0.10


#-- file names to output results ready for insertion in TopSpin lists
freq_out_fname='freq_ek_rccpmg.txt'
vdlist_out_fname='vdlist_ek_rccpmg.txt'
vclist_out_fname='vclist_ek_rccpmg.txt'
ttotal_out_fname='ttotal_ek_rccpmg.txt'

#---------- END OF USER INPUT ----------------------







#-- NOTE: all calculations are done in seconds, only output values are adjusted to ms


#-- interpulse delay
taucp=[]
#-- number of repeats of  a single echo 
cycles=[]

for freq in frequency:

    

    #-- REFERENCE EXPERIMENT (NO RELAXATION DELAY)

   #-- set up reference experiment, freq=0
   if freq<0.1:
     taucp.append(REF_FLAG) #-- append flag value
     cycles.append(1)       #-- formally assign a number, no meaning
     continue

    

    #--- CALCULATE TAUCP VALUE ----
    
   #-- calculate taucp (interval between 180 pulses)  
   tcp=1/(2.0*freq)
   
   #-- check duty cycle
   pX180=2*pX90 #-- 180 pulse, sec
   min_taucp=pX180/DUTY_CYCLE_MAX
   if tcp< min_taucp:
      print "\n--------------- ERROR! -------------------"
      print "Duty cycle exceeds limiting value of %.0f%% !" % (100*DUTY_CYCLE_MAX)
      print "Frequency=%.1f Hz, taucp = %.3f ms, 180-time = %.3f ms" % (freq,tcp*1000,pX180*1000)
      #-- Suggest frequency for maximum allowed duty cycle
      maxfreq=1/(2*min_taucp)
      print "Your maximum frequency is: %.1f Hz" % maxfreq
      sys.exit()
      
   #-- taucp is good, save it   
   taucp.append(tcp)




   #----- CALCULATE LOOP COUNTER ----
   
   #-- calculate number of repeats of the spin-echo pair (combined for both periods)
   ncyc=Ttotal/(4.0*tcp)
   
   #-- check if less than 1 pair to execute
   if ncyc<1:
      print "\n-------------WARNING!-------------"
      print "Frequency %.1f Hz cannot be executed with Ttotal = %.3f ms" % (freq, Ttotal*1000)
      print "Suggestions:"
      newTtotal=4*tcp
      print "(1) use VCLIST value=2 and take into account new Ttotal for this point = %.3f ms" % (newTtotal*1000)
      minfreq=4/(2.0*Ttotal)
      print "(2) use higher frequency: %.1f Hz" % minfreq
      print "By default I use case (1). Make a note of your new Ttotal!"
      ncyc=1

   #-- check if number of repeats is non-integer   
   if ncyc-math.floor(ncyc)>0:
     print "\n------------- WARNING! ------------------ "
     print "Frequency of %.1f Hz causes non-integer number of repeats of the spin-echo pair %.3f!" % (freq, ncyc)
     ncyc=int(ncyc)
     newTtotal=tcp*4.0*ncyc
     print "I will round it  to %d. Please, note new Ttotal %.3f ms." % (ncyc,newTtotal*1000)
     
   cycles.append(ncyc*2) #-- add number of repeats of a SINGLE spin echo (it is even by default)
   


#--Output

print"\n-------------------- Output to the files: ------------------"
print"Frequency,Hz => '%s'\ntaucp,ms => '%s'\nNumber of cycles,n => '%s'\n\
Total relaxation time, ms => '%s'\n" %  ( freq_out_fname,vdlist_out_fname,vclist_out_fname,ttotal_out_fname) 
freq_file=open(freq_out_fname,'w')
vdlist_file=open(vdlist_out_fname,'w')
vclist_file=open(vclist_out_fname,'w')
ttotal_file=open(ttotal_out_fname,'w')
print "Frequency, Hz\ttaucp, ms\tcycles\tTotal relaxation time, ms"
print "-"*60
for count in range(len(taucp)):
   freq=frequency[count]
   tcp=taucp[count]
   vclist_value=cycles[count]
   T=tcp*2.0*vclist_value   #-- here I already use number of repeats for single echo
   if tcp<REF:
     comment= "  -- reference expt"
     vclist_value=1 #- formal assignment (not used)
   else:
     comment=""
   print "%d\t%.1f\t%.4f\t\t%d\t\t%.3f%s" % (count,freq, tcp*1000, vclist_value, T*1000,comment) 

     
   freq_file.write('%.6f\n'%freq)
   vdlist_file.write('%.6f\n'%tcp)
   vclist_file.write('%d\n'%vclist_value)
   ttotal_file.write('%.6f\n'%T)
   
   

freq_file.close()
vdlist_file.close()
vclist_file.close()
ttotal_file.close()



 
