"""

Chemical equilibria




"""


class Equilibrium:
    """chemical equilibrium object

    --- basic usage ---
    - define three internally used functions here
    
    class Specific_System(Equilibrium):

        def _calculate_velocities(self):
            'definition of current velocities'
            #--- k1*A*B 
            self.v[0]=self.k[0]*self.C[0]*self.C[1]
            #--- k2*AB
            self.v[1]=self.k[1]*self.C[2]

        #---------------------------------------------------------------
        def _calculate_dC(self):
            'definition of dC[i]/dt'
            self.dC[0]=-self.v[0]+self.v[1]
            self.dC[1]=-self.v[0]+self.v[1]
            self.dC[2]=self.v[0]-self.v[1]

        #---------------------------------------------------------------
        def _is_equilibrium(self, epsilon):
            'v(forward)-v(backward) < epsilon'
            if abs(self.v[0]-self.v[1]) < epsilon:
                return 1
            else:
                return 0
            
        #--------------------------------------------------------------


    - load constants
    a=Specific_Equilibrium(k1,k2,k3)
    
    - load initial concentrations
    a.set_concentrations(c1,c2)

    - find equilibrium
    a.equilibriate( max_num_steps, epsilon)

    - generate titration curve
    
    
    


    --- members ---
    self.k[i] - constants of forward and backward reactions    
    self.C[i] - current concentrations of the components
   
    self.dC[i] - last equilibration step concentration change
    self.v[i] - velocities at the current point


    """
    #--------------------------------------------------------------    
    def __init__(self, constants):
        'load rate constants: k1,k2,...,kn'
        if self.number_of_constants!=len(constants):
            raise UserWarning, (
                "\n\nNeed %d constants, got %d instead!\n\n" %
                (self.number_of_constants, len(constants)))
                
        self.k=constants
        #--- set all to different numbers (== out of equilibrium)                
        self.v=range(len(self.k))

    #--------------------------------------------------------------
    def set_concentrations(self, *concentrations):
        'load starting conditions: Ca1, Ca2, ... ,Cam - ! all species !'
        if self.number_of_species!=len(concentrations):
            raise UserWarning, (
                "\n\nNeed %d concentrations, got %d instead!\n\n" %
                (self.number_of_species, len(concentrations)))

        self.C=list(concentrations)
        #--- set all to 1 (== out of equilibrium)
        self.dC=[1]*self.number_of_species
        
        

    #--------------------------------------------------------------
    def _calculate_velocities(self):
        'velocities are specific to each subclass'
        raise UserWaring, "Not implemented for this subclass!"
        
    #---------------------------------------------------------------
    def _calculate_dC(self):
        'dC[i]/dt - specific to each subclass'
        raise UserWaring, "Not implemented for this subclass!"        
    
    #---------------------------------------------------------------
    def _is_equilibrium(self, epsilon):
        'v(forward)-v(backward) < epsilon'
        raise UserWaring, "Not implemented for this subclass!"
    
    #---------------------------------------------------------------
    def equilibrate(self, max_num_steps, epsilon):
        """iterate untill max_num_steps or difference in forward and
           reverse rates is less than epsilon
           -> 0 - not equilibrated, 1- equilibrated
        """
        step=1
        small_step=0.001 #- the scaling factor for making a step
        
        while not self._is_equilibrium(epsilon):

            #--- too long ? --
            if step>max_num_steps:
                return 0
            
            self._calculate_velocities()
            self._calculate_dC()
            #-- step up ---
            for i in range(self.number_of_species):
                self.C[i]+=self.dC[i]*small_step


            #--- follow it --
            if not step%100 :
                print "Step %d:" % step,
                for C in self.C:
                    print C, 
                print

            step+=1
            
        #--- converged !
        return 1

##    #----------------------------------------------------------------
##    def make_titration_curve(self,
##                             in_syringe_component,  # index 
##                             in_cell_component,     # index
##                             max_injectant_to_compincell_ratio # L/P to reach
##                             number_of_injections,  # number
##                             max_num_steps=1e6,     # steps in search for equilibrium 
##                             epsilon=1e-6           # limit for difference in rates at equilibrium
##                             ):
##        for number_of 
##    
#-------------------------------------------------------------------

class Single_Step(Equilibrium):

    def __init__(self, *constants):
        
        #-- these are constant for the current system
        self.number_of_species=3
        self.number_of_constants=2
        Equilibrium.__init__(self,constants)
        

    def _calculate_velocities(self):
        'definition of current velocities'
        #--- k1*A*B 
        self.v[0]=self.k[0]*self.C[0]*self.C[1]
        #--- k2*AB
        self.v[1]=self.k[1]*self.C[2]

    #---------------------------------------------------------------
    def _calculate_dC(self):
        'definition of dC[i]/dt'
        self.dC[0]=-self.v[0]+self.v[1]
        self.dC[1]=-self.v[0]+self.v[1]
        self.dC[2]=self.v[0]-self.v[1]

    #---------------------------------------------------------------
    def _is_equilibrium(self, epsilon):
        'v(forward)-v(backward) < epsilon'
        if abs(self.v[0]-self.v[1]) < epsilon:
            return 1
        else:
            return 0
        
    #--------------------------------------------------------------




#####################################################################
#####################################################################
#####################################################################
if __name__=="__main__":

    a=Single_Step(2,2)
    a.set_concentrations(1,1,0)
    print a.equilibrate(1e6,1e-6)
    
    
