"""

Chemical equilibria
v.2



"""
import copy
import xmgr, mylib

DEBUG="on"
#-- Equilibration modes
UNLIMITED="unlimited equilibration time"
FIXED="fixed equilibration time"

class Equilibrium:
    """chemical equilibrium object

    - equilibration_mode=UNLIMITED  -  equilibration means difference between
                                      forward and backward rates less than
                                      epsilon (as programmed in is_equilibrium())
    - equilibration_mode=FIXED - means that we keep equlibration time fixed
                                    with no regard to completness of equilibration
                                    (this is a way to simulate non-equilibrium
                                    conditions on ITC)
                                    
    'equilibrate()'
    - if 'max_num_iterations' is reached without equilibration - exception is raised
                                to draw attention to reasons of long convergence
                                (bad starting conditions or periodicity in calculated
                                concentration increments)
                                
    - if equilibration_mode=FIXED - than 'max_time' sets the maximum time interval (a sum
                                of 'time steps' available for equilibration. The method exits
                                regardless of whether equilibrium is reached or not

   

    """
    #--------------------------------------------------------------    
    def __init__(self, name, kinetics_mode=1, equilibration_mode=UNLIMITED):
        'load rate constants: k1,k2,...,kn'
        self.name=name                      #-- reaction name
        self.kinetics_mode=kinetics_mode    #-- collect equilibration course
        self.equilibration_mode=equilibration_mode #-- which way equilibration is watched

        self.kinetics_data=[]   #-- kinetics of one equilibration
        
        self.titration_curve=[] #-- titration curve with equilibrium values
        self.full_titration_curve=[] #-- titration curve with kinetics_data objects        
                
        #-- prepare constants and velocities indexed as 1,2,3...
        self.k={}
        self.v={}
        for i in range(self.number_of_constants):
            self.k[i+1]="not set"
            self.v[i+1]="not set"

        #--- prepare place for concentrations indexed as names
        self.C={}
        self.dC={}
        for name in self.species:
            self.C[name]="not set"
            self.dC[name]="not set"

        self.titration_mode="none"
        
    #--------------------------------------------------------------
    def __repr__(self):
        'printout constants'
        text="System: "+self.name
        text+="\n---------------------\nSpecies:\n"
        for name in self.species:
            text+=name+"\t"
            
        text+="\n----------------------\nConstants:\n"
        for i in range(self.number_of_constants):
            text+="k[%d]=%e\n" % (i+1, self.k[i+1])
            
        text+="\n---------------------\nEnthalpies:\n"
        for name in self.enthalpies:
            text+="dH(%s)=%e\n" % (name, self.enthalpies[name])

        text+="\n---------------------\nCurrent titration mode:\n"
        text+=self.titration_mode

        return text
        
        
    #--------------------------------------------------------------
    def _calculate_changes(self):
        'v, dC, ddH: specific to each subclass'
        raise UserWaring, "Not implemented for this subclass!"
    #---------------------------------------------------------------
    def _is_equilibrium(self, epsilon, max_time):
        'v(forward)-v(backward) < epsilon or max_time is reached depending on equilibration_mode'
        raise UserWaring, "Not implemented for this subclass!"


    #---------------------------------------------------------------
    def _record_kinetics(self,time_interval):
        'keep record of kinetics state'
        self.kinetics_data.append(
            { "time_interval":   time_interval,
              "C"   :   copy.deepcopy(self.C),
              "dC"  :   copy.deepcopy(self.dC),
              "v"   :   copy.deepcopy(self.v),
              "ddH" :   self.ddH,   #-- heat of a current step
              "dH":  self.dH     #-- integrated heat
            })

    #----------------------------------------------------------------

    def equilibrate(self, max_num_iterations,  #-- maximum number of equilibration steps
                                    epsilon,   #-- equilibration criterion if 'equilibration_mode' is UNLIMITED
                                                #  a fraction of v(forward) - difference forward
                                                #  and backward should be less
                                    max_time="not set" #- equilibration time if 'equilibration_mode' is FIXED
                    ):
        """iterate until max_num_steps or difference in forward and
           reverse rates is less than epsilon
           -> 0 - not equilibrated, 1- equilibrated
        """
        #-- check if constants were set
        if "not set" in self.k.values():
            raise UserWarning, (
            "\n\nNot all rate constants were set!\n%s\n\n" % self.k)
 
        #-- check if concentrations were set
        if "not set" in self.C.values():
            raise UserWarning, (
            "\n\nNot all concentrations of species were set!\n%s\n\n" % self.C)

        #-- check if enthalpies were set
        if "not set" in self.enthalpies.values():
            raise UserWarning, (
            "\n\nNot all enthalpies were set!\n%s\n\n" % self.enthalpies)

        if self.equilibration_mode==FIXED:
            if max_time=="not set":
                raise UserWarning, "\n\n'Equilibration_mode' is FIXED but 'max_time' - time available for equilibration is not set!"

        #-- simulate fixed interval for equilibration as in ITC
        self.total_time=0
        
        self.dH=0 #-- begin new heat integration

        if self.kinetics_mode: self.kinetics_data=[]   #-- allocate new storage

        iteration=1
        #- the scaling factor for making a step
        reduction_factor=1.11111e-3 #-- make it irrational and prevent glitches

        while not self._is_equilibrium(epsilon,max_time):
            
            #--- too long ? --
            if iteration>max_num_iterations:
                return 0
            
 
            #-- find reduction_factor size
            found=0
            while not found:
                for name in self.species:
                    if  (self.C[name] + self.dC[name]*reduction_factor) < 0:
                        reduction_factor/=2.1 #-- make step irrational
                        if DEBUG=="on": print "Step reduction to", reduction_factor
                        break
                else: #-- all was OK !
                    found=1

            
            #-- step up ---
            for name in self.species:
                self.dC[name]*=reduction_factor
                self.C[name]+=self.dC[name]

            self.ddH*=reduction_factor                
            self.dH+=self.ddH

            if self.kinetics_mode: self._record_kinetics(reduction_factor)
            
            #--- follow it --
            if DEBUG=="on":
                if not iteration%100000 :
                    print "\nIteration %d:" % iteration,
                    for name in self.species:
                        print "%s:%s " % (name, self.C[name]),
                    print
                    print 'v', self.v
                    print 'dC', self.dC
                    print 'step', reduction_factor

            iteration+=1
            self.total_time+=reduction_factor*1 #-- a fraction of a unit time!

            
        #--- converged !
        return 1

    #----------------------------------------------------------------
    def make_titration_curve(self,
                             in_syringe_component,  # name 
                             in_cell_component,     # name
                             max_injectant_to_compincell_ratio, # ratio to reach
                             number_of_injections,  # number
                             max_num_iterations=1e6,     # steps in search for equilibrium 
                             epsilon=1e-6,           # limit for difference in rates at equilibrium
                             max_time="not set" #- equilibration time if 'equilibration_mode' is FIXED
                             ):

        self.titration_mode=in_syringe_component + "->" + in_cell_component
        #-- calculate injection amount
        injected_amount=self.C[in_cell_component]* max_injectant_to_compincell_ratio/ number_of_injections

        #-- total concentrations in the cell
        total_component_in_cell=self.C[in_cell_component]
        total_injectant_added=injected_amount   #- just for calculation of the ratio
        self.C[in_syringe_component]=injected_amount

        self.titration_curve=[]  #-- record equilibrium values
        self.full_titration_curve=[]  #-- record full kinetics 
        
        for i in range(number_of_injections):
            if not self.equilibrate(max_num_iterations, epsilon,max_time):
                raise UserWarning, (
                "\n\nEquilibrium wasn't reached at %s iterations and %s epsilon!\n\n"
                % (max_num_iterations,epsilon))

            ratio=total_injectant_added/total_component_in_cell

            #-- store equilibrium results
            self.titration_curve.append({
               "ratio"     :   ratio,
                "C"         :   copy.deepcopy(self.C),
                "Enthalpy"  :   self.dH/injected_amount
                })

            #-- store kinetics data
            if self.kinetics_mode: self.full_titration_curve.append({
                                                            "injected_amount": injected_amount,
                                                            "kinetics":self.kinetics_data
                                                            })
            
            if DEBUG=="on":
                print "Ratio: %s, dH: %s" %(ratio, self.dH/injected_amount) #-- specific heat

            #--- next step ---
            self.C[in_syringe_component]+=injected_amount
            total_injectant_added+=injected_amount

        mylib.beep()

    #------------------------------------------------------------------------------------
    def equilibration_to_xmgr(self, file_name, graph_name="Single_dynamics",
                         max_number_of_points=20):
        'visualize one equilibration event'
        if not self.kinetics_mode:
            raise UserWarning, "\n\nKinetics data were not collected! Nothing to export."
        
        #-- X variable, time (steps)
        X=0
        
        #-- concentrations of species storage
        species_C={}
        species_dC={}
        v={}        
        for name in self.species:
            species_C[name]=[]
            species_dC[name]=[]

        for i in range(self.number_of_constants):
            v[i+1]=[]

        #-- dynamics of a heat evolution storage
        heat=[]
        integrated_heat=[]

        N=len(self.kinetics_data)
        
        if max_number_of_points<N: skip_points=N/max_number_of_points
        else: skip_points=1

        for i in range(N):
            
            iteration=self.kinetics_data[i]            
            X+=iteration["time_interval"]            

            if i%skip_points:
                if i!=0:
                    continue
            
            for name in self.species:
                species_C[name].append([X,iteration["C"][name]])
                species_dC[name].append([X,iteration["dC"][name]])
                
            for i in range(self.number_of_constants):
                v[i+1].append([X,iteration["v"][i+1]])
                
            heat.append([X,iteration["ddH"]])
            integrated_heat.append([X,iteration["dH"]])


        #---- XMGR ----
        C=species_C
        dC=species_dC

        single_dynamics=xmgr.Graph(graph_name,a.name)

        color=1
        for name in self.species:

            C_data=xmgr.Dataset(C[name],name+".C")
            C_data.no_sym()
            C_data.solid_line()
            C_data.linecolor=color
            single_dynamics.add_dataset(C_data)


            dC_data=xmgr.Dataset(dC[name],name+".dC")
            dC_data.no_sym()
            dC_data.dotted_line()
            dC_data.linecolor=color
            single_dynamics.add_dataset(dC_data)

            color+=1

        H_data=xmgr.Dataset(heat,"heat")
        H_data.symbol_size(0.5)
        H_data.solid_line()
        single_dynamics.add_dataset(H_data)
            
        Htot_data=xmgr.Dataset(integrated_heat,"total heat")
        Htot_data.symbol_size(0.5)
        Htot_data.solid_line()
        Htot_data.red_sym()    
        single_dynamics.add_dataset(Htot_data)

        
        single_dynamics.writeout(file_name)

        #--- constants ---
        open(file_name+".txt",'w').write(str(self))


    
    #---------------------------------------------------------------
    def titration_to_xmgr(self, fname, graph_name,
                          max_number_of_points=10 #-- on kinetics curves for injections
                          ):
        'writes the files: fname.eq.xmgr; fname.kin.xmgr and fname.its.xmgr if kinetics_mode==1'


        #--- constants ---
        open(fname+".txt",'w').write(str(self))
        

        ##### EQUILIBRIUM DATA #############
        
        #-- storage equilibrium values ---
        equilibrium_titration_results={}
        for name in self.species:
            equilibrium_titration_results[name]=[]
            
        equilibrium_titration_results["Enthalpy"]=[]

        #--- load data ----
        N=len(self.titration_curve)
        
        for i in range(N):
            
            injection=self.titration_curve[i]            
            X=injection["ratio"]            

            for name in self.species:
                equilibrium_titration_results[name].append([X,injection["C"][name]])
                
            equilibrium_titration_results["Enthalpy"].append([X,injection["Enthalpy"]])


        equilibrium_graph=xmgr.Graph(graph_name,"Equilibrium "+self.titration_mode+" "+self.name)

        color=1
        for name in self.species:

            C_data=xmgr.Dataset(equilibrium_titration_results[name],"["+name+"]")
            C_data.no_sym()
            C_data.solid_line()
            C_data.linecolor=color
            equilibrium_graph.add_dataset(C_data)

            color+=1

        H_data=xmgr.Dataset(equilibrium_titration_results["Enthalpy"],"Enthalpy")
        H_data.symbol_size(0.5)
        H_data.solid_line()
        equilibrium_graph.add_dataset(H_data)

        equilibrium_graph.set_xaxis("Y",0,1)
        equilibrium_graph.set_xaxis("ratio",0,1)
        equilibrium_graph.writeout(fname+".eq.xmgr")





        ############ KINETICS DATA #################

        if not self.kinetics_mode: return
        
        #-- X variable, time (steps)
        X=0
        
        #-- prepare the storage
        species_C={}
        for name in self.species:
            species_C[name]=[]
            
        v={}
        for i in range(self.number_of_constants):
            v[i+1]=[]


        heat=[]
        integrated_heat=[]

        #-- load all the data ---    
        for kinetics_data in self.full_titration_curve:

            N=len(kinetics_data["kinetics"])
            
            if max_number_of_points<N: skip_points=N/max_number_of_points
            else: skip_points=1

            for i in range(N):
                
                iteration=kinetics_data["kinetics"][i]

                #-- increment time 
                X+=iteration["time_interval"]            

                if i%skip_points:
                    if i!=0:
                        continue
                
                for name in self.species:
                    species_C[name].append([X,iteration["C"][name]])

                for i in range(self.number_of_constants):
                    v[i+1].append([X,iteration["v"][i+1]])


                #-- calculate specific heats
                injected_amount=kinetics_data["injected_amount"]
                heat.append([X,iteration["ddH"]/injected_amount])
                integrated_heat.append([X,iteration["dH"]/injected_amount])


        #---- XMGR ----
        C=species_C

        kinetics_graph=xmgr.Graph(graph_name,"Kinetics "+self.titration_mode+" "+self.name)

        color=1
        for name in self.species:

            C_data=xmgr.Dataset(C[name],"["+name+"]")
            C_data.no_sym()
            C_data.solid_line()
            C_data.linecolor=color
            kinetics_graph.add_dataset(C_data)

            color+=1
            
        color=1
        for i in range(self.number_of_constants):

            v_data=xmgr.Dataset(v[i+1],"v%s"%(i+1))
            v_data.no_sym()
            v_data.dotted_line()
            v_data.linecolor=color
            kinetics_graph.add_dataset(v_data)

            color+=1

            

        H_data=xmgr.Dataset(heat,"heat")
        H_data.symbol_size(0.5)
        H_data.solid_line()
        H_data.blue_sym()
        H_data.blue_line()
        kinetics_graph.add_dataset(H_data)
            
        Htot_data=xmgr.Dataset(integrated_heat,"total heat")
        Htot_data.symbol_size(0.5)
        Htot_data.solid_line()
        Htot_data.red_line()
        Htot_data.red_sym()    
        kinetics_graph.add_dataset(Htot_data)

        kinetics_graph.set_xaxis("Y",0,1)
        kinetics_graph.set_xaxis("time",0,1)
        
        kinetics_graph.writeout(fname+".kin.xmgr")












#-------------------------------------------------------------------




class AB(Equilibrium):
    """A+B=AB single step equilibrium"""

    def __init__(self, name="A+B=AB", kinetics_mode=1,equilibration_mode=UNLIMITED):
        
        #-- these are constant for the current system
        self.species=["A","B","AB"]
        self.number_of_species=len(self.species)
        self.number_of_constants=2
        self.enthalpies={"AB": "not set"}   # molar heat of formation
        

        
        Equilibrium.__init__(self,name,kinetics_mode,equilibration_mode)
        

    def _calculate_changes(self):
        #-- aliasing for easier use
        v=self.v
        C=self.C
        dC=self.dC
        k=self.k

        #'definition of current velocities'
        
        v[1]= k[1] * C["A"] * C["B"]
        v[2]= k[2] * C["AB"]

        #---'definition of dC[i]/dt'
        
        dC["A"]=  -v[1] + v[2]
        dC["B"]=  -v[1] + v[2]
        dC["AB"]=  v[1] - v[2]

        #-- heat (no aliasing!!!)
        self.ddH= dC["AB"] * self.enthalpies["AB"]
        
    #---------------------------------------------------------------
    def _is_equilibrium(self, epsilon, max_time):
        '|v(forward)-v(backward)| < |epsilon*v(forward)|) or if max_time is reached'
        self._calculate_changes() #-- restore dC to unreduced values
        
        if self.equilibration_mode==FIXED:  #-- check only time elapsed
            if self.total_time>max_time: return 1
            else: return 0

        #-- available time is unlimited: watch true equilibration
        if abs(self.v[1]-self.v[2]) < abs(epsilon*self.v[1]):
            return 1
        else:
            return 0

##        #-- different way to look for equilibrium condition reached:
##        #-- check if changes are gonna be less than a fraction of
##        #    the total concentration
##
##        equilibrated=0
##        for name in self.species:
##            if self.dC[name] < epsilon*self.C[name]: equilibrate=1
##            else: equilibrate*=0
##        if equilibrate: return 1
##        else: return 0
##        



##
##
###-----------------------------------------------------------------------
##class AB2_dependent_sites(Equilibrium):
##    """"A+B=AB, AB+B=AB2 - two unequivalent, dependent sites on A
##        Second site binds B only after the first site is occupied
##     """
##    def __init__(self, name="A+B=AB', AB'+B=AB'B''", kinetics_mode=1,equilibration_mode=UNLIMITED):
##        
##        #-- these are constant for the current system
##        self.species=["A","B","AB'","AB'B''"]
##        self.number_of_species=len(self.species)
##        self.number_of_constants=4
##        self.enthalpies={"AB'":  "not set",   # molar heat of formation
##                         "AB'B''": "not set"
##                         }
##        
##
##        
##        Equilibrium.__init__(self,name, kinetics_mode,equilibration_mode)
##
##    def _calculate_changes(self):
##        #-- aliasing for easier use
##        v=self.v
##        C=self.C
##        dC=self.dC
##        k=self.k
##
##        #--- velocities 
##        v[1]=k[1]*C["A"]*C["B"]
##        
##        v[2]=k[2]*C["AB'"]
##        
##        v[3]=k[3]*C["AB'"]*C["B"]
##        
##        v[4]=k[4]*C["AB'B''"]
##
##
##        #--'definition of dC[i]/dt'
##        dC["A"]=    -v[1]   +v[2]
##        
##        dC["B"]=    -v[1]   +v[2]   -v[3]   +v[4]
##        
##        dC["AB'"]=  +v[1]   -v[2]   -v[3]   +v[4]
##        
##        dC["AB'B''"]=               +v[3]   -v[4]
##
##
##        #--'proportional to changes'
##        
##        self.ddH=dC["AB'"]*self.enthalpies["AB'"]
##
##        self.ddH+=dC["AB'B''"]*self.enthalpies["AB'B''"]
##
##
##    #---------------------------------------------------------------
##    def _is_equilibrium(self, epsilon, max_time):
##        v=self.v
##        self._calculate_changes() #-- restore dC to unreduced values
##        
##        if self.equilibration_mode==FIXED:  #-- check only time elapsed
##            if self.total_time>max_time: return 1
##            else: return 0
##           
##        if (abs((v[1]-v[2])/v[1]) < epsilon and
##            abs((v[3]-v[4])/v[3]) < epsilon):
##            return 1
##        else:
##            return 0
##        
##        
##



##
##
##
##
##
###-----------------------------------------------------------------------
##class AB2_independent_sites(Equilibrium):
##    """A+B=AB', AB'+B=AB'B'',A+B=AB'', AB''+B=AB'B'', - two unequivalent, independent sites 
##    """
##    def __init__(self, name="A+B=AB, AB+B=AB2 independent"):
##        
##        #-- these are constant for the current system
##        self.species=["A","B","AB'","AB''","AB'B''"]
##        self.number_of_species=len(self.species)
##        self.number_of_constants=4
##        self.enthalpies={"AB'":  "not set",   # molar heat of formation
##                         "AB''": "not set",
##                         "AB'B''": "not set"                         
##                         }
##
##        Equilibrium.__init__(self,name)
##        
##
##    def _calculate_velocities(self):
##        'definition of current velocities'
##
##        self.v[1]=self.k[1]*self.C["A"]*self.C["B"]
##        self.v[2]=self.k[2]*self.C["AB'"]
##        self.v[3]=self.k[3]*self.C["A"]*self.C["B"]
##        self.v[4]=self.k[4]*self.C["AB''"]
##
##        self.v[5]=self.k[5]*self.C["AB'"]*self.C["B"]
##        self.v[6]=self.k[6]*self.C["AB'B''"]
##        self.v[7]=self.k[7]*self.C["AB''"]*self.C["B"]  
##        self.v[8]=self.k[8]*self.C["AB'B''"]
##
##
##
##    #---------------------------------------------------------------
##    def _calculate_dC(self):
##        'definition of dC[i]/dt'
##        self.dC["A"]=-self.v[1]+self.v[2]-self.v[3]+self.v[4]
##        
##        self.dC["B"]=-self.v[1]+self.v[2]-self.v[3]+self.v[4]-self.v[5]+self.v[6]-self.v[7]+self.v[8]
##
##        self.dC["AB'"]=self.v[1]-self.v[2]-self.v[5]+self.v[6]
##
##        self.dC["AB''"]=self.v[3]-self.v[4]-self.v[7]+self.v[8]
##
##        self.dC["AB'B''"]=self.v[5]-self.v[6]+self.v[7]-self.v[8]
##
##    #---------------------------------------------------------------
##    def _is_equilibrium(self, epsilon):
##        'v(forward)-v(backward) < epsilon'
##            
##        if (abs(self.v[1]-self.v[2]) < epsilon and abs(self.v[3]-self.v[4]) < epsilon
##            and abs(self.v[5]-self.v[6]) < epsilon and abs(self.v[7]-self.v[8]) < epsilon):
##            return 1
##        else:
##            return 0
##        
##    #--------------------------------------------------------------
##    def _integrate_heat(self,reduction_factor):
##        "proportional to AB'', AB' and AB'B'' changes"
##        self.dH+=self.dC["AB'"]*self.enthalpies["AB'"]*reduction_factor
##        self.dH+=self.dC["AB''"]*self.enthalpies["AB''"]*reduction_factor
##        self.dH+=self.dC["AB'B''"]*self.enthalpies["AB'B''"]*reduction_factor
##
##
##
##








#####################################################################
#####################################################################
#####################################################################
if __name__=="__main__":
    import xmgr


###########################################################################
###########################################################################
    #----------- simple 1:1 --------------------
###########################################################################
###########################################################################
    a=AB("AB",kinetics_mode=1,equilibration_mode=UNLIMITED)

    a.k[1]=0.5e8
    a.k[2]=0.5
    a.name="AB Ka=%.4e" % (a.k[1]/a.k[2])

    a.enthalpies["AB"]=-10e-6

    a.C["A"]=4e-6
    a.C["B"]=4e-6
    a.C["AB"]=0

    
##    #--- Single equilibrium study
##    if a.equilibrate(1e6,1e-6,0.025): print "Converged!"
##    a.equilibration_to_xmgr("equilibration_dynamics.xmgr",max_number_of_points=100)


    #-- Titration curve study
    ratio=2

    a.make_titration_curve("A","B",ratio,2,1e6,1e-6)
    a.titration_to_xmgr("titration",a.name)

    print a
