#!/usr/bin/env python

doc1="""
=================== es2m_full.py ======================
Prepare peak heights for relaxation dispersion analysis
by Evgenii Kovrigin, 2/29/08
version 1.5
"""

doc2="""

The utility to convert 'es lists' from Sparky to files of x,y,dy structure for fitting in Matlab.
I separate intensities per individual frequencies grouped by residue in the corresponding folders.
The goal is to make this data presented as simple text data matrices easily importable by MatLab.




====== INPUT ========
1. Peak list from es-command in Sparky ('es' in Sparky: 'Displaying the peak heights vs. time parameter').
   Spectra must have value of INDEX set to a serial number
   of the experiment in the acquisition schedule (simply taken from spectral names generated
   by 'process_rccpmg.py')
   
   NOTE: Your assignments have to be of the form xNNNx-x. First letter - single letter code for amino acid,
   then a numeral goes and LAST THREE letters are assignment to the atom types. No assignments like
   A1N-Ha2 acceptable!!!!
   
2. vclist and vdlist files  (number of cycles and taucp values, taken from ../../ folder
   with raw acquisition data)


3. pulseprogram (parsed to extract REF value - taucp threshold; taucp < REF flags  reference
                experiment where relaxation delay was skipped)

4. if you want to remove any spectrum from fitting :
    1. export "es_list.txt" from 'es' with ALL spectra
    2. make plain text file "spectra_to_exclude.es_list.txt" .
       Enter all indices on a single line (separated by spaces)
       from es Setup which you like to remove from fitting.
       Make sure you see a message "Dropping indices: ...."
       Only the first line of this file is read. In order to switch back to full
       set of spectra: rename the file (say, add underscore).
       If you remove all points for specific frequency it will not appear in the lists
       altogether.




'es' peak list format expected:
ResNum index1 index2 index3 ....
------------
x1 h1 h2 h3 ...
.
.
.
XNNN

======OUTPUT ========

The program will produce the following file structure:

[es_list].out/ folder (first part comes from supplied es_list name) containing:

    - reslist.txt - a list of assignments (like AnnnN-H..)
    - resnums.txt - a list of residue NUMBERS (nnn..., for plotting in Matlab)
    
    - XNNN/  - a number of folders with the names listed in reslist.txt
            Each of these folders will contain:
            
            - freq.txt - a list of frequencies to guide fitting
                        (delete any lines if needed to remove bad datasets
                        and the data for this frequency will be not read). Reference
                        experiment is represented as '0' frequency
                        
            - Num.txt - a series of files corresponding to each of frequencies in the freq.txt list
                        Each file contains [total relaxation time, s] vs [intensity] pairs.
                        Reference exeriment comes as '0.txt'.


======= USAGE ==================================

  es2m_full.py es_list.txt 







Tested on 12/22/07

1/9/08 - peak number output added
1/18/08 - peak number output corrected
1/29/08 - peak list sorted by residue number
        - option of dropping some spectra from fitting added
2/29/08 - output of reference frequency suppressed

=============================================================
"""

import os, sys, string
import mylib, residues


print doc1
#-- checking if the file name is supplied
if len(sys.argv)<2 :
    print doc2
    sys.exit()
else:
    es_list_fname=sys.argv[1]
    
#-- file and folder names
outdirname=es_list_fname+".full_parsed"
excluded_spectra="spectra_to_exclude."+es_list_fname  #-- to be able to drop certain spectrum
                                                        #  from the data set for all peaks
reslistname='reslist.txt'
resnumsname='resnums.txt'

freqlistname='freq.txt'
vc_vd_path='../../'   #-- path to vclist and vdlist files in acquisition data folder





#-- make output folder
if os.path.isdir(outdirname): os.system('rm -r %s' % outdirname)
os.mkdir(outdirname)



#------ PROCESS es-list ----
#-- read es-list
infile=mylib.open_file(es_list_fname,'r').readlines()

#--- extract indices ---
indices=map(float,infile[0].split()[1:])

#--- extract heights ---
lenres=len(infile)
resdict={}
heightslist=[]
reslist=[]   # a list of residues in the original order
for i in range(1,lenres):
    line=infile[i].split()
    resname=line[0]
    reslist.append(resname)

    #-- check for duplicate names (unassigned peaks like ?-?)
    if resname in resdict.keys():
        print "\n\nERROR!\n Residue '%s' appeared in the list twice!\n" % resname
        sys.exit()
        

    #-- extract heights
    #print "Extract data for %s" % resname
    resdict[resname]=map(float,line[1:])

    #---

#--- Write a residue list ----------------------------
resdict_sorted={}
#--- get the list
resfile=open(os.path.join(outdirname,reslistname),'w')
resnumfile=open(os.path.join(outdirname,resnumsname),'w')
for res in reslist:
    temp=res[1:]
    resnum=int(temp[:-3])
    resdict_sorted[resnum]=res
#------- output sorted list ---
reslist_sorted=resdict_sorted.keys()
reslist_sorted.sort()
for resnum in reslist_sorted:
    resname=resdict_sorted[resnum]
    resfile.write(resname+'\n')
    resnumfile.write('%d\n' % resnum)
resfile.close()
resnumfile.close()




#--- find which spectra to drop from datasets ---
excluded=[]
if os.path.isfile(excluded_spectra):
  text=mylib.open_file(excluded_spectra,'r').readlines()[0]
  excluded=map(int, string.split(text))
  print "\nDropping indices (as listed in '%s'):\n" % excluded_spectra,
  for spectrum in excluded:
      print spectrum, ' ',
  print
else: 
  print "\nUsing all spectral data."

  


#----- Calculate Ttotal -----
#-- read and extract vdlist and vclist values
vclist=map(int,mylib.open_file(os.path.join(vc_vd_path,'vclist'),'r').readlines())
vdlist=map(float,mylib.open_file(os.path.join(vc_vd_path,'vdlist'),'r').readlines())

#-- read and find reference value
pulseprogram=mylib.open_file(os.path.join(vc_vd_path,'pulseprogram'),'r').readlines()
ref_found=0
for line in pulseprogram:
    words=string.split(line,'=') #-- separate variable name
    if words[0]=='"REF':
        REF=float(string.split(words[1],'"')[0])  #-- get the number out
        print "\nReference experiment flag REF=%f extracted from 'pulseprogram' line:" % REF
        print line
        print
        ref_found=1
if ref_found==0:
    print "\n\nERROR!\nPulse program does NOT contain a flag variable 'REF=...' to identify reference experiments!"
    sys.exit()



#--- calculate Ttotal and freq in original order 

freqlist=[] 
Ttotallist=[] 
for i in range(len(vclist)):
    Ttotal=0
    freq=0
    ncyc=vclist[i]
    taucp=vdlist[i]
    if taucp > REF:
       Ttotal=2.0*ncyc*taucp
       freq=int(1/(2.0*taucp))  #-- rounded. I use it to identify reference experiment from now

    #--- collect data
    freqlist.append(freq)
    Ttotallist.append(Ttotal)
    

        
#---- Collect spectral intensities into folders by residue and into files by frequency


for resnum in reslist_sorted:
    res=resdict_sorted[resnum]
    
    print 'Processing "%s"' % res

    #-- make folder for current residue
    os.mkdir(os.path.join(outdirname,res))

    
    #-- collect heights for corresponding frequencies and reference expt (freq=0)
    heights=resdict[res]
    freqdict={}
    for i in range(len(freqlist)):
        if i in excluded:
            continue #-- do not use data from this spectrum
        freq=freqlist[i]
        if freq not in freqdict.keys(): freqdict[freq]=[]
        freqdict[freq].append([Ttotallist[i],heights[i]])
        


    #-- output frequency list (for matlab to find files)
    freqfile=open(os.path.join(outdirname,res,'frequencies.txt'),'w')  
    frequency_list=	freqdict.keys()
    frequency_list.sort()
    
    for freq in frequency_list:
        if freq>0: #-- do not output reference experiment frequency (fake frequency of 0Hz)
            freqfile.write('%d\n'%freq)
    freqfile.close()
    
                
    #--- output frequency-specific data 
    for freq in freqdict.keys():
        #if freq==0: continue

        #-- open frequency-specific file
        outfile=open(os.path.join(outdirname,res,('%d'%freq)),'w')
        
            
        #-- output relaxation experiments
        for line in freqdict[freq]:
            outfile.write('%f\t%f\t%f\n'%(line[0], line[1], freq*2*3.1415926))
            
        outfile.close()
        

#-- output global frequency list
os.system('cp %s %s' %(os.path.join(outdirname,res,'frequencies.txt'),
                       os.path.join(outdirname,'frequencies.txt')))

print '\n%d peaks done.\n' % len(resdict.keys())


























               
