#!/usr/bin/env python

doc1="""
=================== es2m_full_dispersion.py ======================
Prepare peak heights for relaxation dispersion analysis
by Evgenii Kovrigin, 2/29/08
version 1.7
"""

doc2="""

The utility to convert 'es lists' from Sparky to files of x,y,dy structure for fitting in Matlab.
I separate intensities per individual frequencies grouped by residue in the corresponding folders.
The goal is to make this data presented as simple text data matrices easily importable by MatLab.

Here I expect the pseudo-3D experiments added together and interleaved points
to represent random scatter for fitting procedure to estimate errors.


====== INPUT ========
1. Peak list from es-command in Sparky ('es' in Sparky: 'Displaying the peak heights vs. time parameter').
   Spectra must have value of INDEX set to a serial number
   of the experiment in the acquisition schedule (simply taken from spectral names generated
   by 'process_rccpmg.py')
   
   NOTE: Your assignments have to be of the form xNNNx-x. First letter - single letter code for amino acid,
   then a numeral goes and LAST THREE letters are assignment to the atom types. No assignments like
   A1N-Ha2 acceptable!!!!
   
2. vclist and vdlist files  (number of cycles and taucp values, taken from ../../ folder
   with raw acquisition data)


3. pulseprogram (parsed to extract REF value - taucp threshold; taucp < REF flags  reference
                experiment where relaxation delay was skipped)

4. if you want to remove any spectrum from fitting :
    1. export "es_list.txt" from 'es' with ALL spectra
    2. make plain text file "spectra_to_exclude.txt" .
       Enter all indices on a single line (separated by spaces)
       from es Setup which you like to remove from fitting.
       Make sure you see a message "Dropping indices: ...."
       Only the first line of this file is read. In order to switch back to full
       set of spectra: rename the file (say, add underscore).
       If you remove all points for specific frequency it will not appear in the lists
       altogether.



'es' peak list format expected:
ResNum index1 index2 index3 ....
------------
x1 h1 h2 h3 ...
.
.
.
XNNN

====== OUTPUT DATA STRUCTURE ========

The program WILL REMOVE the output folder if it exists to prevent mixing 'old' and 'new' data.
Make sure you move previous result somewhere if you wnat to save it.

The program will produce the following file structure:

[es_list].out/ folder (first part comes from supplied es_list name) containing:

    - reslist.txt - a list of assignments (like AnnnN-H..)
    - resnums.txt - a list of residue NUMBERS (nnn..., for plotting in Matlab).
    If you want to edit residue list to fit then edit reslist.txt and use
    Annn2nnn.py to strip the first letter and produce resnums.txt
    
    - XNNN/  - a number of folders with the names listed in reslist.txt
            Each of these folders will contain:
            
            - freq.txt - a list of frequencies to guide fitting
                        (delete any lines if needed to remove bad datasets
                        and the data for this frequency will be not read). Reference
                        experiment is represented as '0' frequency
                        
            - Num.txt - a series of files corresponding to each of frequencies in the freq.txt list
                        Each file contains
                        
   [total relaxation time, s]  [intensity]   [CPMG frequency, Hz]  [offset from carrier, ppm] [CPMG-nucleus frequency, MHz]
   
                        Reference exeriment comes as '0.txt'. You may also delete any lines in these
                        files to remove data from fitting


======= PARAMETERS AND USAGE ==================================

  es2m_full_dispersion_v1.py es_list.txt peak_list X_carrier_ppm X_dimension X_freq_MHz

            
      "es_list.txt" - a peak list from es-command in Sparky ('es' in Sparky: 'Displaying the peak heights vs. time parameter').
               Spectra must have value of INDEX set to a frequency (Hz). Sparky does that automatically if spectra
               are labeled with the frequency in the beginning of the file name.
               
               NOTE: Your assignments have to be of the form xNNNx-x. First letter - single letter code for amino acid,
               then a numeral goes and LAST THREE letters are assignment to the atom types. No assignments like
               A1N-Ha2 acceptable!!!!
               
      "peak_list.txt" - a lt-list of peak ppm positions to record offset from carrier

      X_carrier_ppm - CPMG-nucleus carrier ppm position (carrier position for the nucleus subject to CPMG pulse train)
                       
      X_dimension - 1 or 2. Look up HSQC and find how X-nucleus is numbered

      X_freq_MHz - CPMG-nucleus frequency, MHz








Tested on 12/22/07

1/9/08 - peak number output added
1/18/08 - peak number output corrected
1/29/08 - peak list sorted by residue number
        - option of dropping some spectra from fitting added
2/29/08 - output of reference frequency suppressed
5/13/08 - outputs taucp instead of frequency per second for fitting purposes
5/22/08 - field_factor introduced to allow for multi-field data fitting
6/11/08 - input-output parameter format change: add offset value and make format
          uniform with es2m_full_dispersion_v2.py
6/11/08 - negative intensities reported
6/15/08 - residue number output removed


=============================================================
"""

import os, sys, string
import mylib, residues


print doc1
#-- checking if the file name is supplied
if len(sys.argv)<6 :
    print doc2
    sys.exit()
else:
    es_list_fname=sys.argv[1]
    peak_list_fname=sys.argv[2]
    X_carrier_ppm=float(sys.argv[3])
    X_dimension=int(sys.argv[4])-1 #-- as indices should start with 0 in computer world
    X_freq_MHz=float(sys.argv[5])
    
#-- file and folder names
outdirname=es_list_fname+".parsed.v1"
excluded_spectra="spectra_to_exclude.txt"  #-- to be able to drop certain spectrum
                                                        #  from the data set for all peaks
reslistname='reslist.txt'
#resnumsname='resnums.txt'

freqlistname='freq.txt'
vc_vd_path='../../'   #-- path to vclist and vdlist files in acquisition data folder





#-- make output folder
if os.path.isdir(outdirname): os.system('rm -r %s' % outdirname)
os.mkdir(outdirname)

#-- read peak list
peak_positions=residues.Sparky_lt_Peaklist(peak_list_fname)


#------ PROCESS es-list ----
#-- read es-list
infile=mylib.open_file(es_list_fname,'r').readlines()

#--- extract indices ---
indices=map(float,infile[0].split()[1:])

#--- extract heights ---
lenres=len(infile)
resdict={}
heightslist=[]
reslist=[]   # a list of residues in the original order
for i in range(1,lenres):
    
    #-- get residue name and convert from AnnnN-H to Annn
    line=infile[i].split()
    resname=line[0][:-3]
    reslist.append(resname)

    #-- check for duplicate names (unassigned peaks like ?-?)
    if resname in resdict.keys():
        print "\n\nERROR!\n Residue '%s' appeared in the list twice!\n" % resname
        sys.exit()
        

    #-- extract heights
    #print "Extract data for %s" % resname
    resdict[resname]=map(float,line[1:])

    #---

#--- Write a residue list ----------------------------
resdict_sorted={}
#--- get the list
resfile=open(os.path.join(outdirname,reslistname),'w')
#resnumfile=open(os.path.join(outdirname,resnumsname),'w')
for res in reslist:
    temp=res[1:]
    resnum=int(temp)
    resdict_sorted[resnum]=res
#------- output sorted list ---
reslist_sorted=resdict_sorted.keys()
reslist_sorted.sort()
for resnum in reslist_sorted:
    resname=resdict_sorted[resnum]
    resfile.write(resname+'\n')
    #resnumfile.write('%d\n' % resnum)
resfile.close()
#resnumfile.close()




#--- find which spectra to drop from datasets ---
excluded=[]
if os.path.isfile(excluded_spectra):
  text=mylib.open_file(excluded_spectra,'r').readlines()[0]
  excluded=map(int, string.split(text))
  print "\nDropping indices (as listed in '%s'):\n" % excluded_spectra,
  for spectrum in excluded:
      print spectrum, ' ',
  print
else: 
  print "\nUsing all spectral data."

  


#----- Calculate Ttotal -----
#-- read and extract vdlist and vclist values
vclist=map(int,mylib.open_file(os.path.join(vc_vd_path,'vclist'),'r').readlines())
vdlist=map(float,mylib.open_file(os.path.join(vc_vd_path,'vdlist'),'r').readlines())

#-- read and find reference value
pulseprogram=mylib.open_file(os.path.join(vc_vd_path,'pulseprogram'),'r').readlines()
ref_found=0
for line in pulseprogram:
    words=string.split(line,'=') #-- separate variable name
    if words[0]=='"REF':
        REF=float(string.split(words[1],'"')[0])  #-- get the number out
        print "\nReference experiment flag REF=%f extracted from 'pulseprogram' line:" % REF
        print line
        print
        ref_found=1
if ref_found==0:
    print "\n\nERROR!\nPulse program does NOT contain a flag variable 'REF=...' to identify reference experiments!"
    sys.exit()



#--- calculate Ttotal and freq in original order 

freqlist=[] 
Ttotallist=[] 
taucplist=[]
for i in range(len(vclist)):
    Ttotal=0
    freq=0
    ncyc=vclist[i]
    taucp=vdlist[i]
    if taucp > REF:
       Ttotal=2.0*ncyc*taucp
       freq=int(1/(2.0*taucp))  #-- rounded. I use it to identify reference experiment from now

    #--- collect data
    freqlist.append(freq)
    taucplist.append(taucp)
    Ttotallist.append(Ttotal)
    

        
#---- Collect spectral intensities into folders by residue and into files by frequency


for resnum in reslist_sorted:
    res=resdict_sorted[resnum]
    
    print 'Processing "%s"' % res

    #-- make folder for current residue
    os.mkdir(os.path.join(outdirname,res))


    #-- calculate offset from carrier
    offset_ppm= peak_positions[res][X_dimension] - X_carrier_ppm

    
    #-- collect heights for corresponding frequencies and reference expt (freq=0)
    heights=resdict[res]
    
    #--- abort if there are heights <=0
    for height in heights:
        if height > 0: continue
        raise 'NEGATIVE INTENSITIES', "\n\n\nERROR! Residue '%s' has negative intensity values (cannot be fit)!\nPlease, remove %s from  '%s'!\n" % (
        res, res, es_list_fname)
    
    
    freqdict={}
    for i in range(len(freqlist)):
        if i in excluded:
            continue #-- do not use data from this spectrum
        freq=freqlist[i]
        if freq not in freqdict.keys(): freqdict[freq]=[]
        
        #----  PREPARE A DATA STRING FOR OUTPUT ------
        freqdict[freq].append([Ttotallist[i],heights[i], freq, offset_ppm, X_freq_MHz ])
        


    #-- output frequency list (for matlab to find files)
    freqfile=open(os.path.join(outdirname,res,'frequencies.txt'),'w')  
    frequency_list=	freqdict.keys()
    frequency_list.sort()
    
    for freq in frequency_list:
        if freq>0: #-- do not output reference experiment frequency (fake frequency of 0Hz)
            freqfile.write('%d\n'%freq)
    freqfile.close()
    
                
    #--- output frequency-specific data 
    for freq in freqdict.keys():
        #if freq==0: continue

        #-- open frequency-specific file
        outfile=open(os.path.join(outdirname,res,('%d.txt'%freq)),'w')
        
            
        #-- output relaxation experiments
        for line in freqdict[freq]:
            out_line=''
            for word in line:
                out_line+="%f\t" % word
            outfile.write('%s\n' % out_line)
        outfile.close()
       

#-- output global frequency list
os.system('cp %s %s' %(os.path.join(outdirname,res,'frequencies.txt'),
                       os.path.join(outdirname,'frequencies.txt')))

print '\n%d peaks done.\n' % len(resdict.keys())


























               
