#!/usr/bin/env python

doc1="""
=================== es2m_full_dispersion.py ======================
Prepare peak heights for relaxation dispersion analysis
by Evgenii Kovrigin, 6/9/08
version 2.0
"""

doc2="""

The utility to convert 'es lists' from Sparky to files of x,y,dy structure for fitting in Matlab.
I separate intensities per individual frequencies grouped by residue in the corresponding folders.
The goal is to make this data presented as simple text data matrices easily importable by MatLab.

Here I utilize Sparky project where interleaved points are added together and I have multiple data sets
from different days

If you want to remove some spectrum from fitting: simply unselect it in eS Setup window before generating
and saving the peak list.

'es' peak list format expected:
ResNum index1 index2 index3 ....
------------
x1 h1 h2 h3 ...
.
.
.
XNNN

====== OUTPUT DATA STRUCTURE ========

The program WILL REMOVE the output folder if it exists to prevent mixing 'old' and 'new' data.
Make sure you move previous result somewhere if you wnat to save it.

The program will produce the following file structure:

[es_list].out/ folder (first part comes from supplied es_list name) containing:

    - reslist.txt - a list of assignments (like AnnnN-H..)
    - resnums.txt - a list of residue NUMBERS (nnn..., for plotting in Matlab).
    If you want to edit residue list to fit then edit reslist.txt and use
    Annn2nnn.py to strip the first letter and produce resnums.txt
    
    - XNNN/  - a number of folders with the names listed in reslist.txt
            Each of these folders will contain:
            
            - freq.txt - a list of frequencies to guide fitting
                        (delete any lines if needed to remove bad datasets
                        and the data for this frequency will be not read). Reference
                        experiment is represented as '0' frequency
                        
            - Num.txt - a series of files corresponding to each of frequencies in the freq.txt list
                        Each file contains
                        
   [total relaxation time, s]  [intensity]   [CPMG frequency, Hz]  [offset from carrier, ppm] [CPMG-nucleus frequency, MHz]
   
                        Reference exeriment comes as '0.txt'. You may also delete any lines in these
                        files to remove data from fitting


======= PARAMETERS AND USAGE ==================================

  es2m_full_dispersion_v2.py es_list.txt peak_list frequency_ttotal.txt X_carrier_ppm X_dimension X_freq_MHz

            
      "es_list.txt" - a peak list from es-command in Sparky ('es' in Sparky: 'Displaying the peak heights vs. time parameter').
               Spectra must have value of INDEX set to a frequency (Hz). Sparky does that automatically if spectra
               are labeled with the frequency in the beginning of the file name.
               
               NOTE: Your assignments have to be of the form xNNNx-x. First letter - single letter code for amino acid,
               then a numeral goes and LAST THREE letters are assignment to the atom types. No assignments like
               A1N-Ha2 acceptable!!!!
               
      "peak_list.txt" - a lt-list of peak ppm positions to record offset from carrier

      "frequency_ttotal.txt" - file with frequency- Ttotal pairs for R2 fitting (created by analyze_interleaved_replicates.py)
               Located in ../../../ from Sparky folder

      X_carrier_ppm - CPMG-nucleus carrier ppm position (carrier position for the nucleus subject to CPMG pulse train)
                       
      X_dimension - 1 or 2. Look up HSQC and find how X-nucleus is numbered

      X_freq_MHz - CPMG-nucleus frequency, MHz


=============================================================
Bugs and fixes:
6/11/08 - negative intensities reported
6/15/08 - residue number output removed
"""

import os, sys, string
import mylib, residues



print doc1
#-- checking if the parameters are supplied
if len(sys.argv)<7 :
    print doc2
    sys.exit()
else:
    es_list_fname=sys.argv[1]
    peak_list_fname=sys.argv[2]
    frequency_ttotal_fname=sys.argv[3]
    X_carrier_ppm=float(sys.argv[4])
    X_dimension=int(sys.argv[5])-1 #-- as indices should start with 0 in computer world
    X_freq_MHz=float(sys.argv[6])
    
#-- file and folder names
outdirname=es_list_fname+".parsed.v2"
reslistname='reslist.txt'
#resnumsname='resnums.txt'




#-- make output folder
if os.path.isdir(outdirname): os.system('rm -r %s' % outdirname)
os.mkdir(outdirname)



#------ PROCESS es-list ----

#-- read peak list
peak_positions=residues.Sparky_lt_Peaklist(peak_list_fname)

#-- read es-list
infile=mylib.open_file(es_list_fname,'r').readlines()

#--- extract frequency indices from es_list file header---
frequency_indices=map(float,infile[0].split()[1:])

#--- extract heights ---
lenres=len(infile)
resdict={}
heightslist=[]
reslist=[]   # a list of residues in the original order
for i in range(1,lenres):
    
    #-- get residue name and convert from AnnnN-H to Annn
    line=infile[i].split()
    resname=line[0][:-3]
    reslist.append(resname)

    #-- check for duplicate names (unassigned peaks like ?-?)
    if resname in resdict.keys():
        print "\n\nERROR!\n Residue '%s' appeared in the list twice!\n" % resname
        sys.exit()
        

    #-- extract heights
    #print "Extract data for %s" % resname
    resdict[resname]=map(float,line[1:])

    #---


#--- Write a residue list ----------------------------
resdict_sorted={}
#--- get the list
resfile=open(os.path.join(outdirname,reslistname),'w')
#resnumfile=open(os.path.join(outdirname,resnumsname),'w')
for res in reslist:
    temp=res[1:]
    resnum=int(temp)
    resdict_sorted[resnum]=res
#------- output sorted list ---
reslist_sorted=resdict_sorted.keys()
reslist_sorted.sort()
for resnum in reslist_sorted:
    resname=resdict_sorted[resnum]
    resfile.write(resname+'\n')
    #resnumfile.write('%d\n' % resnum)
resfile.close()
#resnumfile.close()  


#----- Get Frequency - Ttotal correspondence table -----
#-- 
frequency_ttotal_list=mylib.open_file(frequency_ttotal_fname,'r').readlines()

Ttotal_summary_dict={} 
 
for line in frequency_ttotal_list:
    words=line.split()
    Ttotal=0
    freq=int(words[0])
    if freq > 0:
       Ttotal=float(words[1])

    #--- collect data
    Ttotal_summary_dict[freq ]= Ttotal
    

        
#---- Collect spectral intensities into folders by residue and into files by frequency

for resnum in reslist_sorted:
    res=resdict_sorted[resnum]
    
    #-- calculate offset from carrier
    offset_ppm= peak_positions[res][X_dimension] - X_carrier_ppm
    
    print 'Processing "%s"' % res

    #-- make folder for current residue
    os.mkdir(os.path.join(outdirname,res))

    
    #-- collect heights for corresponding frequencies and reference expt (freq=0)
    heights=resdict[res]
    
    #--- abort if there are heights <=0
    for height in heights:
        if height > 0: continue
        raise 'NEGATIVE INTENSITIES', "\n\n\nERROR! Residue '%s' has negative intensity values (cannot be fit)!\nPlease, remove %s from  '%s'!\n" % (
        res, res, es_list_fname)
   
    
    freqdict={}
    for i in range(len(frequency_indices)):
        freq=frequency_indices[i]
        if freq not in freqdict.keys(): freqdict[freq]=[]
        
        #----  PREPARE A DATA STRING FOR OUTPUT ------
        freqdict[freq].append([Ttotal_summary_dict[freq ],heights[i], freq, offset_ppm, X_freq_MHz  ])
        


    #-- output frequency list (for matlab to find files)
    freqfile=open(os.path.join(outdirname,res,'frequencies.txt'),'w')  
    frequency_list=	freqdict.keys()
    frequency_list.sort()
    
    for freq in frequency_list:
        if freq>0: #-- do not output reference experiment frequency (fake frequency of 0Hz)
            freqfile.write('%d\n'%freq)
    freqfile.close()
    
                
    #--- output data files for each frequency within current residue 
    for freq in freqdict.keys():
        #if freq==0: continue

        #-- open frequency-specific file
        outfile=open(os.path.join(outdirname,res,('%d.txt'%freq)),'w')
        
            
        #-- output relaxation experiments
        for line in freqdict[freq]:
            out_line=''
            for word in line:
                out_line+="%f\t" % word
            outfile.write('%s\n' % out_line)
        outfile.close()
        

#-- output global frequency list
os.system('cp %s %s' %(os.path.join(outdirname,res,'frequencies.txt'),
                       os.path.join(outdirname,'frequencies.txt')))

print '\n%d peaks done.\n' % len(resdict.keys())


























               
