#!/usr/bin/env python

doc1="""
=================== es2m_v2.py ======================
Extract peak heights for relaxation dispersion analysis
by Evgenii Kovrigin, 6/8/08
version 2
"""

doc2="""

The utility to convert 'es lists' from Sparky to files of x,y,dy structure for fitting in Matlab

('es' in Sparky: 'Displaying the peak heights vs. time parameter')


====== INPUT ========
1. Peak list from es-command in Sparky. Spectra must have value of INDEX set to a serial number
   of the experiment in the acquisition schedule (simply taken from spectral names generated
   by 'process_rccpmg.py')
   
2. If you want to remove some spectrum from fitting: simply unselect it in eS before saving
   the peak list.

'es' peak list format expected:
ResNum index1 index2 index3 ....
------------
x1 h1 h2 h3 ...
.
.
.
XNNN

======OUTPUT ========

The program will produce the following file structure:

[es_list].out/ folder (first part comes from supplied es_list name) containing:

    - reslist.txt - a list of residues
    
    - XNNN/  - a number of folders with the names listed in reslist.txt
            Each of these folders will contain:
            
            - freq.txt - a list of frequencies to guide fitting
                        (delete any lines if needed to remove bad datasets
                        and the data for this frequency will be not read)
                        
            - Num.txt - a series of files corresponding to each of frequencies in the freq.txt list
                        Each file contains [total relaxation time, s] vs [intensity] pairs to fit.


======= USAGE ==================================

  es2m_v2.py es_list.txt 

    - es_list is a   list  saved from 'es' command  in Sparky.
      Spectra must have value of INDEX set to a serial number
      of the experiment in the acquisition schedule (simply taken from spectral names generated
      by 'process_rccpmg.py')




=============================================================
"""

import os, sys, string
import mylib, residues


print doc1
#-- checking if the file name is supplied
if len(sys.argv)<2 :
    print doc2
    sys.exit()
else:
    es_list_fname=sys.argv[1]
    
#-- file and folder names
outdirname=es_list_fname+".parsed"
excluded_spectra="spectra_to_exclude."+es_list_fname  #-- to be able to drop certain spectrum
                                                        #  from the data set for all peaks
reslistname='reslist.txt'
freqlistname='freq.txt'
vc_vd_path='../../'   #-- path to vclist and vdlist files in acquisition data folder





#-- make output folder
if os.path.isdir(outdirname): os.system('rm -r %s' % outdirname)
os.mkdir(outdirname)



#------ PROCESS es-list ----
#-- read es-list
infile=mylib.open_file(es_list_fname,'r').readlines()

#--- extract indices ---
indices=map(float,infile[0].split()[1:])

#--- extract heights ---
lenres=len(infile)
resdict={}
heightslist=[]
reslist=[]   # a list of residues in the original order
for i in range(1,lenres):
    line=infile[i].split()
    resname=line[0]
    reslist.append(resname)

    #-- check for duplicate names (unassigned peaks like ?-?)
    if resname in resdict.keys():
        print "\n\nERROR!\n Residue '%s' appeared in the list twice!\n" % resname
        sys.exit()
        

    #-- extract heights
    print "Extract data for %s" % resname
    resdict[resname]=map(float,line[1:])

    #---

#--- output residue list
resfile=open(os.path.join(outdirname,reslistname),'w')
for res in reslist:
    resfile.write(res+'\n')
resfile.close()    

#--- find which spectra to drop from datasets ---
excluded=[]
if os.path.isfile(excluded_spectra):
  excluded=map(int, mylib.open_file(excluded_spectra,'r').readlines())
  print "\nDropping indices:",
  for line in excluded:
      print line, ' ',
  print
else: 
  print "\nUsing all spectral data."

  


#----- Calculate Ttotal -----
#-- read and extract vdlist and vclist values
vclist=map(int,mylib.open_file(os.path.join(vc_vd_path,'vclist'),'r').readlines())
vdlist=map(float,mylib.open_file(os.path.join(vc_vd_path,'vdlist'),'r').readlines())

#-- read and find reference value
pulseprogram=mylib.open_file(os.path.join(vc_vd_path,'pulseprogram'),'r').readlines()
ref_found=0
for line in pulseprogram:
    words=string.split(line,'=') #-- separate variable name
    if words[0]=='"REF':
        REF=float(string.split(words[1],'"')[0])  #-- get the number out
        print "\nReference experiment flag REF=%f extracted from 'pulseprogram' line:" % REF
        print line
        print
        ref_found=1
if ref_found==0:
    print "\n\nERROR!\nPulse program does NOT contain a flag variable 'REF=...' to identify reference experiments!"
    sys.exit()



#--- calculate Ttotal and freq in original order 

freqlist=[] 
Ttotallist=[] 
for i in range(len(vclist)):
    Ttotal=0
    freq=0
    ncyc=vclist[i]
    taucp=vdlist[i]
    if taucp > REF:
       Ttotal=2.0*ncyc*taucp
       freq=int(1/(2.0*taucp))  #-- rounded. I use it to identify reference experiment from now

    #--- collect data
    freqlist.append(freq)
    Ttotallist.append(Ttotal)
    

        
#---- Collect spectral intensities into folders by residue and into files by frequency


for res in resdict.keys():
#    print 'Processing "%s"' % res

    #-- make folder for current residue
    os.mkdir(os.path.join(outdirname,res))

    
    #-- collect heights for corresponding frequencies and reference expt (freq=0)
    heights=resdict[res]
    freqdict={}
    for i in range(len(freqlist)):
        if i in excluded:
            continue #-- do not use data from this spectrum
        freq=freqlist[i]
        if freq not in freqdict.keys(): freqdict[freq]=[]
        freqdict[freq].append([Ttotallist[i],heights[i]])
        

    #-- output frequency list (for matlab to find files)
    freqfile=open(os.path.join(outdirname,res,'frequencies.txt'),'w')  
    frequency_list=	freqdict.keys()
    frequency_list.sort()
    
    for freq in frequency_list:
        if freq==0: continue
        #if len(freqdict[freq])==0: continue  #-- do not output frequency for which all points were dropped
        freqfile.write('%d\n'%freq)
    freqfile.close()
    
                
    #--- output frequency-specific data 
    for freq in freqdict.keys():
        if freq==0: continue

        #-- open frequency-specific file
        outfile=open(os.path.join(outdirname,res,('%d'%freq)),'w')
        
        #-- output reference expt
        for line in freqdict[0]:
            outfile.write('%f\t%f\n'%(line[0], line[1]))
            
        #-- output relaxation experiments
        for line in freqdict[freq]:
            outfile.write('%f\t%f\n'%(line[0], line[1]))
            
        outfile.close()
        
        

print '\n%d peaks done.\n' % len(resdict.keys())


#-- output global frequency list
os.system('cp %s %s' %(os.path.join(outdirname,res,'frequencies.txt'),
                       os.path.join(outdirname,'frequencies.txt')))
























               
