# -----------------------------------------------------------------------------
doc="""---------- Grab Peaks -----------------
The utility to tabulate observed resonances without explicit assigning them
in Sparky. This may be needed in case where just a partial assignment is possible
and the data are to be analyzed in the tabulated form rather them as a raw spectrum.

Usage:
  - open HSQC spectrum, containing assignments of interest
  - open (unassigned) 3D-HSQC spectrum
  - sincronize axis properly
  - hit F1 (region selection)
  - open this extension from HSQC spectrum (to have it selected)
  - hit "Define file" - to enter or select the file name to save/append data to
    (each time you hit the button it closes previous file and opens a new one)
  - hit "w1","w2" or "w3" to define dimension to take range values in
  - select one peak in HSQC
  - hit "Grab diag peak" - this peak becomes 'current'
  - find cross peak in 3D
  - drag a box around it caring only about boundaries in the selected dimension:
    these boundaries mean "where the maxima of peaks may be located"
  - boundaries displayed
  - "Undo" - removes the previous selection
  - repeat cross-peak selection as many times as needed
  - hit "Append data" when finished
  - select another HSQC peak
  - hit "Close" to close the file and finish


  - if no cross peaks are found - entry is NONE for the particular residue

"""
program_name="Grab Peaks"


import sputil
import tkutil
import tkMessageBox, tkFileDialog
import os, sys
import residues,mylib


from math import *

# ---------------------- UTILITY ---------------------
def peakname(assignment):
  "N67N-H to N67"
  return assignment[:-3]
  
# ---------------------- DATA STORAGE ---------------------  
class cross_peaks:
  "storage of picked peaks"
  filename=None
  data=None
  #----------------------------------  
  def __init__(self,session,filename):
    if os.path.isfile(filename): self.data=residues.Annn_Text(filename)
    else: self.data=residues.Annn_Text()
    self.filename=filename
    
  #----------------------------------
  def add_data(self,peak,list):
    if peak in self.data.keys():
      if tkMessageBox.askyesno("Grab Peaks","'%s' was already added! Replace?"):
        #--- replace if exists
        self.data[peak]=list
    else:
      #--- insert new record
      self.data.add(peak,list)
  #----------------------------------
  def close(self):
    #--- close file ---
    self.data.writeout(self.filename,header="OFF")









      
#----------------------------- DIALOG ---------------------------------
class grab_peaks_dialog(tkutil.Dialog):
  current_filename=None 
  current_peak=None
  grabbed_regions=[]
  current_data=None
  current_axis=0
  
  
  def __init__(self, session):

    self.session = session
    self.drag_notice = None

    tkutil.Dialog.__init__(self, session.tk, 'Grab Cross Peaks v.1')

    self.top.columnconfigure(0, weight = 1)

    import Tkinter
    explain = ('Please, define database file:')
    w = Tkinter.Label(self.top, text = explain, justify = 'left')
    w.grid(row = 0, column = 0, sticky = 'nw')

    r = Tkinter.Label(self.top, justify = 'left')
    r.grid(row = 1, column = 0, sticky = 'nw')
    self.result_label = r

    br = tkutil.button_row(self.top,
                           ('Define File', self.define_file),
                           ('Current Peak',self.set_current_peak),
                           ('w1',self.set_w1),
                           ('w2',self.set_w2),
                           ('w3',self.set_w3),
                           ('Append data', self.append_data),
                           ('Close', self.close),
                           ('Cancel session', self.cancel_session),
                           ('Help', self.help)
			   )
    br.frame.grid(row = 2, column = 0, sticky = 'nw')

    self.register_callback(1)
    self.top.bind('<Map>', lambda e, s=self: s.register_callback(1))
    self.top.bind('<Unmap>', lambda e, s=self: s.register_callback(0))

  # ---------------------------------------------------------------------------
  #


  def error(self,text):
    "generic error message"
    tkMessageBox.showerror("Grab Peaks", text)


  # ----- define file --------------------------
  def define_file(self):
    "get file name and load the file"
    if self.current_filename:
      self.error("Close or Cancel current session first!")
      return
    
    fname=tkFileDialog.asksaveasfilename()

    if not fname: return

    #--- open new file 
    self.current_filename=fname    
    self.current_data=cross_peaks(self.session,fname)

 # ---------------------------------------------------------------------------
 #
  def set_current_peak(self):

    #--- file is open ? --
    if not self.current_filename:
      self.error("Please, Define file first!")
      return

    #--- current peak not finished ? --
    if self.current_peak:
      self.error("Please, Append or Cancel '%s'data first!" % self.current_peak)
      return

    #--- one peaks selected ?
    if len(self.session.selected_peaks())!=1:
      self.error("Please, select one HSQC peak!")
      return
    
    self.current_peak=peakname(self.session.selected_peaks()[0].assignment)
    print "current peak", self.current_peak

  def set_w1(self):
    self.current_axis=0
    self.result_label['text'] = "Current axis - w1"
    
  def set_w2(self):
    self.current_axis=1
    self.result_label['text'] = "Current axis - w2"
    
  def set_w3(self):
    self.current_axis=2
    self.result_label['text'] = "Current axis - w3"
    


  #-------------------------------------------------------
  def append_data(self):

    #--- file is open ? --
    if not self.current_filename:
      self.error("Please, Define file first!")
      return
    
    if not self.current_peak:
      self.error("Please, select one HSQC peak!")
      return

    if not self.grabbed_regions:
      data="NONE"
    else:
      data=self.grabbed_regions
    self.current_data.add_data(self.current_peak,data)

    #--- reset things ---
    self.current_peak=None
    self.grabbed_regions=[]
    
  #------------------------------------------------------------
  def close(self):
    #-- closing file
    self.current_data.close()
    self.current_filename=None
    self.current_peak=None
    self.grabbed_regions=[]
    self.close_cb()

  #------------------------------------------------------------
  def cancel_session(self):
    self.current_data=None
    self.current_filename=None
    self.current_peak=None
    self.grabbed_regions=[]
    self.close_cb()

  #--------------------------------------------------------------
  def help(self):
    self.session.command_characters('py')
    print "\n\n==========================================="
    print doc
    print "============= see help message above ! =======\n"

  # ---------------------------------------------------------------------------
  #
  


















  def register_callback(self, register):

    if register:
      if self.drag_notice == None:
        self.drag_notice = self.session.notify_me('drag region',
                                                  self.region_selected_cb)
    else:
      if self.drag_notice:
        self.session.dont_notify_me(self.drag_notice)
        self.drag_notice = None

  # ---------------------------------------------------------------------------
  #
  def region_selected_cb(self, region):

    if self.is_window_destroyed():
      self.register_callback(0)
      return

    #--ek
    self.my_region=region

    s = self.session.selected_spectrum()
    if s == None:
      self.result_label['text'] = 'No spectrum selected'
      return

    rmin, rmax = region
    ranges = map(lambda vmin, vmax: '%.4g-%.4g' % (vmin, vmax), rmin, rmax)
    my_range=ranges[self.current_axis]
    print my_range    
    #-----------------------------------------------------------------------
    self.grabbed_peaks.append(my_range)
    
    msg = ('  Region :     %s\n\n' % self.grabbed_regions)
    print msg
    #self.result_label['text'] = msg
    


    
#######################################################################################
#######################################################################################
#######################################################################################
#######################################################################################


  # ---------------------------------------------------------------------------
  #
  def region_w1(self):
    'integrate dimension w1'
    print self.integrate_region(self.session.selected_spectrum(),self.my_region, 'w2')
  def region_w2(self):
    'integrate dimension w2'
    print self.integrate_region(self.session.selected_spectrum(),self.my_region, 'w1')

  # -----------------------------------------------------------------------------
  #
  def integrate_region(self,spectrum, region, X_axis='w1'):
    '->integrated spectrum along "x" or "y"'

    if spectrum.dimension!=2:
        tkMessageBox.showwarning("Integrate Region","Cannot work with %d dimensions!" % spectrum.dimension)
    
    ppm_per_index = map(lambda w,s: float(w)/s,
                        spectrum.spectrum_width, spectrum.data_size)
    print ppm_per_index

    dims={ 'w1':0,  # numerical order of dimensions in arrays
           'w2':1
           }
    #--- assign dimensions
    primary_dimension=dims[X_axis]
    secondary_dimension= not primary_dimension

#    print spectrum.data_height([120,8])

    
    min_X=region[0][primary_dimension]
    min_Y=region[0][secondary_dimension]

    max_X=region[1][primary_dimension]
    max_Y=region[1][secondary_dimension]
    
    curr_point=[0,0]

    #--- go along main dimension, integrate secondary
    
    integrated=[]
    max_y=0 # just for xmgr scaling

    X_steps=(max_X-min_X)/ppm_per_index[primary_dimension]
    Y_steps=(max_Y-min_Y)/ppm_per_index[secondary_dimension]

    for i in range(X_steps):
      
      curr_X=min_X + ppm_per_index[primary_dimension]*i
      curr_point[primary_dimension]=curr_X
      
      sum=0

      for j in range(Y_steps):

          curr_Y=min_Y + ppm_per_index[secondary_dimension]*j
          curr_point[secondary_dimension]=curr_Y
                     
          height=spectrum.data_height(curr_point)
                     
          sum+=height

      Y_point= sum/Y_steps
      integrated.append([curr_X,Y_point])
                     
      if Y_point>max_y: max_y=Y_point  # just for xmgr scaling



    fname=tkFileDialog.asksaveasfilename()
    if not fname: return

    #--- plain output ----
    file=open(fname,'w')
    for line in integrated:
      file.write("%f\t%f\n" % (line[0], line[1]))
    file.close()

    #-------- make XMGR file to look at -------
    int_dataset=xmgr.Dataset(integrated,str(region))
    int_dataset.solid_line()
    int_dataset.no_sym()
    int_graph=xmgr.Graph("Integrated 2D")
    int_graph.add_dataset(int_dataset)
    int_graph.set_xaxis(X_axis,min_X, max_X)
    int_graph.set_yaxis("Data height",0, max_y)
    int_graph.writeout(fname+".xmgr")
    return integrated

#-------------------------------------------------------------
    
  


def run(session):

  sputil.the_dialog(grab_peaks_dialog,session).show_window(1)

if __name__=="__main__":
  a=cross_peaks("/a")

