doc="""
Utilities to deal with PDB files Version 2.2



Usage:

pdb1=textfile.Text_File(pdbfname1)

array_of_records=[]
for pdb_line in pdb.file:

    #-- obtain record name
    name=pdb_record_name(pdb_line)

    #-- convert to ATOM records to dictionaries indexed by DEFINITION of PDB fields
    record=pdb_atom_record(pdb_line, white_spaces='strip')
    array_of_records.append(record)


#-- if PDB file contains only one molecule:
array_of_records= extract_pdb_data_1molecule(pdb_Text_File,pdbfname)

#-- converts array_of_records from extract_pdb_names() to a hash ResidueNumber->{AtomName->RecordNumber}
    so to be able to extract the line corresponding to the correct atom from array_of_records
    
pdb_dictionary= pdb_to_dictionary(array_of_records)


"""



#############################################################
#  PDB file format reading
##############################################################
pdb_format_doc=""" Vesion 2.2
        ATOM 

        Overview

        The ATOM records present the atomic coordinates for standard residues. They also present the occupancy and temperature factor for each atom. Heterogen coordinates use the HETATM record type. The element symbol is always present on each ATOM record; segment identifier and charge are optional.

        Record Format

        COLUMNS        DATA TYPE       FIELD         DEFINITION
        ---------------------------------------------------------------------------------
         1 -  6        Record name     "ATOM  "

         7 - 11        Integer         serial        Atom serial number.

        13 - 16        Atom            name          Atom name.

        17             Character       altLoc        Alternate location indicator.

        18 - 20        Residue name    resName       Residue name.

        22             Character       chainID       Chain identifier.

        23 - 26        Integer         resSeq        Residue sequence number.

        27             AChar           iCode         Code for insertion of residues.

        31 - 38        Real(8.3)       x             Orthogonal coordinates for X in
                                                     Angstroms.

        39 - 46        Real(8.3)       y             Orthogonal coordinates for Y in
                                                     Angstroms.

        47 - 54        Real(8.3)       z             Orthogonal coordinates for Z in
                                                     Angstroms.

        55 - 60        Real(6.2)       occupancy     Occupancy.

        61 - 66        Real(6.2)       tempFactor    Temperature factor.

        73 - 76        LString(4)      segID         Segment identifier, left-justified.

        77 - 78        LString(2)      element       Element symbol, right-justified.

        79 - 80        LString(2)      charge        Charge on the atom.

         The list of ATOM records in a chain is terminated by a TER record.

        -------------------------------

        Main rule in extraction from this line: if it says
        in PDB manual: 10-20 then your extraction selection is [9:20]"""
    

def pdb_record_name(pdb_line):
    'return the record name from a string out of PDB file'
    return pdb_line[0:6].strip()
    
def pdb_atom_record(pdb_line, white_spaces='strip'):
    """decomposes PDB format v.2.2 ATOM line into parts;
    !!! 
     OPTIONS:
     white_spaces='strip'/'leave'
    'leave' - if you want to compose the line back again.
    """

    if len(pdb_line)<80:
        #--- extend with extra spaces --
        pdb_line+=' '*80

    record_name=pdb_line[0:6].strip()
    #-- check if the record type is correct ---
    if record_name!='ATOM':
        print "ERROR! Instead of 'ATOM' record I am given the following:\n%s" % pdb_line
        sys.exit()
        
    #--- OK keep working ---
    #-- I 'strip()' all of mandatory fields
    record={}

    if white_spaces=='strip':
        record["Atom serial number"]=pdb_line[6:11].strip()
        record["Atom name"]=pdb_line[12:16].strip() 
        record["Alternate location indicator"]=pdb_line[16:17] 
        record["Residue name"]=pdb_line[17:20].strip()
        record["Chain identifier"]=pdb_line[21:22]  
        record["Residue sequence number"]=pdb_line[22:26].strip() 
        record["Code for insertion of residues"]=pdb_line[26:27] 

        record["Orthogonal coordinates for X"]=pdb_line[30:38].strip() 
        record["Orthogonal coordinates for Y"]=pdb_line[38:46].strip() 
        record["Orthogonal coordinates for Z"]=pdb_line[46:54].strip()

    else:
        record["Atom serial number"]=pdb_line[6:11] 
        record["Atom name"]=pdb_line[12:16]  
        record["Alternate location indicator"]=pdb_line[16:17] 
        record["Residue name"]=pdb_line[17:20] 
        record["Chain identifier"]=pdb_line[21:22]  
        record["Residue sequence number"]=pdb_line[22:26]  
        record["Code for insertion of residues"]=pdb_line[26:27] 

        record["Orthogonal coordinates for X"]=pdb_line[30:38]  
        record["Orthogonal coordinates for Y"]=pdb_line[38:46]  
        record["Orthogonal coordinates for Z"]=pdb_line[46:54] 
       
      
    record["Occupancy"]=pdb_line[54:60] 
   
    record["Temperature factor"]=pdb_line[60:66] 
    record["Segment identifier"]=pdb_line[72:76]

    record["Element symbol"]=pdb_line[76:78] 
    record["Charge on the atom"]=pdb_line[78:80]
    
    return record

#======================================================================

def extract_pdb_data_1molecule(pdb_Text_File,pdbfname):
    'Extract ATOM data from the pdb file read as Text_File object (ONLY ONE MOLECULE!!!)'
    names_array=[]
    for line in pdb_Text_File.file:

        if pdb_record_name(line)=='TER':
            #-- this record finished
            break
        
        if pdb_record_name(line)!='ATOM':
            #-- skip other stuff
            continue

        #-- extract ATOM fields
        record=pdb_atom_record(line)
        
        #-- check for presence of alternative conformations (would-be stupid mistake)
        alt_flag=record["Alternate location indicator"]
        if alt_flag!=' ':
            print "Alternative conformations are not allowed! Check '%s' file and remove alternatives!." % pdbfname

        #-- take it
        names_array.append(record)
        
    return names_array

#-- legacy
def extract_pdb_names(pdbfile,pdbfname):
    return extract_pdb_data_1molecule(pdb_Text_File,pdbfname)
#======================================================================


#======================================================================

def pdb_to_dictionary(array_of_records):
    """converts array_of_records from extract_pdb_names() to a hash ResidueNumber->{AtomName->RecordNumber}
    so to be able to extract the line corresponding to the correct atom from array_of_records"""
    pdb_dictionary={}
    #-- make entries for all residues first
    for record in array_of_records:
        residue_number=record["Residue sequence number"]
        if residue_number not in pdb_dictionary:
            pdb_dictionary[residue_number]={}
            
    #--- fill in atom names and their serial numbers in the file     
    for record in array_of_records:
        residue_number=record["Residue sequence number"]
        pdb_dictionary[residue_number][record["Atom name"]]=record["Atom serial number"]

    return pdb_dictionary   
#======================================================================


