"""
Fit 2D Peak

v.4 Working version


Comments on usage:

- process your data using EM in indirect dimension! See how it fits, change -lb parameter
  value accorgingly to make lineshape be closer to lorentzian.

- Don't use too much zero-filling - it makes spectra just look prettier but it doesn't add
  any new information and makes program work slower

- Reducing zero-filling helps to remove this problem (partially):
When fitting peaks completely overlapped in one of dimensions - try to set the range
    narrower. It help Gnuplot converge to reasonable values (probably because difference
    between peak positions is larger then relatively to fit interval)


To do:

- make normalization of Imax_w1 and Imax_w2 so that their ranges are not too different.

- load resulting parameters after fitting is done and save it to the group for easier re-fitting

- Final output is prepared by analysis of .fitpar.txt files to collect all errors as well!

- After fitting 'reality check':

    1. Check that peaks didn't exchange places:
        |w(i,initial)-W(i, final)| < |w(j, initial) -w(i, final)|

    2. Check that fitting placed a peak approximately where you asked to:
        |w(i,initial)-W(i, final)| < 2*linewidth(i,initial)

- Display individual components for visual control (just initial and final in the 1st spectrum)

- Include time parameter in a group : [Enter init value] and [Load from peak list]

- Read initial R from the list [button]

- New button:   [Baseline offset region] - for cases of crowded spectra were no baseline is seen

"""
import sputil
import tkutil, pyutil, re
import tkMessageBox, tkFileDialog, tkSimpleDialog
import popen2,string, copy, pickle, glob

import mylib, keyfile, ek_tkutil
import os
from math import *

#--- Import the module which contains custom time dependence of the signal intensity
import time_evolution_part
reload(time_evolution_part)


#-- Default fontsize for my Sparky projects --
default_font=("Helvetica",10)
larger_font=('Helvetica',11)



# -----------------------------------------------------------------------------
#

#--- GNUPLOT GLOBAL PARAMETERS -------
      
GNUPLOT_CALL="/usr/local/bin/gnuplot"

#--- gnuplot iterations between reporting the progress: don't set to less than 5! Gets stuck!
max_iterations=10

#---- END OF GNUPLOT GLOBAL PARAMETERS ---


#--- Global error diagnostics: return codes --
RETURN_ERROR=0
RETURN_OK=1

#-- global definition of dimensions: following sparky's convention
W1=0  
W2=1

TIMEPOINT_SEPARATION=time_evolution_part.TIMEPOINT_SEPARATION #-- s, Timepoints should differ by more than this value from one another
PPM_BORDER = 15   #-- ppm value separating w1 and w2 ranges

#--- This is the parameter in sparky files set to some value corresponding
#     to experiment you used. 

INDEX_PARAMETER='relaxation_time_parameter'
SCALE_FACTOR='scale_factor'
#--- positions of these in spectrum_times()
SPECTRUM_POS=0
INDEX_PARAMETER_POS=1
SCALE_FACTOR_POS=2



#-------- Peak_Parameters ------------------------------------------------------

class Peak_Parameters:
    'parameters of a single peak in a single spectrum'


    def __init__(self, w1, L_w1, w2, L_w2, Time_coeff):
        'load a peak'
        #-- frequencies --
        self.w1=w1
        self.w2=w2
        #-- linewidths ---
        self.L_w1=L_w1
        self.L_w2=L_w2
        self.assignment=None
        #--- Time-dependence constant -
        self.Time_coeff=Time_coeff
        
    def set_name(self, assignment):
        'set name of the peak'
        self.assignment=assignment 
      

    def __repr__(self):
        return "%s - w1=%.2f (L_w1=%.2f), w2=%.2f (L_w2=%.2f), Time_coeff=%.3f" % (
               self.assignment,
               self.w1,self.L_w1,
               self.w2,self.L_w2,
               self.Time_coeff
               )

    def intensity(self,spectrum, scale_factor=1):
        'find peak intensity without baseline correction'
        height=spectrum.data_height([self.w1,self.w2])*scale_factor
        return height

    #--- check for completeness of object contents ---
    def check_completeness(self):
      error_message=""
      if not self.assignment :
        error_message+="Peak name is not defined!\n"

      #-- any errors?
      if error_message:
         tkMessageBox.showerror("2D Peak Fitting",error_message)
         return RETURN_ERROR
      else:
         return RETURN_OK
 
# -----------------------------------------------------------------------------




#---------- Data_Region ----------------------------------------------------
     
class Data_Region:
    """Region of spectrum.
    Offset calculation for the region must be done
    using separate region because the region picked
    to fit may be not suitable."""

    def __init__(self, sparky_region):

        if not sparky_region:
          tkMessageBox.showerror("2D Peak Fitting","Region '%s' is not valid!" % sparky_region)
      

        self.region=sparky_region

        self.min_w1=sparky_region[0][W1]
        self.min_w2=sparky_region[0][W2]

        self.max_w1=sparky_region[1][W1]
        self.max_w2=sparky_region[1][W2]


    def __repr__(self):
        return "w1 = [ %.2f - %.2f ]     w2 = [ %.2f - %.2f ]" % (self.min_w1, self.max_w1, self.min_w2, self.max_w2)



    #--- INTEGRATED 1D BASELINE OFFSET ESTIMATE ---
    def integrated_baseline_estimate(self,spectrum, X_axis, scale_factor=1):
        'estimate baseline offset as a median of end of integrated 1D. No scaling by number of points'

        if X_axis==W1:
          w_min=self.min_w1
          w_max=self.max_w1
        else:
          w_min=self.min_w2
          w_max=self.max_w2
          
          
        min=self.integrate_one_point(spectrum, X_axis, w_min, scale_factor)
        max=self.integrate_one_point(spectrum, X_axis, w_max, scale_factor)
        
        offset=(max+min)/2

        return offset

    #---- Export just one 2D for plotting purposes ---
    def save_one_spectrum_data(self, fname, spectrum, scale_factor=1 ):
      'save for Gnuplot'

      outp=open(fname,'w')
      outp.write(self.export_data(spectrum,time_parameter="None", scale_factor=scale_factor))
      outp.close()



    #--- Export a series of spectra ----
    def save_time_series(self,fname, spectrum_times):
      'saves a list from spectrum_times (spectrum, time-parameter, scale-factor)'
      outstring=""
      for i in range(len(spectrum_times)):
        
        spectrum=spectrum_times[i][SPECTRUM_POS]
        time_parameter=spectrum_times[i][INDEX_PARAMETER_POS]
        scale_factor=spectrum_times[i][SCALE_FACTOR_POS]

        outstring+=self.export_data(spectrum, time_parameter, scale_factor)

      outstring+="\n\n\n"

      outp=open(fname,'w')
      outp.write(outstring)
      outp.close()
      
        
      
    #--- Export data from a spectrum ----
    def export_data(self,spectrum, time_parameter="None", scale_factor=1):
      'return a text string'

      ppm_per_index = map(lambda w,s: float(w)/s,
                          spectrum.spectrum_width, spectrum.data_size)


      X_steps=(self.max_w1-self.min_w1)/ppm_per_index[W1]
      Y_steps=(self.max_w2-self.min_w2)/ppm_per_index[W2]

      #--- go through the spectrum ---
      string="\n\n# %s \n" % spectrum.name
      for i in range(X_steps):
        
        curr_X=self.min_w1 + ppm_per_index[W1]*i

        string+="\n" #-- separator of 'scans' for gnuplot

        for j in range(Y_steps):

            curr_Y=self.min_w2 + ppm_per_index[W2]*j
                       
            height=spectrum.data_height([curr_X,curr_Y])*scale_factor
                       
            if time_parameter=="None":
                string+="%e\t%e\t%e\n" % (curr_X, curr_Y, height)
            else:
                #--- output a 4D dataset --
                string+="%e\t%e\t%e\t%e\n" % (curr_X, curr_Y, time_parameter, height)
              
      
      return string


    #---- INTEGRATION ALONG ONE OF DIMENSIONS -----  
    def integrate_region(self,spectrum, X_axis, time_parameter, scale_factor):
      '->integrated spectrum along "x" or "y"'

          
        
      ppm_per_index = map(lambda w,s: float(w)/s,
                          spectrum.spectrum_width, spectrum.data_size)
      #--- assign dimensions
      primary_dimension=X_axis
      secondary_dimension= not primary_dimension

     
      curr_point=[0,0]

      #--- go along main dimension, integrate secondary
      
      integrated=""

      #--- one index ---    
      min_X=self.region[0][primary_dimension]
      min_Y=self.region[0][secondary_dimension]

      max_X=self.region[1][primary_dimension]
      max_Y=self.region[1][secondary_dimension]
      
      X_steps=(max_X-min_X)/ppm_per_index[primary_dimension]
      Y_steps=(max_Y-min_Y)/ppm_per_index[secondary_dimension]

      # just for xmgr scaling
      max_y=spectrum.data_height(curr_point)/Y_steps 
      min_y=max_y

      for i in range(X_steps):
        
        curr_X=min_X + ppm_per_index[primary_dimension]*i
        curr_point[primary_dimension]=curr_X

        Y_point= self.integrate_one_point(spectrum, X_axis, curr_X, scale_factor)
        
        integrated+="%e\t%e\t%e\n" % (curr_X,time_parameter,Y_point)
                       
     
      return integrated


   #---- INTEGRATION ALONG ONE OF DIMENSIONS at the fixed frequency of another-----  
    def integrate_one_point(self,spectrum, X_axis, w_fixed, scale_factor):
      '->integrated point along "x" or "y" and normalize it by number of scans'

          
        
      ppm_per_index = map(lambda w,s: float(w)/s,
                          spectrum.spectrum_width, spectrum.data_size)
      #--- assign dimensions
      primary_dimension=X_axis
      secondary_dimension= not primary_dimension

     
      curr_point=[0,0]
      curr_point[X_axis]=w_fixed

      #--- integrate secondary at one point
      

      #--- one index ---    

      min_Y=self.region[0][secondary_dimension]

      max_Y=self.region[1][secondary_dimension]
 
      Y_steps=(max_Y-min_Y)/ppm_per_index[secondary_dimension]

        
      sum=0

      for j in range(Y_steps):

          curr_Y=min_Y + ppm_per_index[secondary_dimension]*j
          curr_point[secondary_dimension]=curr_Y
                     
          height=spectrum.data_height(curr_point)
                     
          sum+=height

      return scale_factor * sum/Y_steps




#--------------------------------------------------------------
    
        
#---------- Peak_Group ----------------------------------------------------

class Peak_Group:
    'group of peaks to be fit together'

    def __init__(self, group_name):
        
        self.group_name=group_name #-- composite name - identifier / string
        self.peaks={} #-- assignment : peak_parameter_set
        self.w1_region=None
        self.w2_region=None  #-- the region to pick a baseline from


    def set_w1_region(self, region):
        self.w1_region=Data_Region(region)

    def set_w2_region(self, region):
        self.w2_region=Data_Region(region)

    def add_peak(self,peak):
        self.peaks[peak.assignment]=peak

    def remove_peak(self,peak):
        if peak.assignment in self.peaks.keys():
          del self.peaks[peak.assignment]

    

    def __repr__(self):
        text= "Group name: '%s'\nW1 Region: %s\nW2 Region: %s\nPeaks: ----\n" % (
            self.group_name, self.w1_region, self.w2_region)

        for peakname in self.peaks.keys():
            text+=str(self.peaks[peakname]) + "\n"
        return text + "----\n"

    #--- check for completeness of object contents ---
    def check_completeness(self):
      error_message=""
      if not self.group_name :
        error_message+="Peak group name is not defined!\n"
      if not self.peaks:
        error_message+="Peaks are not selected!\n"
      if not self.w1_region :
        error_message+="w1 region is not defined!\n"
      if not self.w2_region :
        error_message+="w2 region is not defined!\n"

      #-- any errors?
      if error_message:
         tkMessageBox.showerror("2D Peak Fitting",error_message)
         return RETURN_ERROR
      else:
         return RETURN_OK



    #--- Export a series of integrated spectra along both dimensions separately ----
    def save_integrated_time_series(self,fname, spectrum_times):
      'saves a list of integrated spectra in the order w1 after w2 for each spectrum in spectrum_times'
      if self.check_completeness()==RETURN_ERROR:
        return
      
      #--- for fitting --
      outstring_all=""
      #--- for plotting --
      outstring_W1=""
      outstring_W2=""

      for i in range(len(spectrum_times)):
        
        spectrum=spectrum_times[i][SPECTRUM_POS]

        if spectrum.dimension!=2:
          tkMessageBox.showerror("Fit 2D peak","Need 2D spectrum. '%s' has  %d dimensions!" % (spectrum.name,
                                                                                               spectrum.dimension))
          return RETURN_ERROR

        time_parameter=spectrum_times[i][INDEX_PARAMETER_POS]


        #--- Check that timepoints separated enough to be distinguished in combined dataset ---
        if i>0:
            
            if abs(previous_time_parameter-time_parameter)< TIMEPOINT_SEPARATION:
                tkMessageBox.showerror("Fit 2D peak","""Difference between timepoints '%s' and '%s' is less \
than TIMEPOINT_SEPARATION=%s!\nThe program needs this to differentiate between these two time points. Make them more different of reduce TIMEPOINT_SEPARATION value in 'time_evolution_part.py.
""" % (previous_time_parameter, time_parameter, TIMEPOINT_SEPARATION))
                  
                return RETURN_ERROR
                

        previous_time_parameter=time_parameter
        
        scale_factor=spectrum_times[i][SCALE_FACTOR_POS]


        #--- check if PPM_BORDER condition is satisfied (w1 and w2 regions are not overlapping --
        if not(self.w1_region.min_w1 > PPM_BORDER and self.w1_region.max_w1 > PPM_BORDER and
               self.w1_region.min_w2 < PPM_BORDER and self.w1_region.max_w2 < PPM_BORDER and
               self.w2_region.min_w1 > PPM_BORDER and self.w2_region.max_w1 > PPM_BORDER and
               self.w2_region.min_w2 < PPM_BORDER and self.w2_region.max_w2 < PPM_BORDER):
            tkMessageBox.showerror("Fit 2D peak","PPM_BORDER is not set right! It must be larger than w2 range and \
smaller than w1 range. Current value is PPM_BORDER=%.2f" % PPM_BORDER)
            return
          

        W1_int=self.w1_region.integrate_region(spectrum, W1, time_parameter, scale_factor)
        W2_int=self.w2_region.integrate_region(spectrum, W2, time_parameter, scale_factor)


        #--- for fitting ---        
        outstring_all+= "\n\n# %s\n\n" % spectrum.name + W1_int +  '\n' + W2_int 

        #--- for plotting ---
        outstring_W1+= "\n\n# %s\n\n" % spectrum.name + W1_int 

        #--- for plotting ---        
        outstring_W2+= "\n\n# %s\n\n" % spectrum.name + W2_int 

      outstring_all+="\n\n\n"
      outstring_W1+="\n\n\n"
      outstring_W2+="\n\n\n"

      outp=open(fname+".all",'w')
      outp.write(outstring_all)
      outp.close()

      outp=open(fname+".w1",'w')
      outp.write(outstring_W1)
      outp.close()

      outp=open(fname+".w2",'w')
      outp.write(outstring_W2)
      outp.close()



      
         
#--------------------------------------------------------------
    


#--------------------------------------------------------------









###################################### 2D FITTING DIALOG ###########################




class peak_fitting_dialog(tkutil.Dialog):




  def __init__(self, session):

    self.session = session
    self.drag_notice = None

    #--- new ---
    #self.peak_groups=[]           #--- Analog of a peak list ---
    self.current_peak_group=None  #--- current group ---
    self.current_peak=None

    #-- relax dialog legacy --
    self.spectrum_times = []

    self.gnuplot=None #-- gnuplot I/O pipes
    self.current_spectrum=None

    
    tkutil.Dialog.__init__(self, session.tk, '2D Peak Fitting')

    self.top.columnconfigure(0, weight = 1)

    import Tkinter
    explain = ('Dragging a box defines the region. Buttons assign parameters.')
    w = Tkinter.Label(self.top, text = explain, justify = 'left', font=default_font)
    w.grid(row = 0, column = 0, sticky = 'nw')



    #--- region line ---
    br = ek_tkutil.button_row(self.top,
                           ('Define Peaks Group', self.define_peaks_group),
                           ('Define W1 Region', self.define_w1_region),
                           ('Define W2 Region', self.define_w2_region),
                           ('Enter Time Constant', self.enter_time_constant),
                              #                           ('Test code', self.test_code),
			   )
    br.frame.grid(row = 1, column = 0, sticky = 'nw')



    #--- parameters line ---

    br4 = ek_tkutil.button_row(self.top,
                           ('Take Region for Initial Parameters', self.estimate_parameters),
                           ('Set Peak Label', self.set_label),
                            
                           ('Add Peak to Group', self.add_to_group),
                           ('Undo Last', self.undo_add),
                           ('Reinitialize', self.reinitialize),
			   )
    br4.frame.grid(row = 2, column = 0, sticky = 'nw')
    

    #-- main info ---
    r = Tkinter.Label(self.top, justify = 'left', font=larger_font)
    r.grid(row = 3, column = 0, sticky = 'nw')
    self.result_label = r


    #-- service buttons ---
    br3 = ek_tkutil.button_row(self.top,
                           ('Load Group', self.load_group),
                           ('Save/Update Group', self.save_group),
                           ('Display Group', self.show_group),
			   )
    br3.frame.grid(row = 4, column = 0, sticky = 'nw')

    #--- gnuplot line ---
    br2 = ek_tkutil.button_row(self.top,
                           ('Select Spectra to Fit', self.setup_spectrum_list),
                           ('Save Fitting Project', self.save_fitting_project),
                           ('Show Data and Functions', self.show_fit_results),
                           ('Fit', self.fit),
                           ('Read Fitting Results', self.read_parameters),
                              
        		   )
    br2.frame.grid(row = 5, column = 0, sticky = 'nw')
    #-- service buttons ---
    br3 = ek_tkutil.button_row(self.top,
                           ('Close', self.close_cb),
                           ('Help', self.help),
			   )
    br3.frame.grid(row = 6, column = 0, sticky = 'nw')




    self.register_callback(1)
    self.top.bind('<Map>', lambda e, s=self: s.register_callback(1))
    self.top.bind('<Unmap>', lambda e, s=self: s.register_callback(0))


  # ---------------------------------------------------------------------------
  #
  def register_callback(self, register):

    if register:
      if self.drag_notice == None:
        self.drag_notice = self.session.notify_me('drag region',
                                                  self.region_selected_cb)
    else:
      if self.drag_notice:
        self.session.dont_notify_me(self.drag_notice)
        self.drag_notice = None

  # ---------------------------------------------------------------------------
  #
  def region_selected_cb(self, region):

    if self.is_window_destroyed():
      self.register_callback(0)
      return

    #--ek
    self.current_region=region

    self.refresh_info("Selected region: %s" % Data_Region(region))
    

  #----- REFRESH INFO --------
    
  def refresh_info(self,additional_text=""):
    'Refresh message in the main window'

    self.current_spectrum = self.session.selected_spectrum()

    msg="Current time constant: %.3f\n" % time_evolution_part.Time_coefficient_init
    
    
    if self.current_peak_group:
      msg+=str(self.current_peak_group)+'\n'

    if self.current_peak:
      msg+=str(self.current_peak)+'\n'

    self.result_label['text'] = msg + additional_text +'\n'
    
    
  #-------- menu:  My help function ----------
  def help(self):
    tkMessageBox.showinfo("2D Peak Fitting",
    """Written by Evgenii Kovrigin 05/09/05
(1) Define data region: drag the box around the peak, make sure peak gets selected.
(2) Hit 'Take as Region to Fit'.
(3) Estimate initial peak position and the linewidth: drag a smaller box. Its width and height will be taken as estimate of the line widths.
    Middle of the box is taken for initial peak position.
(4) Hit 'Take for Initial Parameters' to save.
Repeating (3),(4) adds another peak estimate for overlapped fitting. To remove all hit 'Reinitialize'.



Important to remember:

  - Before using this extension: run 'cv' and click on 'Center on Peak' to allow proper display of selected regions!

  - I assume that your times are separated by more than %d s from each other. It is done to allow Gnuplot distinguish
    data coming from different spectra which need separate baseline offset corrections. This number is controlled by
    TIMEPOINT_SEPARATION constant.

  - offsets in orthogonal integrated spectra a fitted independently though they are not strictly independent.
    Think of adding one more constraint for fitting.

  - fit function discriminate two dimensions by using PPM_BORDER=%f value. x less than that belongs to one integrated
  trace and x larger than that belongs to another trace. In case of overlapping ppm ranges of w1 and w2 you have to
  shift ppm range in one of dimensions to satisfy this condition.


""" % TIMEPOINT_SEPARATION, PPM_BORDER)
    

  def reinitialize(self):
    self.current_peak_group=None
    self.current_peak=None
    
    self.result_label['text'] = "Parameters reset. Pick new."


   # --------------menu: DEFINE PEAKS GROUP ---------------------------------------------------------------
  #
  def define_peaks_group(self):
    'define group of peaks'    

    if not  self.session.selected_peaks(): 
        tkMessageBox.showerror("2D Peak Fitting","\n\n!!! Error!  Switch to selection mode (F1)and have at least one peak in a selected region !!!\n\n")
        return

      
    #-- form a composite name for a group
    peaks=""
    for peak in self.session.selected_peaks():
      peaks+=peak.assignment[:-3]+"_"
    group_name=peaks[:-1]

    self.current_peak_group= Peak_Group(group_name)
    self.refresh_info()
 # --------------menu: DEFINE W1 FITTING REGION ---------------------------------------------------------------
  #
  def define_w1_region(self):
    'define w1 region'    

    #-- check if there was any assignment selected to form a peak name
    if not self.current_region: 
        tkMessageBox.showerror("2D Peak Fitting","\n\n!!! Select the region first with the mouse !!!\n\n")
        return

    
    if not  self.session.selected_peaks(): 
        tkMessageBox.showerror("2D Peak Fitting","\n\n!!! Error!  Switch to selection mode (F1)and have at least one peak in a selected region !!!\n\n")
        return

      
    self.current_peak_group.set_w1_region(self.current_region)
    self.refresh_info()
    
 # --------------menu: DEFINE W2 FITTING REGION ---------------------------------------------------------------
  #
  def define_w2_region(self):
    'define w2 region'    

    #-- check if there was any assignment selected to form a peak name
    if not self.current_region: 
        tkMessageBox.showerror("2D Peak Fitting","\n\n!!! Select the region first with the mouse !!!\n\n")
        return

    
    if not  self.session.selected_peaks(): 
        tkMessageBox.showerror("2D Peak Fitting","\n\n!!! Error!  Switch to selection mode (F1)and have at least one peak in a selected region !!!\n\n")
        return

      
    self.current_peak_group.set_w2_region(self.current_region)
    self.refresh_info()




  # ------- menu: estimate parameters for fitting --------------------------------------------------------
  #
  
  def estimate_parameters(self):
    'estimate linewidth and peak position from the rectangle'


    if not self.current_peak_group: 
        tkMessageBox.showerror("2D Peak Fitting","\n\n!!! Select the fit region first!!!\n\n")
        return

    est_region=Data_Region(self.current_region)
    


    peak_w1=(est_region.max_w1+est_region.min_w1)/2
    peak_w2=(est_region.max_w2+est_region.min_w2)/2

    lw_w1=est_region.max_w1-est_region.min_w1
    lw_w2=est_region.max_w2-est_region.min_w2

    self.current_peak=Peak_Parameters(peak_w1,lw_w1,peak_w2,lw_w2,
                                      time_evolution_part.Time_coefficient_init)

    self.refresh_info() #--- refreshes the main window information 
    


  #--- Manual entering of the default value of a  time constant 
  def enter_time_constant(self):
      'Enter time dependence constant'
      value=tkSimpleDialog.askfloat("2D Peak Fitting",
            "Current default Time_coeff= %.3f\nEnter new value or hit [Cancel] to keep current one." % (
                time_evolution_part.Time_coefficient_init))

      if value:       time_evolution_part.Time_coefficient_init=float(value)
      self.refresh_info() #--- refreshes the main window information 

      

  #----- menu: set peak label ----
  def set_label(self):
    'save selected peak name as a label'
    peaklist=self.session.selected_peaks()
    if len(peaklist)>1:
        tkMessageBox.showerror("2D Peak Fitting","\n\n Select just one peak for the label!\n\n")
        return

    name=peaklist[0].assignment

    #--- assign to a last parameter set
    if not self.current_peak: 
        tkMessageBox.showerror("2D Peak Fitting","\n\nTake the region to estimate parameters first!\n\n")
        return

    self.current_peak.set_name(name)
    self.refresh_info()

  #-------- menu: add peak to peak group -----
  def add_to_group(self):
    'add current peak to a current group'
    if not self.current_peak_group: 
        tkMessageBox.showerror("2D Peak Fitting","\n\n!!! Select the fit region first!!!\n\n")
        return
    if not self.current_peak: 
        tkMessageBox.showerror("2D Peak Fitting","\n\n!!! Take peak parameters and label first!!!\n\n")
        return

    if self.current_peak.check_completeness()==RETURN_ERROR:
      return
    self.current_peak_group.add_peak(self.current_peak)
    self.refresh_info()
    

  #-------- menu: undo adding last peak  -----

  def undo_add(self):
    'undo peak addition to a group'
    if not self.current_peak_group: 
        tkMessageBox.showerror("2D Peak Fitting","\n\n!!! Select the fit region first!!!\n\n")
        return
    if not self.current_peak: 
        tkMessageBox.showerror("2D Peak Fitting","\n\n!!! Take peak parameters and label first!!!\n\n")
        return
      
    self.current_peak_group.remove_peak(self.current_peak)
    self.refresh_info()


  #---- menu: saving group information ----
  def save_group(self):
    'save with the group name as a pickle object'

    if  not self.current_peak_group :
        tkMessageBox.showerror("2D Peak Fitting","\n\nGroup of peaks is not defined!\n\n")
        return
    if self.current_peak_group.check_completeness()==RETURN_ERROR:
      return

    fname=self.current_peak_group.group_name+'.group'
    outfile=open(fname,'w')
    outfile.write(pickle.dumps(self.current_peak_group))
    outfile.close
    self.current_peak=None
    self.refresh_info("File '%s' is written." % fname)

  #---- menu: loading group information ----
  def load_group(self):
    'load the group from a pickle object'

    infile=tkFileDialog.askopenfile(filetypes=[("Saved Peak Groups","*group")])

    if not infile: return

    string=infile.read()
    self.current_peak_group=pickle.loads(string)
    infile.close()
    self.current_peak=None
    self.refresh_info()

  #--- menu: show group ---
  def show_group(self):
    'display group'

    if  not self.current_peak_group :
        tkMessageBox.showerror("2D Peak Fitting","\n\nGroup of peaks is not defined!\n\n")
        return
    if self.current_peak_group.check_completeness()==RETURN_ERROR:
      return

    self.refresh_info()
  
    #-- show Gnuplot figure ---
    temp_w1_2D_fname='temp_show_group_w1_2D'
    temp_w2_2D_fname='temp_show_group_w2_2D'
    temp_w1_1D_fname='temp_show_group_w1_1D'
    temp_w2_1D_fname='temp_show_group_w2_1D'
    
    self.current_peak_group.w1_region.save_one_spectrum_data(temp_w1_2D_fname, self.current_spectrum)
    self.current_peak_group.w2_region.save_one_spectrum_data(temp_w2_2D_fname, self.current_spectrum)

    self.current_peak_group.save_integrated_time_series(temp_w1_1D_fname, ((self.current_spectrum,0,1),))
    self.current_peak_group.save_integrated_time_series(temp_w2_1D_fname, ((self.current_spectrum,0,1),))
    


  
    plotting=("""
set term x11 
set multiplot
unset xtics
unset ytics
unset ztics
unset key
unset colorbox
set bmargin 0
set lmargin 0
set rmargin 0



#--- 1Ds ---
set xrange [*:*] reverse
set origin 0.6,0.6
set size 0.4, 0.4
set xzeroaxis lt -1
set xlabel 'w2'
set title 'Integration of W1 dimension'
plot '%s.w2' using 1:3 with lines
""" % temp_w2_1D_fname +
"""set origin 0.5,0
set size 0.6, 0.6
set view 0, 270
set xlabel 'w1'
set title 'Integration of W2 dimension'
splot '%s.w1' using 1:3:2 with lines
unset xlabel
unset title
""" % temp_w1_1D_fname +
"""
set pm3d map
set view 0, 90
set yrange [*:*] reverse
set xrange [*:*] noreverse

set origin 0,0.5
set title '%s'
set size 0.5,0.5
set ylabel 'w2'
splot '%s' using 1:2:3 with pm3d
unset ylabel
unset title
""" % (self.current_peak_group.group_name, temp_w2_2D_fname) +
              
"""
set origin 0,0
set size 0.5,0.5
set xlabel 'w1'
splot '%s' using 1:2:3 with pm3d
unset multiplot
""" %  temp_w1_2D_fname) 
              

    gnuplot_command=plotting
    
    self.plot(gnuplot_command)
    
    #--- remove temporary files
    temp_files=glob.glob('temp_show_group*')
    for file in temp_files:  os.remove(file)
    
    
    
    self.refresh_info()
      


  #------- GENERAL PLOTTING WITH PAUSE UNTIL 'OK' IS PRESSED -----
  #
  def plot(self, gnuplot_command):
    'plotting with Gnuplot, press OK to exit'

    gnuplot_command+="\n"
    finish="\nexit\n"

    print gnuplot_command
    self.gnuplot=popen2.Popen4(GNUPLOT_CALL+'\n',0)
    self.gnuplot.tochild.write(gnuplot_command)
    tkMessageBox.showinfo("2D Peak Fitting","Press OK to continue.")
    self.gnuplot.tochild.write(finish)
    gnuplot_output=self.gnuplot.fromchild.readlines() #-- can only be executed after gnuplot exits
    self.gnuplot=None
    print "--- Gnuplot messages ----"
    for line in gnuplot_output:
        print line
    print "--- End of Gnuplot messages ----"

    
     

















  #----- SAVE FITTING PROJECT ----
  def save_fitting_project(self):
    'save start.par, spectra.dat and equations.txt'

    if not  self.current_peak_group: 
        tkMessageBox.showerror("2D Peak Fitting","\n\nPeak group is not defined!\n\n")
        return

    if  self.current_peak_group.check_completeness()==RETURN_ERROR:
        return

    if not  self.spectrum_times: 
        tkMessageBox.showerror("2D Peak Fitting","\n\nSelect spectra by 'Setup' first!\n\n")
        return


    parameters,equations=self.generate_parameters_v1(self.current_peak_group, self.spectrum_times)

    #--- Save parameters ---
    par_fname=self.current_peak_group.group_name+".fitpar.txt"
    par_file=open(par_fname,'w')
    par_file.write(parameters)
    par_file.close()

    #--- Save equations ---
    eq_fname=self.current_peak_group.group_name+".equations.txt"
    eq_file=open(eq_fname,'w')
    eq_file.write(equations)
    eq_file.close()
    
    #--- Save spectral data ---
    sp_fname=self.current_peak_group.group_name+".spectra.dat"
    if self.current_peak_group.save_integrated_time_series(sp_fname,self.spectrum_times)==RETURN_ERROR:
      self.refresh_info("Remove spectrum from the project!") #--- refreshes the main window information 
      return


    #-- Refresh window --   
#    self.refresh_info("Files '%s', '%s' and '%s' written." % (par_fname,eq_fname, sp_fname)) #--- refreshes the main window information 
    self.refresh_info("Files written.")
        


  #-- MAIN GENERATOR OF EQUATIONS AND STARTING PARAMETERS ---
    
  def generate_parameters_v1(self, current_peak_group, spectrum_times):
    """Signal intensity evolution with same linewidths and peak positions.

    Plan:
    - Each group has a few peaks labelled as p1,p2,p3, they have corresponding
      frequencies x0_w1,y0_w2 and linewidths Lw1,Lw2 (constant)

    - In a parameter set all frequencies and the linewidths are expressed relative
      to a first peak parameters - it helps stabilize fitting process

    - Peak intensity Imax changes but Imax value is measured in the spectrum with minimal
      value of time-parameter (this is the first spectrum, thanks to sorting
      procedure by Tom Goddard).

    - 


  
    """


    if not  current_peak_group: 
        tkMessageBox.showerror("2D Peak Fitting","\n\nPeak group is not defined!\n\n")
        return

    if  current_peak_group.check_completeness()==RETURN_ERROR:
        return

    if not  spectrum_times: 
        tkMessageBox.showerror("2D Peak Fitting","\n\nSelect spectra by 'Setup' first!\n\n")
        return

    
    parameters=""
    peaks= current_peak_group.peaks.keys()
    first_spectrum= spectrum_times[0][SPECTRUM_POS]
    first_spectrum_scale_factor= spectrum_times[0][SCALE_FACTOR_POS]
    first_spectrum_offset_w1= current_peak_group.w1_region.integrated_baseline_estimate(
                                                                    first_spectrum, W1,
                                                                    first_spectrum_scale_factor)
    first_spectrum_offset_w2=current_peak_group.w1_region.integrated_baseline_estimate(
                                                                    first_spectrum, W2,
                                                                    first_spectrum_scale_factor)

 
    #--- Add parameters for each peak ---
    for i in range(len(peaks)):

        #--- show peak name ---
        parameters+="""
        #peak_%d : %s
        """ % (i, current_peak_group.peaks[peaks[i]].assignment)

        w1=current_peak_group.peaks[peaks[i]].w1
        w2=current_peak_group.peaks[peaks[i]].w2
        L_w1=current_peak_group.peaks[peaks[i]].L_w1
        L_w2=current_peak_group.peaks[peaks[i]].L_w2
 
        
        
        data_height_w1=current_peak_group.w1_region.integrate_one_point(
                                            spectrum=first_spectrum,
                                            X_axis=W1,
                                            w_fixed=w1,
                                            scale_factor=first_spectrum_scale_factor)
        
        data_height_w2=current_peak_group.w2_region.integrate_one_point(
                                            spectrum=first_spectrum,
                                            X_axis=W2,
                                            w_fixed=w2,
                                            scale_factor=first_spectrum_scale_factor)
      
        peak_intensity_w1 = data_height_w1-first_spectrum_offset_w1
        peak_intensity_w2 = data_height_w2-first_spectrum_offset_w2

        
        

        if i==0: #--- This is the first peak ---
            parameters+="""
                   w10_p0 = %.9e
                   w20_p0 = %.9e
                   Lw1_p0 = %.9e
                   Lw2_p0 = %.9e
                   Imax_w1_p0 = %.9e
                   Imax_w2_p0 = %.9e
                   """ % (w1,
                          w2,
                          L_w1,
                          L_w2,
                          peak_intensity_w1,peak_intensity_w2
                          )
            
            first_peak_w1=w1                               
            first_peak_w2=w2                               
            first_peak_Lw1=L_w1                               
            first_peak_Lw2=L_w2
            first_peak_intensity_w1=peak_intensity_w1
            first_peak_intensity_w2= peak_intensity_w2

        else:
            #--- 2nd, 3rd ... peaks will have incremental parameters
            parameters+="""
                   dw10_p%d = %.9e
                   dw20_p%d = %.9e
                   dLw1_p%d = %.9e
                   dLw2_p%d = %.9e
                   dImax_w1_p%d = %.9e
                   dImax_w2_p%d = %.9e
                   """ % (i, w1-first_peak_w1,
                          i, w2-first_peak_w2,
                          i, L_w1-first_peak_Lw1,
                          i, L_w2-first_peak_Lw2,
                          i, peak_intensity_w1-first_peak_intensity_w1,
                          i, peak_intensity_w2-first_peak_intensity_w2
                          )
            
            
                


    #-- Add offsets for each spectrum ---

    for i in range(len( spectrum_times)):
      spectrum= spectrum_times[i][SPECTRUM_POS]
      scale_factor= spectrum_times[i][SCALE_FACTOR_POS]
      offset_w1 = current_peak_group.w1_region.integrated_baseline_estimate(spectrum, W1, scale_factor)
      offset_w2 = current_peak_group.w2_region.integrated_baseline_estimate(spectrum, W2, scale_factor)

      parameters+=("""
                  offset_w1_s%d = %.9e
                  offset_w2_s%d = %.9e
                  """ % (i, offset_w1, i, offset_w2)
                   )



    # --- Introducing time dependence of intensity ---

    #-- Fixation of time dependent parameter in case of one spectrum (as Gnuplot requires)
    #   This parameter will not be varied in fitting process then. --
    fixed=""
    
    #-- for the first peak --
    if len( spectrum_times)==1:
        fixed="# FIXED"
        time_constant=time_evolution_part.Time_coefficient_zero
    else:
        time_constant=current_peak_group.peaks[peaks[0]].Time_coeff
    
    parameters+=("""
                Time_coeff_p0 = %.9e %s
              """ % (time_constant,fixed)
               )
         

    #-- for other peaks peak --
   
    for i in range(1, len(peaks)):
              
        if len( spectrum_times)==1:
            fixed="# FIXED"
            time_constant=time_evolution_part.Time_coefficient_zero
        else:
            time_constant=current_peak_group.peaks[peaks[i]].Time_coeff
        
        parameters+=("""
                Time_coeff_p%d = %.9e %s
                  """ % (i,time_constant,fixed)
                   )
        
     

    #------- Generate equations ------------
    function_head="""
                  Y(x,y)="""
        

    #-- Generate baseline offset equations ---
    N=len(spectrum_times)  
    if N==1:
        offset_equations="""
                        Baseline_w1_s0(x)= offset_w1_s0 
                        Baseline_w2_s0(x)= offset_w2_s0
                        """
    else:
        offset_equations=""

    for i in range(N-1):
      
      time=spectrum_times[i][INDEX_PARAMETER_POS]

      if i<N-2:
        offset_equations="""
                        Baseline_w1_s%d(x)=( x>%.9e )?  Baseline_w1_s%d(x) : offset_w1_s%d 
                        Baseline_w2_s%d(x)=( x>%.9e )?  Baseline_w2_s%d(x) : offset_w2_s%d
                        """% (i,time+TIMEPOINT_SEPARATION,i+1,i,
                              i,time+TIMEPOINT_SEPARATION,i+1,i) + offset_equations
      else:
        offset_equations="""
                        Baseline_w1_s%d(x)=( x>%.9e )?  offset_w1_s%d : offset_w1_s%d 
                        Baseline_w2_s%d(x)=( x>%.9e )?  offset_w2_s%d : offset_w2_s%d
                        """% (i,time+TIMEPOINT_SEPARATION,i+1,i,
                              i,time+TIMEPOINT_SEPARATION,i+1,i) + offset_equations


       



    equations=""
    function_body=""
    
    for i in range(len(peaks)):

        #--- One-dimensional lorentzians ---
        time_evolution=time_evolution_part.Time_evolution_equation % i

        if i==0:
            #--- First peak only ---
            equations+="""
            
            W1_p%d(x,y) = Imax_w1_p%d * %s * Lw1_p%d**2 / ( Lw1_p%d**2 + (w10_p%d - x)**2 ) + Baseline_w1_s0(y)
            W2_p%d(x,y) = Imax_w2_p%d * %s * Lw2_p%d**2 / ( Lw2_p%d**2 + (w20_p%d - x)**2 ) + Baseline_w2_s0(y)

            W_p%d(x,y) = (x>%e)?W1_p%d(x,y):W2_p%d(x,y)
                  
                  """ % (i,i, time_evolution,   i,i,i,
                         i,i, time_evolution,   i,i,i,
                         i,PPM_BORDER,i,i)


            #--- Two-dimensional lorentzian --
            function_body+=" W_p%d(x,y) +" % i  

        else:
            #--- All subsequent peaks ---
            equations+="""
            
            W1_p%d(x,y) = (Imax_w1_p0+dImax_w1_p%d) * %s *\
                            (Lw1_p0+dLw1_p%d)**2 / ( (Lw1_p0+dLw1_p%d)**2 + (w10_p0+dw10_p%d - x)**2 )\
                            + Baseline_w1_s0(y)
            
            W2_p%d(x,y) = (Imax_w2_p0+dImax_w2_p%d) * %s *\
                            (Lw2_p0+dLw2_p%d)**2 / ( (Lw2_p0+dLw2_p%d)**2 + (w20_p0+dw20_p%d - x)**2 )\
                            + Baseline_w2_s0(y)

            W_p%d(x,y) = (x>%e)?W1_p%d(x,y):W2_p%d(x,y)
                  
                  """ % (i,i, time_evolution,   i,i,i,
                         i,i, time_evolution,   i,i,i,
                         i,PPM_BORDER,i,i)


            #--- Two-dimensional lorentzian --
            function_body+=" W_p%d(x,y) +" % i  



    final_equations= offset_equations + equations + "\n\n" + function_head+function_body[:-1] + "\n\n"
      

    #--- return results
    
    return parameters, final_equations









  #-- READING PARAMETERS FROM FITTING RESULTS AND UPDATING THE GROUP ---
    
  def read_parameters(self):


    if not  self.current_peak_group: 
        tkMessageBox.showerror("2D Peak Fitting","\n\nPeak group is not defined!\n\n")
        return

    if  self.current_peak_group.check_completeness()==RETURN_ERROR:
        return


    par_fname=self.current_peak_group.group_name+".fitpar.txt"

    if not os.path.isfile(par_fname) :
        tkMessageBox.showerror("2D Peak Fitting","\n\nSave your project first!\n\n")
        return
        

    par_file=keyfile.Ini_File(par_fname)


    peaks= self.current_peak_group.peaks.keys()

 
    #--- Read parameters for the first peak ---
    self.current_peak_group.peaks[peaks[0]].w1=float(par_file.get_all_line("w10_p0")[2])
    self.current_peak_group.peaks[peaks[0]].w2=float(par_file.get_all_line("w20_p0")[2])
    self.current_peak_group.peaks[peaks[0]].L_w1=float(par_file.get_all_line("Lw1_p0")[2])
    self.current_peak_group.peaks[peaks[0]].L_w2=float(par_file.get_all_line("Lw2_p0")[2])
    self.current_peak_group.peaks[peaks[0]].Time_coeff=float(par_file.get_all_line("Time_coeff_p0")[2])




    
    for i in range(1,len(peaks)):

        dw1=float(par_file.get_all_line("dw10_p%d"%i)[2])
        dw2=float(par_file.get_all_line("dw20_p%d"%i)[2])
        dL_w1=float(par_file.get_all_line("dLw1_p%d"%i)[2])
        dL_w2=float(par_file.get_all_line("dLw2_p%d"%i)[2])


        self.current_peak_group.peaks[peaks[i]].w1=dw1+self.current_peak_group.peaks[peaks[0]].w1
        self.current_peak_group.peaks[peaks[i]].w2=dw2+self.current_peak_group.peaks[peaks[0]].w2
        self.current_peak_group.peaks[peaks[i]].L_w1=dL_w1+self.current_peak_group.peaks[peaks[0]].L_w1
        self.current_peak_group.peaks[peaks[i]].L_w2=dL_w2+self.current_peak_group.peaks[peaks[0]].L_w2
        self.current_peak_group.peaks[peaks[i]].Time_coeff=float(par_file.get_all_line("Time_coeff_p%d"%i)[2])


    self.refresh_info("Fitting results loaded.")    

        



                

    #--------- DISPLAY OF INTEGRATED SPECTRA WITH CORRESPONDING FITTING RESULTS --

  def show_fit_results(self):
    'displays integrated data as well as functions'

    if not  self.current_peak_group: 
        tkMessageBox.showerror("2D Peak Fitting","\n\nPeak group is not defined!\n\n")
        return

    if  self.current_peak_group.check_completeness()==RETURN_ERROR:
        return

    if not  self.spectrum_times: 
        tkMessageBox.showerror("2D Peak Fitting","\n\nSelect spectra by 'Setup' first!\n\n")
        return

    group_name=self.current_peak_group.group_name
    parfile=group_name+'.fitpar.txt'
    eqfile=group_name+'.equations.txt'

    if ( not os.path.isfile(parfile) or
         not  os.path.isfile(eqfile)):

        tkMessageBox.showerror("2D Peak Fitting","\n\nSave Fitting Project first!\n\n")
        return
        

    text_0_a="""
    set term x11
    set multiplot
    """
    text_0_b="""
    set key off
    set title '%s group'
    set xrange [*:*] reverse
    """ % self.current_peak_group.group_name



    parameters_text,equations_text=self.generate_parameters_v1(self.current_peak_group, self.spectrum_times)


    text_1="""
    set size 0.5,0.5
    set origin 0,0.5
    """

    #---- Plot data and fitting curves ---


    text_peaks=""
    plot_peaks=""
    N=len(self.spectrum_times)
    for i in range(N):
        time=self.spectrum_times[i][INDEX_PARAMETER_POS]

        text_peaks+="""
            combined_%d(x)=Y(x,%s)
            """ % (i,time)
        
        if i==0: add=" title 'fit' "
        else: add=''
        
        plot_peaks+=" combined_%d(x) %s with lines lt 1," % (i,add)

        


    

    text_2="""
    set xlabel 'w2'
    set xtics 0.1
    plot '%s.spectra.dat.w2'using 1:3 title 'exp' with points lt -1  pt 7 ps 1, %s\
    """ % (group_name, plot_peaks[:-1]) #+ " title ' fit'"

    text_3="""
    unset title
    set origin 0,0
    set xlabel 'w1'
    set xtics 1
    plot '%s.spectra.dat.w1'using 1:3 with points lt -1  pt 7 ps 1, %s
    """ % (group_name, plot_peaks[:-1])




    #--- Deconvolution plots ---
    first_time=self.spectrum_times[0][INDEX_PARAMETER_POS]

    text_all_separate=""
    plot_all_separate=""
    peak_list=
    for i in range(len(self.current_peak_group.peaks)):

        text_all_separate+="""
            p%d(x)=W_p%d(x,%s)
            """ % (i,i, first_time)
        plot_all_separate+=" p%d(x) title '%d' with lines lt %d ," % (i,i,i+2)



    text_4=("""
    set title 'Overlapped peak deconvolution'
    set key on
    set xlabel 'w2'
    set xtics 0.1
    set origin 0.5, 0.5
    set xrange [%f:%f] reverse
    plot combined_0(x)  title 'sum' with lines lt 1  , """ % (
        self.current_peak_group.w2_region.min_w2, self.current_peak_group.w2_region.max_w2)
        + plot_all_separate[:-1]
        )
    
    text_4+=("""
    unset title
    set xlabel 'w1'
    set xtics 1
    set origin 0.5, 0
    set xrange [%f:%f] reverse

    plot combined_0(x)  title 'sum' with lines lt 1  , """ % (
        self.current_peak_group.w1_region.min_w1, self.current_peak_group.w1_region.max_w1)
        + plot_all_separate[:-1]
        )

    text_last="""
    unset multiplot
    """

    gnuplot_command= (text_0_a + text_0_b + parameters_text + equations_text + text_1 + text_peaks +
                      text_2 + text_3 + text_all_separate + text_4 + text_last)
    print gnuplot_command
    self.plot(gnuplot_command)


    #---- Now we create postscript files (copy pieces from above) ----
    #----------------------------------------------------------------

    
    #--- common parts --
    postscript_text="""
    set term postscript 
    set output '%s.eps'
    """
    exit="\nexit\n\n"


    text1="""
    set key on
    set title '%s group'
    set xrange [*:*] reverse
    """ % self.current_peak_group.group_name



    #---- Plot data and fitting curves ---

    text_peaks=""
    plot_peaks=""
    N=len(self.spectrum_times)
    for i in range(N):
        time=self.spectrum_times[i][INDEX_PARAMETER_POS]

        text_peaks+="""
            combined_%d(x)=Y(x,%s)
            """ % (i,time)
        
        if i==0: add=" title 'fit' "
        else: add=" title '' "
        
        plot_peaks+=" combined_%d(x) %s with lines lt -1 ," % (i,add)

    text2="""
    set xlabel 'w2'
    set xtics 0.1
    plot '%s.spectra.dat.w2'using 1:3 title 'exp' with points lt -1  pt 7 ps 1, %s\
    """ % (group_name, plot_peaks[:-1]) #+ " title ' fit'"


    #---- PLOT ---
    gnuplot_command=(postscript_text % (group_name+".w2") + parameters_text + equations_text +
                    text_peaks + text_all_separate + text1 + text2 + exit)
    
    self.gnuplot=popen2.Popen4(GNUPLOT_CALL+'\n',0)
    self.gnuplot.tochild.write(gnuplot_command)



    text3="""
    set xlabel 'w1'
    set xtics 1
    plot '%s.spectra.dat.w1'using 1:3  title 'exp' with points lt -1  pt 7 ps 1, %s
    """ % (group_name, plot_peaks[:-1])


    #---- PLOT ---
    gnuplot_command=(postscript_text % (group_name+".w1") + parameters_text + equations_text +
                    text_peaks + text_all_separate + text1 + text3 + exit)
    
    self.gnuplot=popen2.Popen4(GNUPLOT_CALL+'\n',0)
    self.gnuplot.tochild.write(gnuplot_command)



    #--- Deconvolution plots ---
    first_time=self.spectrum_times[0][INDEX_PARAMETER_POS]

    text_all_separate=""
    plot_all_separate=""
    for i in range(len(self.current_peak_group.peaks)):

        text_all_separate+="""
            p%d(x)=W_p%d(x,%s)
            """ % (i,i, first_time)
        plot_all_separate+=" p%d(x) title '%d' with lines lt %d ," % (i,i,i+2)



    text4=("""
    set title 'Overlapped peak deconvolution'
    set xlabel 'w2'
    set xtics 0.1
    set xrange [%f:%f] reverse
    plot combined_0(x)  title 'sum' with lines lt 1  , """ % (
    self.current_peak_group.w2_region.min_w2, self.current_peak_group.w2_region.max_w2)
    + plot_all_separate[:-1]
    )

    #---- PLOT ---
    gnuplot_command=(postscript_text % (group_name+"deconvolution.w2") + parameters_text + equations_text +
                    text_peaks + text_all_separate + text1 + text4 + exit)
    
    self.gnuplot=popen2.Popen4(GNUPLOT_CALL+'\n',0)
    self.gnuplot.tochild.write(gnuplot_command)
    

    text5=("""
    set title 'Overlapped peak deconvolution'
    set xlabel 'w1'
    set xtics 1
    set xrange [%f:%f] reverse

    plot combined_0(x)  title 'sum' with lines lt 1  , """ % (
    self.current_peak_group.w1_region.min_w1, self.current_peak_group.w1_region.max_w1)
    + plot_all_separate[:-1]
    )

    #---- PLOT ---
    gnuplot_command=(postscript_text % (group_name+"deconvolution.w1") + parameters_text + equations_text +
                    text_peaks + text_all_separate + text1 + text5 + exit)
    
    self.gnuplot=popen2.Popen4(GNUPLOT_CALL+'\n',0)
    self.gnuplot.tochild.write(gnuplot_command)
  




  #----------------- MAIN FITTING PROCEDURE ------------------------------------
  #
  def fitting(self):
    'general fitting procedure'

    group_name=self.current_peak_group.group_name
    parfile=group_name+'.fitpar.txt'
    eqfile=group_name+'.equations.txt'
    datafile= group_name+'.spectra.dat.all'


    if not  self.current_peak_group: 
        tkMessageBox.showerror("2D Peak Fitting","\n\nPeak group is not defined!\n\n")
        return RETURN_ERROR

    if  self.current_peak_group.check_completeness()==RETURN_ERROR:
        return

    if not  self.spectrum_times: 
        tkMessageBox.showerror("2D Peak Fitting","\n\nSelect spectra by 'Setup' first!\n\n")
        return  RETURN_ERROR

    
    if ( not os.path.isfile(parfile) or
         not  os.path.isfile(eqfile)):

        tkMessageBox.showerror("2D Peak Fitting","\n\nSave Fitting Project first!\n\n")
        return RETURN_ERROR



    #--- Prepare fitting and run it ---
    

    load_command="""      
        load '%s.fitpar.txt'
        load '%s.equations.txt'
        """ % (group_name,group_name)

    fit_command="""
      FIT_MAXITER=%d
      set fit errorvariables
      fit Y(x,y) '%s'  using 1:2:3:(1) via '%s'
      update '%s'
      """ % (max_iterations,datafile,parfile,parfile)


    finish="\nexit\n"
     
    gnuplot_command=load_command + fit_command

    print gnuplot_command

    iterations=0
    while 1:
      iterations+=max_iterations
      #-- run gnuplot fitting --
      self.gnuplot=popen2.Popen4(GNUPLOT_CALL,0)
      self.gnuplot.tochild.write(gnuplot_command + finish)
      gnuplot_output=self.gnuplot.fromchild.readlines() #-- can only be executed after gnuplot exits
      self.gnuplot=None


      #--- Update iterations count in the window --
      self.refresh_info("Iterations: %d" % iterations)
      if mylib.Find_String_Num(gnuplot_output,'converged.')!=-1:
        break
      
    print "--- Gnuplot messages ----"
    for line in gnuplot_output:
        print line
    print "--- End of Gnuplot messages ----"

    return RETURN_OK
    


  #---- menu: Fit
  def fit(self):
      'run fitting'
      #self.save_fitting_project()

      if self.fitting()==RETURN_ERROR: return
      
      self.refresh_info("Fit finished.")
      #self.show_fit_results()
 
     



  #---- menu: Selecting spectra and assigning time parameter/scale factor 
  #
  def setup_spectrum_list(self):
    #-- a resulting spectrum list gets assigned to self.spectrum_times[]

    setup_dialog = sputil.the_dialog(relax_setup_dialog, self.session)
    settings = {'spectrum-times': self.spectrum_times}
    setup_dialog.set_parent_dialog(self, settings, self.set_params)
    setup_dialog.show_window(1)

##---- The stuff below was written by Tom Goddard for his relax.py. Thanks, Tom!
  def set_params(self, settings):
    'actually assign a new spectrum list'
    self.spectrum_times = settings['spectrum-times'] 

#---------------------------------------------------------------------------------------
class relax_setup_dialog(tkutil.Settings_Dialog):

  def __init__(self, session):

    tkutil.Settings_Dialog.__init__(self, session.tk, 'Relaxation Spectra')

    headings = ('Spectrum  ', 'Time parameter', 'Scale factor')
    st = sputil.spectrum_table(session, self.top, headings,
                               self.add_spectrum, self.remove_spectrum)
    st.frame.pack(side = 'top', anchor = 'w')
    self.spectrum_widgets = {}

    spectra = session.project.spectrum_list()
    def time_param(s, self=self):
      return pyutil.string_to_float(self.default_time_parameter(s), 0)
    spectra = pyutil.sort_by_function_value(spectra, time_param)
    for spectrum in spectra:
      st.add_spectrum(spectrum)

   
    br = ek_tkutil.button_row(self.top,
                           ('Ok', self.ok_cb),
                           ('Apply', self.apply_cb),                           
                           ('Close', self.close_cb),
                           #('Help', sputil.help_cb(session, 'RelaxFit')),
			   )
    br.frame.pack(side = 'top', anchor = 'w')

  # ---------------------------------------------------------------------------
  #
  def add_spectrum(self, spectrum, table, row):

    onoff = self.default_onoff(spectrum)

    b = tkutil.checkbutton(table.frame, spectrum.name, onoff)
    b.button.grid(row = row, column = 0, sticky = 'nw')

    ttext = self.default_time_parameter(spectrum)
    e = tkutil.entry_field(table.frame, '', ttext, trailer='s')
    e.frame.grid(row = row, column = 1, sticky = 'nw')

    ftext=spectrum.saved_value(SCALE_FACTOR)
    if ftext==None: ftext=1
    f = tkutil.entry_field(table.frame, 'Intensity *', ftext, width=4)
    f.frame.grid(row = row, column = 2, sticky = 'nw')

    self.spectrum_widgets[spectrum] = (b, e, f)

  # ---------------------------------------------------------------------------
  #
  def remove_spectrum(self, spectrum, table):

    b, e = self.spectrum_widgets[spectrum]
    del self.spectrum_widgets[spectrum]
    b.button.destroy()
    e.frame.destroy()
    
  # ---------------------------------------------------------------------------
  #
  def default_onoff(self, spectrum):

    return spectrum.saved_value(INDEX_PARAMETER) != None
  
  # ---------------------------------------------------------------------------
  #
  def default_time_parameter(self, spectrum):

    #------ get time parameter --- 
    ttext = spectrum.saved_value(INDEX_PARAMETER)
    if ttext:
      return ttext
    
    #--- automatic time determination --
    match = re.search('[0-9]+', spectrum.name)
    if match:
      return match.group(0)
    
    return '0'

  # ---------------------------------------------------------------------------
  #
  def show_settings(self, settings):

    stimes = {}
    for s, t, f in settings['spectrum-times']:
      stimes[s] = (t, f)
      
    for s, (b, te, fe) in self.spectrum_widgets.items():
      b.variable.set(stimes.has_key(s))
      if stimes.has_key(s) :
	t, f = stimes[s]
        if pyutil.string_to_float(te.variable.get()) != t:
          te.variable.set('%.5g' % t)
        if pyutil.string_to_float(fe.variable.get()) != f:
          fe.variable.set('%.5g' % f)
      
  # ---------------------------------------------------------------------------
  #
  def get_settings(self):

    settings = {'spectrum-times': self.spectrum_times() }
    return settings
    
  # ---------------------------------------------------------------------------
  #
  def spectrum_times(self):

    stf = []
    for s, (b, te, fe) in self.spectrum_widgets.items():
      if b.variable.get():
        ttext = te.variable.get()
        t = pyutil.string_to_float(ttext)
        ftext = fe.variable.get()
        f = pyutil.string_to_float(ftext, 1.0)


        stf.append((s, t, f))
        
        s.save_value(INDEX_PARAMETER, ttext)
        s.save_value(SCALE_FACTOR,str(ftext))
    stf.sort(self.spectrum_time_order)
    return tuple(stf)

  # ---------------------------------------------------------------------------

  # ---------------------------------------------------------------------------
  #
  def spectrum_time_order(self, st1, st2):

    return cmp(st1[1], st2[1]) or cmp(st1[0].name, st2[0].name)







def run(session):

  sputil.the_dialog(peak_fitting_dialog,session).show_window(1)
