'v1. Data storage of residue specific values'
import os, sys, string, copy
import mylib
UserWarning="UserWarning"  # for compatibility with Python 1.5
YES,NO=1,0   # conditional flags
SILENT,VERBOSE=0,1  # work mode

KEY_COLUMN=0        # number of word in line which is a key
                    #  (the rest to the left is values list)

class Word_Dict:
    """
    # Dictionary based storage object: 'key':[list of words] (strings)
    # - may be created empty or read whitespaced data from a file
    #   (lines of words, may skip header of a file)
    # - key must be unique among other keys
    # - assign new value: a[existing_key]
    # - add new key-value: a.add(key, value_list)
    # - key position in line is specified by KEY_COLUMN constant
    #   (the rest to the left is values list)
    # - keeps lines in the original order of receiving
    # - keys may be exported for custom ordering
    # - new empty object may be created containing just keys
    # - all copying operations COPY data not REFERENCE!
    # - allows assignment of a list to a new key
    # - allows implementing of special ordering in subclasses
    # - allows checking of a entered or read key format in subclasses
     Usage :
            All count: 0,1,2...
            a=Word_Dict(fname)  - read file ( see optional arguments)
            
            b=a[key]  - get a copy of a key's value list
            a[key]=[1,3,4]  - assign a NEW list to an existing key
            del a[key] - remove key and its list
            a.del_keys(list_of_keys) - delete all entries listed and their lists
            a.add("D8",[1,33,44]) - add a new key and its value list
            
            a.set_word(key,1,13) - assign to a particular word in the value list
            a.append_word("A5",444) - add a word to a value list of the key 
            a.del_word("K7",0) -  delete particular word (list shrinks!)

            keylist=a.sorted_keys() - sorting rules specified by subclasses
            keylist=a.get_keys() - keys in original order of assigning or from the file

            for i in a:  - iterate over keys (in the original order)
                ...
           
            name=a.get_name()  - filename if read from file
            a.set_name("new_name") - if was changed
                
            b=a.make_empty_copy("new_name") - make empty instance with the same
                                                set of keys
            print a  - prints entries sorted by  a.sorted_keys()
            
            a.writeout("WordDict.test")    - write a file
            a.to_xmgr(x,y[,dy]) - returns an (x,y) list or (x,y,dy) list
                for plotting with xmgr module
             
            a.access_by_resnum(5)=[4]

            b=a.get_intersection(key_list) - creates an 'intersection'
                    object which keys are common for a and key_list
            
            b=a.to_xmgr(Xcol,Ycol,dYcol) - prepares data for xmgr -> (x,y[,dy]),
                if x="Key" then x=key;
                if x="Annn->NNN" then x=key[1:]'           
            --- with other lists ---------
     
    """
    # Attention, developer!!!:
    #----This is to be specified in subclasses:
    
    # creating empty object of current class type
    # (formally necessary for right copying
    # -> change the class name to subclass name
    def _my_class_empty(self): return Word_Dict()
    
    # subclass defined key format control 
    def _analyze_key(self,key): pass
    # subclass defined ordering
    def sorted_keys(self): return self.keys_orig_order[:] # a copy!
    
    def __init__(self, fname="", mode=SILENT, nonempty_header_lines=0, print_separator="\t"):
        self.data={}                # main dictionary
        self.keys_orig_order=[]     # list for preserving original order of keys
        self.iter_counter=0
        self.name="No name"
        self.print_separator=print_separator  # separator of words for output
        header_lines=nonempty_header_lines
        
        if fname:
            # aquire data
            temp_file=mylib.open_file(fname,"r")
            temp_storage=temp_file.readlines()
            temp_file.close()
            text=filter(lambda test_string : test_string.strip(), # if anything is in this line ?
                        temp_storage)
            
            self.name=fname
            # empty file?
            if not text:
                raise UserWarning, "\n\nFile is empty!\n-> %s" % fname
            # is there any data?
            if not len(text)>header_lines:
                raise UserWarning, (
                    "\n\nNo data beyond %d header lines in the file!\n-> %s"
                    % (header_lines, fname))
            # remove header
            del text[:header_lines]
            # load text into a dictionary
            if mode==VERBOSE:
                print "\nReading %s" % fname
            for s in text:
                s=s.split()
                key=s[KEY_COLUMN]
                # warn if repeated key:
                if key in self.data.keys(): 
                    raise UserWarning, (
                        "\n\nKey '%s' is found twice in the input file!\n-> %s"
                        % (key,fname))
                # OK
                else: self.add(key,s[KEY_COLUMN+1:])

            if mode==VERBOSE:
                print "%s key-value lines loaded\n" % len(self.data.keys())


#-------- print x, `x`----------------------------------------------------------------
    def __repr__(self):
        'nice viewable representation'
        text="%s\n------------------------------------\n" % self.name
        for key in self.sorted_keys():
            value_list=self.data[key][:]
            value_list.insert(0,key)
            
            text+="%s\n" % string.join(map(str,value_list),self.print_separator)
        return text
#-------------------------------------------------------------------------------------
    def writeout(self, fname, header="ON", verbose="on", mode='w'):
        'write a file, header="ON"/"OFF", verbose=1/0'
        outfile=open(fname,mode)
        if not outfile:
            raise IOError, "\n\nCannot write a file!\n-> %s" % fname

        if mode!='a' and header=="ON": text="%s\n--------------------\n" % self.name
        else: text=""
        
        for key in self.sorted_keys():
            value_list=self.data[key][:]
            value_list.insert(0,key)
            
            text+="%s\n" % string.join(map(str,value_list),self.print_separator)
        
        outfile.write(text)
        outfile.close()
        if verbose in (1,'on'): print "File: '%s' written.\n%s lines." % (fname,len(self))

#-------- if the key exists -----------------------------------------------------------
    def has_key(self,key):
        self._analyze_key(key)  # check format first
        if self.data.has_key(key): return YES
        else: return NO

    def _check_key(self,key):
        self._analyze_key(key)  # check format first
        if self.data.has_key(key): return
        else:
            raise  KeyError, "\n\nError! No key '%s' found in\n-> %s" % (key, self.name)
#---------------------------------------------------------------------------------------
    def __getitem__(self,key):
        '-> copy of a value list'
        self._check_key(key)
        return self.data[key][:]

    def __setitem__(self,key,value_list):
        'Replaces the full list for this key!'
        self._check_key(key)
        self.add(key,value_list)
    
    def add(self,key,value_list):
        self._analyze_key(key)
        
        # case if Evgueni assigned a single number instead of a list by mistake:
        try:
            # try something what single value cannot do
            len(value_list) 
        except TypeError:
            # single value found!!! 
            raise UserWarning, (
                "\n\nWrong assignment of a single value '%s' instead of a list to a key '%s' in\n%s"
                % (value_list,key,self.name))

        else: 
            # if it is a string?
            try:
                # try something what a string cannot do
                a=value_list[:0]
                a.insert(0,999)
            except AttributeError:
                # Error! Evgueni gave a string!!!
                raise UserWarning, (
                "\n\nWrong assignment of a single string '%s' instead of a list to a key '%s' in\n%s"
                % (value_list,key,self.name))
            else:
                # OK, this is a true list
                # Do true copy
                self.data[key]=copy.deepcopy(value_list)

        # keep right order
        if key not in self.keys_orig_order : self.keys_orig_order.append(key)
#--------------------------------------------------------------------------------------        
    def set_word(self,key,wordnum,value):
        'assign one word in the list of the key, Do not touch others'
        if len(self[key])-1<wordnum:
            raise UserWarning, (
                "\n\nError! Cannot assign to a %sth word (C-numbering, %sth in human numbering)! Key '%s' has only %s words in\n-> %s"
                % (wordnum,wordnum+1,key,len(self[key]), self.name))
        
        self.data[key][wordnum]=value
#------------------------------------------------------------------------------
    def del_word(self,key,wordnum):
        'delete a word from a value list of the key'
        if len(self[key])<=wordnum:
            raise UserWarning, (
                "\n\nError! Cannot delete %sth word! Key '%s' has only %s words in\n-> %s"
                % (wordnum,key,len(self[key]), self.name))
        del self.data[key][wordnum]
        
#------------------------------------------------------------------------------

    def append_word(self,key,value):
        'extend a value list for the key'
        self.data[key].append(value)
                    
        
#----------------------------------------------------------------------------------------
    def __delitem__(self,key):
        'remove key and its list'
        self._check_key(key)
        # remove
        del self.data[key]
        self.keys_orig_order.remove(key)
#-----------------------------------------------------------------------
    def __len__(self):
        return len(self.data)
#-----------------------------------------------------------------------
    def __contains__(self,item):
        'item in self'
        if item in self.data.keys(): return 1
        else: return 0
#-----------------------------------------------------------------------

    def del_keys(self, list_of_keys):
        'delete all entries listed'
        for key in list_of_keys:
            del self[key]

#-----------------------------------------------------------------------


    def get_name(self): 
        '-> instance name'
        return self.name
    
    def set_name(self, new_name):
        'setting a name to a dictionary'
        self.name=new_name
#-----------------------------------------------------------------------
    def get_keys(self):
        'a copy of keys to perform customary sorting'
        return self.keys_orig_order[:]
#----------------------------------------------------------------------
#--- Special use: convert keys, for instance, from Annn to NNN format...
    def convert_keys(self,scheme):
        'conversion of keys by specific scheme'
        if scheme=="ANNN->NNN":
            old_keys=copy.copy(self.get_keys())
            for key in old_keys:
                self.add(key[1:],self[key])
                del self[key]
        else:
            raise UserWarning, "Conversion scheme '%s' is not valid!" % scheme
        
    def convert_values_to_float(self):
        'just converts its content to floating point format'
        for key in self.get_keys():
            self[key]=map(float,self[key])

#-----------------------------------------------------------------------
    def make_empty_copy(self,new_name="not given"):
        'make a new empty object with the same set of keys'
        new_obj=self._my_class_empty()

        # make a distinctive name
        if new_name=="not given":
            new_obj.set_name(self.name+"-copy")
        else:
            new_obj.set_name(new_name)

        # copy keys
        for key in self.keys_orig_order:
            new_obj.add(key,[])

        return new_obj

#--------------------------------------------------------------------------

    def get_intersection(self, key_list):
        'returns an object containing intersection of key_list values and self.keys'
        new=copy.deepcopy(self)
        allkeys=self.get_keys()
        for key in allkeys:
            if key not in key_list: del new[key]
        return new

    
#----------------------------------------------------------------------------------------------

    def to_xmgr(self,Xcol,Ycol,dYcol="No Error"):
        'prepares data for xmgr -> (x,y[,dy]), if x="Key" then x=key; x="Annn->NNN" then x=key[1:]'
        out_list=[]
        # ---------------- X ------------------

        for key in self.sorted_keys():

            if Xcol=="Key":
                x=key
            else:
                if Xcol=="Annn->NNN":
                    x=key[1:]
                else:
                    x=self[key][Xcol]

            entry=[x,self[key][Ycol]]    
            if dYcol!="No Error": entry.append(self[key][dYcol])
            
            out_list.append(entry)
                
        return out_list            

#-------------- Constant offset correction --------------------------------------------------------------------------------
    def vert_offset_correction(self, baseline_keys_list, column):
        'constant vertical offset is taken from baseline_keys and subtracted from all points'
        try:
            self.convert_values_to_float()
        except ValueError:
            raise UserWarning, ("\n\nERROR!\nCannot do offset correction because the '%s' contents is not numeric!\nKey '%s' - '%s'\n\n"
                (self.get_name(),key,self[key]))

        #--- calculate constant offset ---
        N=len(baseline_keys_list)
        offset=0
        for key in baseline_keys_list:
                offset+=self[key][column]/N
        
        #-- subtract values ---
        for key in self.get_keys():
            self.set_word(key,column, self[key][column]-offset)
            
        return offset
        
        
#-------------------- End if Word_Dict---------------------------------------------------            














class Annn_Text(Word_Dict):
    """
    Residues in the first column, text words in others
    Additions:
    - check for the key to conform to Annn type
    - allows access residue entry by its number
            
            e=a.get_resnum(5) 
            a.access_by_resnum(5)=[4]
     - however, access by number is limited to assigning
         a complete list replacing previous
    Better, use
        Residue_Name=a.get_resname_by_num(N)
        a[Residue_Name] etc...

    print a  - prints sorted by number
    -
    """
    #---- To be specified in subclasses:
    
    # creating empty object of current class type
    # (formally necessary for right copying
    # -> change the class name to subclass name
    def _my_class_empty(self): return Annn_Text()

    #-- change this in subclasses !!! ----
    def _my_type(self, something):
        '-> converts anything to the type of field of current class'
        value=str(something)
        return value
    
    # subclass defined key format control
    def _split_key(self,key):
        ' Annn i.e. "L108" -> ("L","108")'
        # check if it is a string
        try: key.isalnum()
        except AttributeError:
            raise UserWarning, (
                "\n\nA number '%s'supplied instead of a residue name.\n\
Use .get_resnum(N) or .set_resnum(N) instead!\n-> %s" % (key, self.name))
        return key[:1], key[1:]
    
    def _analyze_key(self,key):
        'check key to be residue name and attached number'
        res,num=self._split_key(key)
        if not (res.isalpha() and num.isdigit()) :
            raise UserWarning, (
                "\n\nWrong key format: need 'Annn' but got '%s' instead in\n->  %s"
                % (key,self.name))

    # subclass defined ordering
    def sorted_keys(self):
        'returns residues ordered by number'
        new_order={}
        for key in self.data.keys():
            resnum=self._split_key(key)[1]
            new_order[int(resnum)]=key
        numbers=new_order.keys()
        numbers.sort()

        ordered_list=[]
        for i in numbers:
            ordered_list.append(new_order[i])

        return ordered_list
    
#--- accessing data by residue number ---------------------------------------------            
    def get_resname_by_num(self,num):
        'get a residue name by its number'
        for key in self.keys_orig_order:
            if int(self._split_key(key)[1])==num: return key
        else:
            raise UserWarning, (
                "\n\nError! There is no residue %s in\n-> %s" % (num, self.name))

    def get_resnum(self, num):
        'read by residue number -> copy of value_list'
        key= self.get_resname_by_num(num)
        return self[key][:]

    def set_resnum(self, num, list):
        'read by residue number, copy a list into a value list'
        key= self.get_resname_by_num(num)
        self[key]=list[:]
        
#-----------------------------------------------------------------------
    def del_keys(self, list_of_keys):
        'delete all entries listed'
        for key in list_of_keys:
            #--- case of just residue numbers ----
            if type(key)==type(1):
                res=self.get_resname_by_num(key)
            #--- normal Annn ---
            else :
                res=key
                
            del self[res]

#----------------------------------------------------------------------------------------------

    def to_xmgr(self,Xcol,Ycol,dYcol="No Error"):
        'prepares data for xmgr -> (x,y[,dy]), if x="ResNum" then x=residue number'
        out_list=[]
        # ---------------- X ------------------
                
        for res in self.sorted_keys():

            if Xcol=="ResNum":  x=int(self._split_key(res)[1])
            else:               x=self[res][Xcol]

            entry=[x,self[res][Ycol]]    
            if dYcol!="No Error": entry.append(self[res][dYcol])
            
            out_list.append(entry)
                
        return out_list            



#---------------------------------------------------------------------------------------------
    def get_ModelFree_results(self, Fitting_Parameters_object, *columns):
        'extracting ModelFree results for my residues'

        resnum_list=Fitting_Parameters_object.Residue_list.get_keys()
        myres_list=self.data.keys()      
        
        for res in myres_list:

            #-- check if it is present in Fitting_Parameters
            resnum=int(self._split_key(res)[1])
            if  resnum not in resnum_list:
                raise UserWarning, "\n\nResidue '%s' is not found in '%s'!!!" % (
                     res,Fitting_Parameters_object.Residue_list.get_name())


            #--- get values now ---
            for col in columns:     
                value=self._my_type(Fitting_Parameters_object.Residue_list[resnum][col])
                self.data[res].append(value)                           
    

#----------------------------------------------------------------------                  
        

class Annn_Num(Annn_Text):
    """the same as Annn_Text but all values are floating point
    
    - statistics:
        a.statistics(COLUMN_NUMBER) -> (mean,sigma)
    - trim_data(column_to_evaluate, trim_percent, mode)
    - max
    - min
    - to_xmgr

    Special interface to 'MFselect.py' object Fitting_Parameters()
    to be able to extract columns of results for residues of the
    current Annn_Num object
    Usage:
        a=b.make_empty_copy()
        a.get_ModelFree_results(Fitting_Parameters_object, *column_numbers)
    
    """

    #-- change this in subclasses !!! ----
    def _my_type(self, something):
        '-> converts anything to the type of field of current class'
        try:
            value=float(something)
        except ValueError:
            raise UserWarning, "\n\nAnnn_Text object '%s' needs a NUMBER for the value!!!\n\
Got '%s' instead!!!" % (self.get_name(), something)

        return value


    def __init__(self, fname="", mode=SILENT, header_lines=0, print_separator="\t"):
        Word_Dict.__init__(self,fname, mode, header_lines, print_separator)
        # convert all word in value lists to floating number
        for key in self.data.keys():
            for i in range(len(self[key])):
                try: 
                    self.data[key][i]=float(self.data[key][i])
                except ValueError:
                    raise ValueError, (
                        "\n\nError! Non-numeric value '%s' found for the key '%s' in\n-> %s"
                        % (self.data[key][i],key,self.name))
                
#----------------------------------------------------------------------------------------------
    def _my_class_empty(self): return Annn_Num()
                
#----------------------------------------------------------------------------------------------

    def statistics(self, column_number):
        '-> mean,sigma  on specified column'
        values=[]
        for res in self.get_keys():
            values.append(self[res][column_number])
        return mylib.statistics(values)

#--------------- Trimmed Statistics -----------------
#
# takes any complexity 'Annn_Num' object and  returns its trimmed subset

    def trim_data(self, value_column, trim_percent, mode=VERBOSE):
        'returns a trimmed subset'

        # extract the column to process
        trimmed_data=self.make_empty_copy("Trimmed %s%%" % trim_percent)
        for res in trimmed_data.get_keys():
            trimmed_data[res]=[self[res][value_column]]
            
        # columns
        VALUE=0
        DELTA=1

        mean,sigma=trimmed_data.statistics(VALUE)

        #----------- find initial delta's ----------------
        for res in trimmed_data.get_keys():
            trimmed_data.append_word(res,abs(mean-trimmed_data[res][VALUE]))

        #----------- how many residues to exclude? -----------------
        NumRes2Trim=int(len(trimmed_data)*int(trim_percent)/100.0)

        if mode==VERBOSE:
            print "\n--------- Trim %s %% ------------" % trim_percent
            print "\nRemove ",
        for i in range(NumRes2Trim):
            res=trimmed_data.max(DELTA)
            
            #--- check data type! ----
            try:
                "%s" % self[res][VALUE]
            except TypeError:
                raise UserWarning, "The object '%s' contains strings instead of floating point numbers!" % self.name

            if mode==VERBOSE:   print  res, self[res] , '\n'
            del trimmed_data[res]
            #-- recalculate delta's
            mean,sigma=trimmed_data.statistics(VALUE)
            for res in trimmed_data.get_keys():
                trimmed_data.set_word(res,DELTA,abs(mean-trimmed_data[res][VALUE]))
                
           
        if mode==VERBOSE:    print
 
        return self.get_intersection(trimmed_data.get_keys())
#-----------------------------------------------------------------------------



#----------------------------------------------------------------------------------------------
    def max(self,column):
        '(column number)-> residue of max value'
        keys=self.sorted_keys()
        max_key=keys[0]
        max=self[max_key][column]
       
        for res in keys[1:]:
            if self[res][column] > max :
                max= self[res][column]
                max_key=res
                
        return max_key
        
#----------------------------------------------------------------------------------------------
    def min(self,column):
        '(column number)-> residue of min value'
        keys=self.sorted_keys()
        min_key=keys[0]
        min=self[min_key][column]
        
        for res in keys[1:]:
            
            if self[res][column] < min :
                min= self[res][column]
                min_key=res
                
        return min_key
        

#-------------------------------------------------------------------------------------
    def writeout(self, fname, format="%s", header="ON", verbose="on", mode='w'):
        'write a file, header="ON"/"OFF", verbose=1/0'
       
        outfile=open(fname,mode)
        
        if not outfile:
            raise IOError, "\n\nCannot write to a file!\n-> %s" % fname
        
        if mode!='a' and header=="ON": text="%s\n--------------------\n" % self.name
        else: text=""
        
        for key in self.sorted_keys():
            value_list=self.data[key][:]
            line=key
            for value in value_list:
                line+=self.print_separator + (format % value)
                
            text+=line +"\n"
        
        outfile.write(text)
        if verbose in ("on",1): print "File: %s written.\n%s lines.\n" % (self.name,len(self))






class Sparky_lt_Peaklist(Annn_Num):
    """deals with Sparky peak lists, use double whitespace to recognize different column names
        - columns are fixed here: you cannot add or delete words


    To load LT v.3.1 lists:

    decoupled_data=residues.Sparky_lt_Peaklist(decoupled_file,nonempty_header_lines=1)

    """

    #-------------------------------------------------------------------------
    def __init__(self, fname, mode=SILENT,nonempty_header_lines=0,print_separator="\t"):
        
        #-- create an empty instance first ---------
        Annn_Num.__init__(self,"", mode, 0, print_separator)

        file=mylib.open_file(fname,'r')

        #-- skip header lines --
        count=1
        for count in range(nonempty_header_lines):
            file.readline()
            
        #-- extract column names ----
        header=file.readline()
        self.column_names=filter(None,map(string.strip, filter(None,string.split(header, '  '))))
        
        #-- check if 'Assignment' is there ---
        if self.column_names[0]!='Assignment':
            raise UserWarning, ("\a\a\a\n\nThe file '%s' must contain 'Assignment'\n in the first \
column! '%s' - found instead..." % (fname, self.column_names[0]))

        #--- load the file ----
        self.name=fname
        file_data=map(string.split,filter(lambda test_string : test_string.strip(), # is there anything in this line ?
                      file.readlines()))

        
        for line in file_data:
            resname=line[0][:-3]
            self._analyze_key(resname)  # - check the resname validity
            self.add(resname,map(float,line[1:]))
        
        self.column_names=self.column_names[1:]

    #-------------------------------------------------------------------------
    def col_names(self):
        'get and set the names'
        return self.column_names
    #-------------------------------------------------------------------------
    def get(self,residue,column_name):
        'read value for "resname" in the "column"'
        colnum=self.column_names.index(column_name)
        return self[residue][colnum]

    def set(self,residue,column_name, value):
        'enter new value'
        colnum=self.column_names.index(column_name)
        self.set_word(residue,colnum, value)


    #-------------------------------------------------------------------------
    def del_word(self,key,wordnum):
        'cannot be used: columns are fixed! Assign None instead to the value!'
        raise UserWarning, (
                "\n\nError! Cannot delete the word from the Sparky_lt_Peaklist object '%s'!\n\
                Assign 'None' to the value instead!" % self.name)
        del self.data[key][wordnum]
        
    #------------------------------------------------------------------------------

    def append_word(self,key,value):
        'cannot be used: columns are fixed! Assign None instead to the value!'
        raise UserWarning,  ("\a\a\a\n\n\nYou cannot append new columns to Sparky_lt_Peaklist '%s'! \n\
        Instead you should make an empty copy a=b.make_empty_copy('new') -> you get \n\
        normal Annn_Num object..."  % self.name)


    #





    
     


#---------------- checks one Word_Dict object contents against another -----------------
def check_keys_match(Annn1,Annn2):
    'raises a UserWarning if keys sets are different'
    # check length
    message=''
    if len(Annn1)!=len(Annn2):
        message= ("Datasets have different number of residues in!\n\
Check %s\n against %s!\n" % (Annn1.name,Annn2.name))
    # check match
    for res in Annn1.sorted_keys():
        if res not in Annn2.sorted_keys():
            raise UserWarning, ("\n\n\a\a\aError!\n"+message+
                        "Residue '%s' from\n %s\n is not found in \n%s!" % (res,Annn1.name,Annn2.name))
    for res in Annn2.sorted_keys():
        if res not in Annn1.sorted_keys():
            raise UserWarning, ("\n\n\a\a\aError!\n"+message+
                        "Residue '%s' from\n %s\n is not found in \n%s!" % (res,Annn2.name,Annn1.name))
#-----------------------------------------------------------------------------

def keys_analysis(Annn1,Annn2):
    '->Annn_common_keys_list, Annn1_rest_of_keys_list, Annn2_rest_of_keys_list'
    a1=Annn1.get_keys()
    a2=Annn2.get_keys()

    common=[]
    a1rest=[]
    a2rest=[]

    #-- find a1 keys not present in a2
    for res in a1:
        if res not in a2: a1rest.append(res)

    #-- find a2 keys not present in a1
    for res in a2:
        if res not in a1: a2rest.append(res)

    #-- find common part
    for res in a1:
        if res in a2: common.append(res)

    return common, a1rest, a2rest

#-----------------------------------------------------------------------------


                
if __name__=="__main__":
    print "Test...."
    a1=Annn_Text()
    a1.add("T3",[1.11111])
    a1.add("T56",[2.22222222])
    a1.add("T1",[3.33333333333333])
    a1.add("T6",[4.444444444444455555])

    print a1
    a1.writeout("residues.test.out", header=0,verbose=1)
    del a1["T3"]
    a1.writeout("residues.test.out", header=0,verbose=1, mode='a')
    #trimmed=a.trim_data(0,10)
    #print trimmed
#    fname="titr.20C_0.00eq.orig.sp.list"
#    fname="test.list"
#    a=Sparky_lt_Peaklist(fname,VERBOSE)

    
    print "Done..."

