#!/usr/bin/env python
doc="""****
General purpose system for running parallel processes and
communicating with them

- the consept: 'input','output' - are input and output
   of the process run in parallel
- Note! Process output is analysed in lines and if the
  process outputs a line without '\n' this lines is not
  read to the outp_list and looks like sitting in the stream
  buffer which is actually is not true. In order to protect
  yourself from a deadlock in such case (you are waiting for the line
  but _reader's readline() is waiting for a newline to deliver the
  new line into the outp_list) try to go through the communication
  session manually by running interactive_shell() method and analyzing
  the log file...
  
"""

import os, sys, time, popen2, string, thread
import mylib

DeadProcess="DeadProcess"    # exception for a dead process detection
#ClosedOutput="ClosedOutput"  # exception for detection of the closed output
STREAM_END_SIGNAL=''        # the last element in the closed output streams
INP_PREFIX, OUTP_PREFIX="inp<:", "outp>:"  # line prefixes for the log file
CLOSED, ALIVE = 0, 1
NO,YES=0,1
mylock=thread.allocate_lock()

class Run_Parallel:
    'the parent class for any parallel_running class'
    def __init__(self, process_command, communication_log_fname=None):

        #-- main storage for the process output --
        self.outp_list=[]
        #-- index of a last string --
        self.new_outpstrings_read=0
        #-- save start parameters ---
        self.log_fname=communication_log_fname
        self.proc_name=process_command
        #-- start the process --
        self.process=popen2.Popen4(process_command,0)
        self.output_state=ALIVE
        thread.start_new(self._reader,())

        #-- user input to control the process
        self.user_input=[]
#        self.interactive_mode=YES
    #-----------------------------------------------------
    def _reader(self):
        'continuously reads the process output stream'
        while self.output_state==ALIVE:
            line=self.process.fromchild.readline()
            self._append_outp(line)
            if line==STREAM_END_SIGNAL: self.output_state=CLOSED
            
    #------------------------------------------------------            
    def _write_log(self, prefix, string):
        'appends a line to a log file'
        if self.log_fname:
            mylib.open_file(self.log_fname, 'a').write(prefix+string)
        

    #------------------------------------------------------
    def _append_outp(self, string):
        'adds a new string to the outp_list'
        mylock.acquire()
        self.outp_list.append(string)
        self.new_outpstrings_read+=1
        self._write_log(OUTP_PREFIX,string)
        mylock.release()

    #-----------------------------------------------------
    def _flush_outp(self):
        'cleans outp_list, assume lock already acquired!!!'
        num_lines=len(self.outp_list)
        for i in range(num_lines):
            del self.outp_list[0]
        self.new_outpstrings_read=0

    #------------------------------------------------------
    def command(self, command_string):
        'sends the command_string to the process'
        # check ending of the command
        if command_string[-1:]!='\n': command_string+='\n'
        # check if alive
        if self.process.poll()==-1:
            self.process.tochild.write(command_string)
            self._write_log(INP_PREFIX, command_string)
        else:
            self._write_log("\nError! ","Process %s found finished when sending\
a command '%s' to it!\n" % (self.proc_name, command_string[:-1]))
    #--------------------------------------------------------
    def wait_for_keyword(self, acquired_mylock, keyword, sec_between_checks=1):
        'pauses till the "keyword" appears in the process output ! remember flushing outp_list\
        when waiting for the same word repeatedly.readlines!'
        #-- in order to search only newly read strings:
        current_pos=self.new_outpstrings_read
        #-- check if it is already there
        if self.outp_list:
             if mylib.Find_String_Num(self.outp_list, keyword)!=-1:
                #-- ! found
                return

        #-- now - wait for it
        while self.output_state==ALIVE:
            
            acquired_mylock.release()
            time.sleep(sec_between_checks)
            acquired_mylock.acquire()
            if mylib.Find_String_Num(self.outp_list, keyword)!=-1:
                    #-- ! found
                    return
 
            
        else:
            self._write_log("\nError!"," Process %s closed its output stream when waiting\
for '%s' !" % (self.proc_name, keyword))
            raise DeadProcess, ("\nError! Process %s closed its output stream when waiting\
for '%s' !" % (self.proc_name, keyword))

    #--------------------------------------------------------
    # Dummy function - to be replaced in subclasses...
    def get_info(self,keyword_to_wait_for):
        """extract information from process output and empties the output
        list olready analysed"""
        mylock.acquire()
        self.wait_for_keyword(mylock,keyword_to_wait_for)
        print self.outp_list    # dummy action
        self._flush_outp()
        mylock.release()
        return None
    #----------------------------------------------------------
    def input_prompt(self, prompt_text="standard"):
        'print prompt for user input'
        if prompt_text!="standard": print prompt_text,
        else:                       print "%s>>" % self.proc_name,

    #------------------------------------------------------------
    def _shell_input_reader(self, stop_shell_command):
        'command line for user control of the process'
        while 1:
            user_command=sys.stdin.readline()
            # stop working anyway
            if user_command==stop_shell_command:
                self.exit_requested=YES
                return
            mylock.acquire()
            self.user_input.append(user_command)
            mylock.release()            
    #----------------------------------------------------------

    def _flush_user_input(self):
        'cleans user_input list, assumes lock already acquired!!!'
        num_lines=len(self.user_input)
        for i in range(num_lines):
            del self.user_input[0]
    #-----------------------------------------------------


    
    def interactive_shell(self, exit_process_command, stop_shell_command="stop\n"):
        'interactive shell for tuning the interface to the process'
        
        print "Welcome to interactive shell to '%s'" % self.proc_name
        print '-'*40
        HELP_REQUEST="help?\n"
        LOGFILE_REQUEST="logfile?\n"

        text="""
        Your input is from this window. For output of your process
        check the log file (it is being appended as soon as output of
        your process comes.
        --------------------
        Available commands:
        
        %s - send exit request to a main thread
        %s - print this text
        %s - get log file name
        
        
        """ % (stop_shell_command[:-1], HELP_REQUEST[:-1], LOGFILE_REQUEST[:-1])
        print text
        print "Received so far:\n----------------"
        for line in self.outp_list:
            print line
            
        thread.start_new(self._shell_input_reader,(stop_shell_command,))
        self.input_prompt()
        while self.exit_requested==NO:
            # check if new commands were received
            mylock.acquire()
            if len(self.user_input)>0:
                #-- look for request to stop program --
                if stop_shell_command in self.user_input:
                    self.exit_requested==YES
                #-- help requested
                elif HELP_REQUEST in self.user_input:
                    print text
                    print "\n%s>>>" % self.proc_name,
                #-- log file name
                elif LOGFILE_REQUEST in self.user_input:
                    print os.path.join(os.getcwd(),self.log_fname)
                    print "\n%s>>>" % self.proc_name,
                #-- execute it ---
                else:
                    self.command(string.join(self.user_input,''))
                    self.input_prompt()
                            
            self._flush_user_input()
            mylock.release()
            
            time.sleep(1)
            
        else:
            self.command(exit_process_command)
            print "Request to stop the program was received.\n Process '%s' has been requested to exit..." % self.proc_name
#------------------------------------------------------------------------


class SFTP(Run_Parallel):
    'secure file transfer protocol'
    def __init__(self,host, password_wait_sec=15, logging=YES):
        #-- logging ?
        if logging==YES:
            self.log_fname=host+".sftp.log"
        else:
            self.log_fname=None

        print "----------- Connecting to %s -----------" % host
        print "%s seconds to enter your password!!!" % password_wait_sec
        Run_Parallel.__init__(self, "sftp "+host, self.log_fname)
        self.host=host
        self.proc_name="SFTP->"+host
        self.exit_requested=NO
        time.sleep(password_wait_sec)

 #------------------------------------------------------------------------


class SSH(Run_Parallel):
    'secure file transfer protocol'
    def __init__(self,host,password_wait_sec=15, logging=YES):
        #-- logging ?
        if logging==YES:
            self.log_fname=host+".ssh.log"
        else:
            self.log_fname=None

        print "----------- Connecting to %s -----------" % host
        print "%s seconds to enter your password!!!" % password_wait_sec
        Run_Parallel.__init__(self, "ssh "+host, self.log_fname)
        self.proc_name="SSH->"+host
        self.host=host
        self.exit_requested=NO
        time.sleep(password_wait_sec)
     
        
###-------------------------------------------------------------------------
##class Watch_Magnet(SSH):
##    'check spectrometer parameters periodically'
##    def __init__(self, sun_host, password_wait_sec=15, logging=YES):
##        SSH.__init__(sun_host, password_wait_sec, logging)
##        self.command('s\n')  # choosing a sun shell
##        
##    #--------------------------------------------------------
##    def get_info(self,keyword_to_wait_for):
##        """extract information from process output and empties the output
##        list olready analysed"""
##        mylock.acquire()
##        self.wait_for_keyword(mylock,keyword_to_wait_for)
##        print self.outp_list    # dummy action
##        self._flush_outp()
##        mylock.release()
##        return None
##  
##    #--------------------------------------------------------
##    def interactive_shell(self, exit_process_command, stop_shell_command="stop_watch\n"):
##        'interactive shell for tuning the interface to the process'
##        
##        print "Watching parameters of '%s'" % self.proc_name
##        print '-'*40
##        HELP_REQUEST="help?\n"
##        LOGFILE_REQUEST="logfile?\n"
##
##        text="""
##        --------------------
##        Available commands:
##        
##        %s - stop program 
##        %s - print this text
##        %s - get log file name
##        
##        
##        """ % (stop_shell_command[:-1], HELP_REQUEST[:-1], LOGFILE_REQUEST[:-1])
##        print text
####        print "Received so far:\n----------------"
####        for line in self.outp_list:
####            print line
##            
##        thread.start_new(self._shell_input_reader,(stop_shell_command,))
##        while self.exit_requested==NO:
##            # check if new commands were received
##            mylock.acquire()
##            if len(self.user_input)>0:
##                #-- look for request to stop program --
##                if stop_shell_command in self.user_input:
##                    self.exit_requested==YES
##                #-- help requested
##                elif HELP_REQUEST in self.user_input:
##                    print text
##                    print "\n%s>>>" % self.proc_name,
##                #-- log file name
##                elif LOGFILE_REQUEST in self.user_input:
##                    print os.path.join(os.getcwd(),self.log_fname)
##                    print "\n%s>>>" % self.proc_name,
##                #-- execute it ---
##                else:
##                    self.command(string.join(self.user_input,''))
##                            
##            self._flush_user_input()
##            mylock.release()
##            
##            time.sleep(1)
##            
##        else:
##            self.command(exit_process_command)
##            print "Request to stop the program was received.\n Process '%s' has been requested to exit..." % self.proc_name
###------------------------------------------------------------------------




##    def watch(self):
##        'main loop for watching the magnet'
##        while 

        





if __name__=="__main__":

    my=SSH("spock")
    my.interactive_shell(exit_process_command="exit\n")
        
