"""xmgr.py  v.1   A simple XMGR file builder

  There are to classes:
  Dataset - creates an object to accomodate data and set their presentation
            properties
            x=Dataset(array, legend)
               where array is either (x,y),(x2,y2)... or (x,y,dy),(x2,y2,dy2)...
            if you have x, y and dy arrays -
            use make_xy_matrix(x,y) and  make_xydy_matrix(x,y)
            functions to transpose your data


  Graph - creates a file for XMGR. Many datasets can be added.
            If you are lazy to set x,y axis - just click Autoscale
            in XMGR then
"""

def make_xy_matrix(x_list,y_list):
    '(x1,x2,..) and (y1,y2,...) -> ((x1,y1),(x2,y2),...)'
    if len(x_list)!=len(y_list):
        raise UserWarning, (
            "Size of X data (%d elements) is unequal to Y data (%d elements)!" %
            (len(x_list), len(y_list)))
    new_list=[]
    for i in range(len(x_list)):
        new_list.append([x_list[i],y_list[i]])
    return new_list

def make_xydy_matrix(x_list, y_list, dy_list):
    '(x1,x2..),(y1,y2..),(dy1,dy2..)->((x1,y1),(x2,y2),...)'
    if len(x_list)!=len(y_list) or len(y_list)!=len(dy_list):
        raise UserWarning, (
            "Unequal data sizes:\n\
X data - %d elements, Y data - %d elements), dY data - %d elements)!" %
            (len(x_list), len(y_list), len(dy_list)))
    new_list=[]
    for i in range(len(x_list)):
        new_list.append([x_list[i],y_list[i],dy_list[i]])
    return new_list




import os, sys, string

class Dataset:
    'keeps experimental data and their presentation parameters: (x,y),(x2,y2)... or (x,y,dy),(x2,y2,dy2)'        
    # constructor for matrix xy data
    def __init__(self,data_list, legend=""):
        # acquire data
        self.data=data_list
        self.legend=legend
        # default presentation parameters
        self.symbol=2
        self.size=0.5
        self.fill=1
        self.symb_color=1
        self.symb_linewidth=1
        self.symb_linestyle=1

        self.linestyle=0
        self.linewidth=1
        self.linecolor=1


#------------------------------
    def set_legend(self,new_legend):
        self.legend=new_legend
        
#-------- properties -----------
    # symbol
    def no_sym(self): self.symbol=0
    def dot(self): self.symbol=1
    def circle(self): self.symbol=2
    def square(self): self.symbol=3
    def diamond(self): self.symbol=4
    def triangle_up(self): self.symbol=5
    def star(self): self.symbol=11
    def bar(self): self.symbol=20
    # symbol color
    def black_sym(self): self.symb_color=1
    def red_sym(self): self.symb_color=2
    def green_sym(self): self.symb_color=3
    def blue_sym(self): self.symb_color=4
    # symbol type
    def filled_sym(self): self.fill=1
    def empty_sym(self): self.fill=0
    def empty_sym_outlinewidth(self,width): self.symb_linewidth=width
    # symbol size
    def symbol_size(self,factor):
        '1,2,3,... or fraction'
        self.size=self.size*factor
    
    # line
    def no_line(self): self.linestyle=0
    def solid_line(self): self.linestyle=1
    def dotted_line(self): self.linestyle=2
    def dash_line(self): self.linestyle=3
    def long_dash_line(self): self.linestyle=4
    def dot_dash_line(self): self.linestyle=5
    # line properties
    def line_width(self, factor):
        '1,2,3,...'
        self.linewidth=self.linewidth*factor
    # line color
    def black_line(self): self.linecolor=1
    def red_line(self): self.linecolor=2
    def green_line(self): self.linecolor=3
    def blue_line(self): self.linecolor=4

     
    #--- Exporting numerical data -------
    def export_data(self, fname=""):
        'output the numeric data as a text file'
        if not fname:   filename=self.legend
        else:           filename=fname

        outfile=open(filename,'w')
        for line in self.data:
            outline=string.join(map(str,line),' ')
            outfile.write(outline+'\n')
        outfile.close()
        
        
class Graph:
    'XMGR file builder'
#-----------------------------------------------------        
    def __init__(self, title="", subtitle=""):
        'creates XMGR graph object'
        # titles
        self.title=title
        self.subtitle=subtitle
        # coordinates
        self.xmin=0.0
        self.xmax=1.0
        self.ymin=0.0
        self.ymax=1.0
        self.xaxis_label="X"
        self.yaxis_label="Y"
        self.xmajticks=0.2
        self.xminticks=0.1
        self.ymajticks=0.2
        self.yminticks=0.1
        self.y_charsize=1
        self.x_charsize=1
        #error bar defaults
        self.err_type="BOTH"
        self.err_len=0.5
        self.err_lnwd=1
        self.err_lnst=1
        self.err_riser="on"

        # empty
        self.datasets=[]
        # legend position
        self.legend_x=0.19
        self.legend_y=0.8
#----------------------------------------------------
    def add_dataset(self, dataset):
        self.datasets.append(dataset)
        
    def remove_dataset(self, dataset):
        self.datasets.remove(dataset)

#--------------------------------------------------
    def header(self):
        'some mandatory stuff'
        return """# ACE/gr parameter file
#
@version 40102
@page layout free
@hardcopy device 1
@with g0
@g0 on\n"""
#------------------------------------------------------    
    def set_title(self,title="", subtitle=""):
        self.title=title
        self.subtitle=subtitle
#-------------------------------------------------------
    def _get_title(self):
        return ("""@    title "%s"
@    title font 4
@    title size 1.500000
@    title color 1
@    title linewidth 1
@    subtitle "%s"
@    subtitle font 4
@    subtitle size 1.000000
@    subtitle color 1
@    subtitle linewidth 1\n""" % (self.title, self.subtitle))

#--------------------------------------------------------------------
    def _get_datasets(self,number_list=None):
        'exports datasets presentation parameters and data'
        out_string=""
        if number_list==None: # print all
            if self.datasets : # there is something to print
                # export parameters
                dataset_number=0
                for set in self.datasets:
                    out_string=(out_string+
"""@    s%s symbol %s
@    s%s symbol size %s
@    s%s symbol fill %s
@    s%s symbol color %s
@    s%s symbol linewidth %s
@    s%s symbol linestyle %s
@    s%s linestyle %s
@    s%s linewidth %s 
@    s%s color %s\n""" % (dataset_number, set.symbol,
                      dataset_number, set.size,
                      dataset_number, set.fill,
                      dataset_number, set.symb_color,
                      dataset_number, set.symb_linewidth,
                      dataset_number, set.symb_linestyle,
                      dataset_number, set.linestyle,
                      dataset_number, set.linewidth,
                      dataset_number, set.linecolor))
                    
                    out_string=(out_string+
"""@    s%s errorbar type %s
@    s%s errorbar length %s
@    s%s errorbar linewidth %s
@    s%s errorbar linestyle %s
@    s%s errorbar riser %s
@    s%s errorbar riser linewidth 1
@    s%s errorbar riser linestyle 1
""" % 
                       (dataset_number, self.err_type,
                        dataset_number, self.err_len,
                        dataset_number, self.err_lnwd,
                        dataset_number, self.err_lnst,
                        dataset_number, self.err_riser,
                        dataset_number,dataset_number))

                    dataset_number=dataset_number+1
                # export data
                out_string=out_string+"""@WITH G0\n@G0 ON\n"""
                dataset_number=0
                for set in self.datasets:
                    # accept (x,y) and (x,y,dy)
                    if len(set.data[0])>2:
                        a="dy\n"
                    else:
                        a="\n"
                    out_string=(out_string+("@TARGET S%d\n@TYPE xy"
                                % dataset_number) +a)

                    for line in set.data:
                        short_string=""
                        for word in line:
                            short_string=short_string+str(word)+"\t"
                        out_string=out_string+short_string+"\n"
                    out_string=out_string+"&\n"
                    dataset_number=dataset_number+1

            else:
                raise UserWarning, "No datasets to export!"
            
            return out_string
        
#-----------------------------------------------------------------
    def _get_legend(self):
        out_string=("""@    legend on
@    legend loctype view
@    legend layout 0
@    legend vgap 2
@    legend hgap 1
@    legend length 4
@    legend box off
@    legend box fill off
@    legend box fill with color
@    legend box fill color 0
@    legend box fill pattern 1
@    legend box color 1
@    legend box linewidth 1
@    legend box linestyle 1
@    legend x1 %s
@    legend y1 %s
@    legend font 4
@    legend char size 0.5
@    legend linestyle 1
@    legend linewidth 1
@    legend color 1
""" % (self.legend_x,self.legend_y))
        count=0
        for set in self.datasets:
            out_string=out_string+('@    legend string %d "%s"\n'
                                   % (count, set.legend))
            count=count+1
        return out_string
#--------------------------------------------------
    def _get_coordinates(self):
        return ("""
@    world xmin %s
@    world xmax %s
@    world ymin %s
@    world ymax %s
@    stack world 0, 0, 0, 0 tick 0, 0, 0, 0\n""" % (
        self.xmin, self.xmax, self.ymin, self.ymax))
#--------------------------------------------------

       

#---------------------------------------------------------
    def set_xaxis(self,label, xmin, xmax, majticksnum=5, minticksnum=2, charsize=1):
        self.xaxis_label=label
        self.xmin=float(xmin)
        self.xmax=float(xmax)
        self.xmajticks=(self.xmax-self.xmin)/majticksnum
        self.xminticks=self.xmajticks/minticksnum
        self.x_charsize=charsize
        
    def _get_xaxis(self):
        return ("""@    xaxis  tick on
@    xaxis  tick major %s
@    xaxis  tick minor %s
@    xaxis  tick offsetx 0.000000
@    xaxis  tick offsety 0.000000
@    xaxis  label "%s"
@    xaxis  label layout para
@    xaxis  label place auto
@    xaxis  label char size 1.000000
@    xaxis  label font 4
@    xaxis  label color 1
@    xaxis  label linewidth 1
@    xaxis  ticklabel on
@    xaxis  ticklabel type auto
@    xaxis  ticklabel prec 5
@    xaxis  ticklabel format general
@    xaxis  ticklabel append ""
@    xaxis  ticklabel prepend ""
@    xaxis  ticklabel layout horizontal
@    xaxis  ticklabel place on ticks
@    xaxis  ticklabel skip 0
@    xaxis  ticklabel stagger 0
@    xaxis  ticklabel op bottom
@    xaxis  ticklabel sign normal
@    xaxis  ticklabel start type auto
@    xaxis  ticklabel start 0.000000
@    xaxis  ticklabel stop type auto
@    xaxis  ticklabel stop 0.000000
@    xaxis  ticklabel char size %s
@    xaxis  ticklabel font 4
@    xaxis  ticklabel color 1
@    xaxis  ticklabel linewidth 1
@    xaxis  tick major on
@    xaxis  tick minor on
@    xaxis  tick default 6
@    xaxis  tick in
@    xaxis  tick major color 1
@    xaxis  tick major linewidth 1
@    xaxis  tick major linestyle 1
@    xaxis  tick minor color 1
@    xaxis  tick minor linewidth 1
@    xaxis  tick minor linestyle 1
@    xaxis  tick log off
@    xaxis  tick size 1.000000
@    xaxis  tick minor size 0.500000
@    xaxis  bar off
@    xaxis  bar color 1
@    xaxis  bar linestyle 1
@    xaxis  bar linewidth 1
@    xaxis  tick major grid off
@    xaxis  tick minor grid off
@    xaxis  tick op both
@    xaxis  tick type auto
@    xaxis  tick spec 0\n"""
        % (self.xmajticks,self.xminticks, self.xaxis_label, self.x_charsize))
#---------------------------------------------------------

#---------------------------------------------------------
    def set_yaxis(self,label, ymin, ymax, majticksnum=5, minticksnum=2, charsize=1):
        self.yaxis_label=label
        self.ymin=float(ymin)
        self.ymax=float(ymax)
        self.ymajticks=(self.ymax-self.ymin)/majticksnum
        self.yminticks=self.ymajticks/minticksnum
        self.y_charsize=charsize
        
    def _get_yaxis(self):
          return ("""@    yaxis  tick on
@    yaxis  tick major %s
@    yaxis  tick minor %s
@    yaxis  tick offsetx 0.000000
@    yaxis  tick offsety 0.000000
@    yaxis  label "%s"
@    yaxis  label layout para
@    yaxis  label place auto
@    yaxis  label char size 1.000000
@    yaxis  label font 4
@    yaxis  label color 1
@    yaxis  label linewidth 1
@    yaxis  ticklabel on
@    yaxis  ticklabel type auto
@    yaxis  ticklabel prec 5
@    yaxis  ticklabel format general
@    yaxis  ticklabel append ""
@    yaxis  ticklabel prepend ""
@    yaxis  ticklabel layout horizontal
@    yaxis  ticklabel place on ticks
@    yaxis  ticklabel skip 0
@    yaxis  ticklabel stagger 0
@    yaxis  ticklabel op left
@    yaxis  ticklabel sign normal
@    yaxis  ticklabel start type auto
@    yaxis  ticklabel start 0.000000
@    yaxis  ticklabel stop type auto
@    yaxis  ticklabel stop 0.000000
@    yaxis  ticklabel char size %s
@    yaxis  ticklabel font 4
@    yaxis  ticklabel color 1
@    yaxis  ticklabel linewidth 1
@    yaxis  tick major on
@    yaxis  tick minor on
@    yaxis  tick default 6
@    yaxis  tick in
@    yaxis  tick major color 1
@    yaxis  tick major linewidth 1
@    yaxis  tick major linestyle 1
@    yaxis  tick minor color 1
@    yaxis  tick minor linewidth 1
@    yaxis  tick minor linestyle 1
@    yaxis  tick log off
@    yaxis  tick size 1
@    yaxis  tick minor size 0.500000
@    yaxis  bar off
@    yaxis  bar color 1
@    yaxis  bar linestyle 1
@    yaxis  bar linewidth 1
@    yaxis  tick major grid off
@    yaxis  tick minor grid off
@    yaxis  tick op both
@    yaxis  tick type auto
@    yaxis  tick spec 0\n"""
        % (self.ymajticks,self.yminticks, self.yaxis_label, self.y_charsize))

#---------------------------------------------------------

    def place_legend(self,x,y):
        'x and y as fractions: 0,0 is lower left corner'
        self.legend_x=x
        self.legend_y=y



#---------------------------------------------------------
    def set_errorbars(self, type, length, line_width=1, line_style=1, riser="on"):
        'set error bars'
        self.err_type=type
        self.err_len=length
        self.err_lnwd=line_width
        self.err_lnst=line_style
        self.err_riser=riser
        


#---------------------------------------------------------------------
        
    def writeout(self, out_fname, legend="on", verbose="on"):
        'creates the XMGR file'
        outfile=open(out_fname,"w")
        if not outfile: raise UserWarning, "Cannot create file '%s'!" % out_fname
        outfile.write(self.header())
        outfile.write(self._get_title())
        if legend=="on":  outfile.write(self._get_legend())
        outfile.write(self._get_coordinates())
        outfile.write(self._get_xaxis())
        outfile.write(self._get_yaxis())
        outfile.write(self._get_datasets())
        outfile.close()
        if verbose=='on': print "File '%s' is successfully written" % out_fname


#********************************************************************
# self testing
if __name__=="__main__":
        

    xdata1=[1,2,3]
    ydata1=[3,3,4]
    dy=[.3,.4,0.2]
    data2=[[1,1,0.2],[2,2,0.2],[3,4,0.2]]


    b1=Dataset(make_xydy_matrix(xdata1,ydata1,dy), "data1")
    b1.circle()
    b1.dotted_line()
    b1.blue_sym()


#    b2=Dataset(data2, "data2")
#    b2.circle()
#    b2.red_sym()
#    b2.filled_sym()
#    b2.solid_line()


    a=Graph("my_title","my_subtitle")
    a.add_dataset(b1)
#    a.add_dataset(b2)
    a.set_xaxis("X axis",0,4,1,0.25)
    a.set_yaxis("Y axis",-1,6,1,0.5)
    a.writeout("test_out.xmgr")





